/*
************************************************************************
**	O.S.		: Linux
**	FILE NAME	: arc13xx.h
**	Author		: Nick Cheng   
**	Description	: SCSI Device Driver for Areca Host Bus Adapter 
************************************************************************
** Copyright (C) 2002 - 2012, Areca Technology Corporation All rights reserved.
**
** Web site: www.areca.com.tw
** E-mail: support@areca.com.tw
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
** GNU General Public License for more details.
************************************************************************
** Redistribution and use in source and binary forms,with or without
** modification,are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice,this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice,this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
** 3. The name of the author may not be used to endorse or promote products
**    derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
** IMPLIED WARRANTIES,INCLUDING,BUT NOT LIMITED TO,THE IMPLIED WARRANTIES
** OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
** IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,INDIRECT,
** INCIDENTAL,SPECIAL,EXEMPLARY,OR CONSEQUENTIAL DAMAGES(INCLUDING,BUT
** NOT LIMITED TO,PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA,OR PROFITS; OR BUSINESS INTERRUPTION)HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY,WHETHER IN CONTRACT,STRICT LIABILITY,OR TORT
**(INCLUDING NEGLIGENCE OR OTHERWISE)ARISING IN ANY WAY OUT OF THE USE OF
** THIS SOFTWARE,EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
************************************************************************
** History
**
** REV#			DATE		NAME		DESCRIPTION
************************************************************************
*/
#include "osm.h"
/*
************************************************************************
** SCSI SPEC: Enable Vital Product Data Flag (EVPD)
************************************************************************
*/
#define CDB_INQUIRY_EVPD		1
#define STANDARD_INQUIRY_LENGTH		0x60
#define TEMPERATURE_LOG_PAGE_LENGTH	0x60
/*
************************************************************************
* Driver Capabilities *
************************************************************************
*/
#define ARCSAS_DRIVER_VERSION		"v1.1.16.161110"
#define ARCSAS_MAX_CMDXFER_LENGTH	(ARCSAS_OS_SG_ENTRIES * PAGE_SIZE)
#define ARCSAS_MAX_CMD_PERDEV		64
#define ARCSAS_DEFAULT_CMD_PERDEV	31
#define ARCSAS_MIN_CMD_PERDEV		1
#define ARCSAS_MAX_CMD_ATAPI		8		
#define ARCSAS_DISCOVERY_DPC		16
#define ARCSAS_VIRTUAL_DEVICE_ID	136

#define ARCSAS_CORE_SLOT_COUNT		512
#define ARCSAS_CORE_CCB_COUNT		ARCSAS_CORE_SLOT_COUNT	/*The original is 128, 512 is a try.*/
#define ARCSAS_OUTSTANDING_CMD		256
#define ARCSAS_MIN_OUTSTANDING_CMD	8

#define ARCSAS_OS_SG_ENTRIES		512 /*the original is 127*/
#define ARCSAS_CORE_SG_ENTRIES		ARCSAS_OS_SG_ENTRIES /*the original is 127*/
#define ARCSAS_PRD_BUFFER_COUNT		256

#define ARCSAS_CMD_LEN			16
#define ARCSAS_EVENT_ELEMENT		128  /* event record */
#define ARCSAS_SAS_SCRATCH_COUNT	32
#define ARCSAS_SMP_SCRATCH_COUNT	32
#define ARCSAS_ROUTE_INDEX		256
#define ARCSAS_CONFIG_ROUTES		136
#define ARCSAS_DISCOVER_REQUESTS	12
#define ARCSAS_CORE_NUMBER		2
#define ARCSAS_WIDEPORT_PHYS		4
#define ARCSAS_MAX_DEVICE_RECOVERED_MEDIUM_ERROR_COUNT	0x20
#define ARCSAS_MAX_DEVICE_RETRY_ERROR_COUNT		0x100
#define ID_NOT_MAPPED					0xff
#define	ARCSAS_CLI_TIMEOUT		10*1000 /* 5 seconds */
#define	ARCSAS_CLI_DONE			1
#define ARCSAS_MAX_SCAN_DELAY		10
#define ARCSAS_1300_PHY_NUM		4
#define ARCSAS_1320_PHY_NUM		4
#define ARCSAS_1300_REGSET_NUM		16
#define ARCSAS_1320_REGSET_NUM		64
#define ARCSAS_NUM_MSIX_VECTORS		16
/*
************************************************************************
************************************************************************
*/
#define ARCSAS_DISK_SUPPORTED				128
#define ARCSAS_EXPANDER_SUPPORTED			8				/* 8 */
#define ARCSAS_ENCLOSURE_SUPPORTED			ARCSAS_EXPANDER_SUPPORTED	/* 8 */
#define ARCSAS_PM_SUPPORTED				8
#define ARCSAS_LUN_SUPPORTED				127

#define ARCSAS_MIN_DISK_ID				0
#define ARCSAS_MAX_DISK_ID				ARCSAS_DISK_SUPPORTED - 1
#define ARCSAS_MIN_DEVICE_ID				ARCSAS_MIN_DISK_ID
#define ARCSAS_MAX_DEVICE_ID				ARCSAS_MIN_DEVICE_ID + ARCSAS_DISK_SUPPORTED + ARCSAS_ENCLOSURE_SUPPORTED - 1		/* 0...135 including enclosure, which equivelent to SCSI device ID*/
#define ARCSAS_MAX_DEVICE_NUMBER			ARCSAS_DISK_SUPPORTED + ARCSAS_ENCLOSURE_SUPPORTED					/* 0..136 "0..127 real","128..135 enclosure",136 virtual */

#define ARCSAS_MIN_ENCLOSURE_ID				ARCSAS_MAX_DISK_ID + 1						/* min enclosure scsi id */
#define ARCSAS_MAX_ENCLOSURE_ID				ARCSAS_MIN_ENCLOSURE_ID + ARCSAS_ENCLOSURE_SUPPORTED - 1	/* max enclosure scsi id */
#define ARCSAS_MIN_EXPANDER_ID				ARCSAS_MAX_ENCLOSURE_ID + 1					/* ARCSAS_MAX_DEVICE_ID + 1 = 136 */
#define ARCSAS_MAX_EXPANDER_ID				ARCSAS_MIN_EXPANDER_ID + ARCSAS_EXPANDER_SUPPORTED - 1 		/* ARCSAS_MIN_EXPANDER_ID + ARCSAS_EXPANDER_SUPPORTED - 1 = 143 */
#define ARCSAS_MIN_PM_ID				ARCSAS_MAX_EXPANDER_ID + 1					/* ARCSAS_MAX_EXPANDER_ID + 1 = 144 */
#define ARCSAS_MAX_PM_ID				ARCSAS_MIN_PM_ID + ARCSAS_PM_SUPPORTED - 1			/* ARCSAS_MIN_PM_ID + ARCSAS_PM_SUPPORTED - 1 = 151 */

#define ARCSAS_MAX_CORE_PHYSICAL_PORT_NUMBER		4
#define ARCSAS_MAX_DEVICE_PER_PM			8
#define ARCSAS_MAX_DEVICE_PER_EXPANDER			32
#define ARCSAS_MAX_CORE_SG_ENTRIES			64

#define ARC_SUCCESS		0
#define ARC_FAILURE		1
/*
************************************************************************
************************************************************************
*/
#ifndef PCI_VENDOR_ID_MARVELL
	#define PCI_VENDOR_ID_MARVELL		0x11ab	/* Marvell Vendor ID */
#endif

#ifndef PCI_VENDOR_ID_MARVELL_EXT
	#define PCI_VENDOR_ID_MARVELL_EXT	0x1B4B
#endif

#ifndef PCI_VENDOR_ID_ARECA
	#define PCI_VENDOR_ID_ARECA		0x17d3	/* Areca Vendor ID */
#endif

#define PCI_DEVICE_ID_ARECA_1320	0x1320 /* Device ID */
#define PCI_DEVICE_ID_ARECA_1300	0x1300 /* Device ID */

/*
************************************************************************
************************************************************************
*/
typedef  struct CommandControlBlock		CCB, *PCCB;
typedef  struct AdapterControlBlock		ACB, *PACB;
typedef  struct _EVPD_INQUIRYDATA		EVPD_INQUIRYDATA, *PEVPD_INQUIRYDATA;
typedef  struct _Tag_Stack			Tag_Stack, *PTag_Stack;
typedef  struct _ARCSAS_SGENTRY			ARCSAS_SGENTRY, *PARCSAS_SGENTRY;
typedef  struct _Core_Extension			Core_Extension,*PCore_Extension, *PCORE;
typedef  struct _Domain_Phy			Domain_Phy, *PDomain_Phy;
typedef  struct _Domain_Port			Domain_Port, *PDomain_Port;
typedef  struct _Domain_Device			Domain_Device, *PDomain_Device;
typedef  struct _Domain_Expander		Domain_Expander, *PDomain_Expander;
typedef  struct _Domain_PM			Domain_PM, *PDomain_PM;
typedef  struct _Discover_Extension		Discover_Extension, *PDiscover_Extension;
typedef  struct  _SMPBuffer_Context		SMPBuffer_Context, *PSMPBuffer_Context;
typedef  struct _ATA_TaskFile			ATA_TaskFile, *PATA_TaskFile;
typedef  struct _AdapterInfo			AdapterInfo, *PAdapterInfo;
typedef  struct _DISCOVERY_ELEMENT		DISCOVERY_ELEMENT, *PDISCOVERY_ELEMENT;
typedef  struct  _Assigned_Uncached_Memory	Assigned_Uncached_Memory, *PAssigned_Uncached_Memory;
typedef  struct _PassThrough_Config		PassThrough_Config, * PPassThrough_Config;
typedef  struct _SD_SmartStatus			SD_SmartStatus, *PSD_SmartStatus;
typedef  struct _HD_Info			HD_Info, *PHD_Info;
typedef  struct _SD_Config			SD_Config, *PSD_Config;
typedef  struct _PM_Info			PM_Info, *PPM_Info;
typedef  struct _Expander_Info			Expander_Info, *PExpander_Info;
typedef  struct _Adapter_Info			Adapter_Info, *PAdapter_Info;
typedef  struct _Version_Info			Version_Info, *PVersion_Info;
typedef  struct _Link_Entity			Link_Entity, *PLink_Entity;
typedef  struct _Link_Endpoint			Link_Endpoint, *PLink_Endpoint;
typedef  struct _BIOS_Info_Page			BIOS_Info_Page, *pBIOS_Info_Page;
typedef  struct _PHY_TUNING			PHY_TUNING, *PPHY_TUNING;
typedef  struct _ARCSAS_Sense_Data		ARCSAS_Sense_Data, *PARCSAS_Sense_Data;
typedef  struct _ARCSAS_SG_Table		ARCSAS_SG_Table, *PARCSAS_SG_Table;
typedef  struct _CORE_CONTEXT			CORE_CONTEXT, *PCore_Context;
typedef  struct _DONE_Q_ENTRY			COMPLETION_QUEUE_ENTRY, *PCOMPLETION_QUEUE_ENTRY;
typedef  struct _POST_Q_ENTRY			POST_Q_ENTRY, *PPOST_Q_ENTRY;
typedef  struct _ARCSAS_Command_Table		ARCSAS_Command_Table, *PARCSAS_Command_Table;
typedef  struct _STATUS_BUFFER			STATUS_BUFFER, *PSTATUS_BUFFER;
typedef  struct _ERR_INFO_RECORD		ERR_INFO_RECORD, *PERR_INFO_RECORD;
typedef  struct _OPEN_ADDRESS_FRAME		OPEN_ADDRESS_FRAME, *POPEN_ADDRESS_FRAME;
typedef  struct _ARCSAS_SATA_STP_Command_Table	ARCSAS_SATA_STP_Command_Table, *PARCSAS_SATA_STP_Command_Table;
typedef  struct _SMPRequest			ARCSAS_SMP_Command_Table, SMPRequest, *PSMPRequest;
typedef  struct _ARCSAS_SSP_Command_Table	ARCSAS_SSP_Command_Table, *PARCSAS_SSP_Command_Table;
typedef  struct _PROTECT_INFO_RECORD		PROTECT_INFO_RECORD, *PPROTECT_INFO_RECORD;
typedef  struct _SSP_RESPONSE_IU		SSP_RESPONSE_IU, *PSSP_RESPONSE_IU;
typedef  struct _SSP_XFERRDY_IU			SSP_XFERRDY_IU, *PSSP_XFERRDY_IU;
typedef  struct _SSP_TASK_IU			SSP_TASK_IU, *PSSP_TASK_IU;
typedef  struct _SSP_COMMAND_IU			SSP_COMMAND_IU, *PSSP_COMMAND_IU;
typedef  struct _SSP_Frame_Header		SSP_Frame_Header, *PSSP_Frame_Header;
typedef  struct _ARCSAS_Command_Header		ARCSAS_Command_Header, *PARCSAS_Command_Header;
typedef  struct _REG_CONFIG_PHY_STATUS		REG_CONFIG_PHY_STATUS, *PREG_CONFIG_PHY_STATUS;
typedef  struct _ATA_Identify_Data		ATA_Identify_Data, *PATA_Identify_Data;
typedef  struct _SATA_FIS_REG_H2D		SATA_FIS_REG_H2D, *PSATA_FIS_REG_H2D;
typedef  struct _Discover_Target		Discover_Target, *PDiscover_Target;
typedef  struct _Discover_Port_Info		Discover_PortInfo, *PDiscover_PortInfo;
typedef  struct _Config_Route_Info		Config_Route_Info, *PConfig_RouteInfo;
typedef  struct _Target_Dev_Map			Target_Dev_Map, *PTarget_Dev_Map;
typedef  struct _Target_Dev			Target_Dev, *PTarget_Dev;
typedef  struct _Target_Disc_Info		Target_Info, *PTarget_Info;
typedef  struct _SENSE_CODE_SET			SENSE_CODE_SET, *PSENSE_CODE_SET;
typedef  struct _PRDTableWalkCtx		PRDTableWalkCtx, *PPRDTableWalkCtx;
typedef  struct _SAS_Scratch_Buffer		SAS_Scratch_Buffer, *PSAS_Scratch_Buffer;
typedef  struct _SMP_Scratch_Buffer		SMP_Scratch_Buffer, *PSMP_Scratch_Buffer;
typedef  struct _PRD_Buffer			PRD_Buffer, *PPRD_Buffer;
typedef  struct SCSI_INQ_DATA			*PSCSI_INQ_DATA;
typedef  struct _Unmap_Block_Descriptor	Unmap_Block_Descriptor, *PUnmap_Block_Descriptor;
/*
************************************************************************
************************************************************************
*/
typedef struct	_sgd_3g_prd			sgd_3g_prd, *psgd_3g_prd;
typedef struct	_sgd_6g_prd			sgd_6g_prd, *psgd_6g_prd;
/*
************************************************************************
************************************************************************
*/
typedef union	_SAS_ADDR			SAS_ADDR, *PSAS_ADDR;
typedef union	_ARCSAS_U64			ARCSAS_U64, *PARCSAS_U64;
/*
************************************************************************
************************************************************************
*/
typedef void (*CCB_CBFunc) (void *InitiatorModuleExtension, PCCB pCCB);
typedef void (*Timer_CBFunc) (unsigned long);
typedef void (*SMP_CBFunc) (PDomain_Port pPort, PSMPRequest pSMPReq);
typedef void (*DISC_CBFunc) (PDiscover_Extension pDiscovery,PDomain_Port pPort);
typedef void (*DPCFUN) (void*);
/*
************************************************************************
************************************************************************
*/
#define ARCSAS_IOP_ERROR_ILLEGALPCI            		0x0001
#define ARCSAS_IOP_ERROR_VENDORID              		0x0002
#define ARCSAS_IOP_ERROR_DEVICEID              		0x0003
#define ARCSAS_IOP_ERROR_ILLEGALCCB           		0x0004
#define ARCSAS_IOP_ERROR_UNKNOW_CDBERR       		0x0005
#define ARCSAS_SYS_ERROR_MEMORY_ALLOCATE    		0x0006
#define ARCSAS_SYS_ERROR_MEMORY_CROSS4G     		0x0007
#define ARCSAS_SYS_ERROR_MEMORY_LACK        		0x0008
#define ARCSAS_SYS_ERROR_MEMORY_RANGE			0x0009
#define ARCSAS_SYS_ERROR_DEVICE_BASE			0x000A
#define ARCSAS_SYS_ERROR_PORT_VALIDATE			0x000B
#define ARCSAS_SYS_ERROR_NTLDR_DUMP			0x000C
#define ARCSAS_SYS_ERROR_PENDINGCMD_QUEUE		0x000D
#define ARCSAS_SYS_ERROR_SCSI_STATUS			0x000E
#define ARCSAS_IOP_WARN_STB_TIMEOUT			0x000F /*stop adapter rebulid timeout*/
#define ARCSAS_IOP_WARN_FAC_TIMEOUT			0x0010 /*flush adapter cache timeout*/
#define ARCSAS_IOP_WARN_SAB_TIMEOUT			0x0011 /*start adapter background rebulid timeout*/
#define ARCSAS_IOP_WARN_ABC_TIMEOUT			0x0012 /*abort all outstanding command timeout*/
#define ARCSAS_IOP_WARN_GFS_TIMEOUT			0x0013 /*get adapter firmware miscellaneous data timeout*/
#define ARCSAS_IOP_WARN_HPY_TIMEOUT			0x0014 /*set ccb high part physical address timeout*/
#define ARCSAS_IOP_WARN_PQW_TIMEOUT			0x0015 /*set ccb post Q window timeout*/
#define ARCSAS_IOP_WARN_SET_DRIVER_MODE_TIMEOUT		0x0016 /*set driver mode timeout*/
#define ARCSAS_IOP_WARN_SET_POST_WINDOW_TIMEOUT		0x0017 /*set driver mode timeout*/
#define ARCSAS_RAID_VOLUME_KICK_OUT			0x0018 /* */
#define ARCSAS_IOP_ENABLE_EOI_MODE_TIMEOUT		0x0019 /* */
/*
************************************************************************
************************************************************************
*/
#define ARCSAS_CCB_INSTANCE_NONE		0x03
#define ARCSAS_CCB_INSTANCE_DPC			0x02
#define ARCSAS_CCB_INSTANCE_DONE		0x01
#define ARCSAS_CCB_INSTANCE_POST		0x00
/*
************************************************************************
************************************************************************
*/
#define NVRAM_ENTRY_ID				0x5A
#define MAX_BOOTDEVICE_SUPPORTED		8
#define ARCSAS_MAX_DPC				16/* defer procedure call */
#define ARCSAS_MAX_EVENTS			16/* event record */
#define ARCSAS_MAX_MAP_ENTRY			16
#define ARCSAS_MAX_MODE_PAGE_LENGTH		192/*Odin Linux Spec*/
#define ARCSAS_SECTOR_SIZE			512
#define ARCSAS_IOCTL_BUF_LEN			(1024 * 1024)
#define ARCSAS_HBA_LOG_BUF_SIZE			2048
#define ARCSAS_BIT(val, x)			((val) << (x))
/*
************************************************************************
** two wire serial interface TWSI EEPROM 
** for retriving vendor spectfic PCI conficguration contents
************************************************************************
*/
#define TWSI_READ				ARCSAS_BIT(1, 4)
#define TWSI_WRITE				ARCSAS_BIT(1, 3)
#define IDENTIFY_CMD_PACKET_SET_MASK		0x1f00
#define IDENTIFY_CMD_PACKET_SET_TAPE		0x0100
#define IDENTIFY_GENERAL_CONFIG_ATAPI 		ARCSAS_BIT(1, 15)
#define ATA_REGISTER_DATA			0x08
#define ATA_REGISTER_ERROR			0x09
#define ATA_REGISTER_FEATURES			0x09
#define ATA_REGISTER_SECTOR_COUNT		0x0A
#define ATA_REGISTER_LBA_LOW			0x0B
#define ATA_REGISTER_LBA_MID			0x0C
#define ATA_REGISTER_LBA_HIGH			0x0D
#define ATA_REGISTER_DEVICE			0x0E
#define ATA_REGISTER_STATUS			0x0F
#define ATA_REGISTER_COMMAND			0x0F
#define ATA_REGISTER_ALT_STATUS			0x16
#define ATA_REGISTER_DEVICE_CONTROL		0x16
#define HD_WRITECACHE_OFF			0
#define HD_WRITECACHE_ON			1

#define MAX_MODE_PAGE_LENGTH            66
#define MAX_MODE_LOG_LENGTH             404

/*
 * Simple doubly linked list implementation.
 *
 * Some of the internal functions ("__xxx") are useful when
 * manipulating whole lists rather than single entries, as
 * sometimes we already know the next/prev entries and we can
 * generate better code by using them directly rather than
 * using the generic single-entry routines.
 */
#define MAX_CDB_SIZE				16  //TBD
#define PCI_CONFIG_DATA_SIZE			256
/* CDB definitions */
#define API_CDB0_ADAPTER			0xF0
#define API_CDB0_LD				0xF1
#define API_CDB0_BLOCK				0xF2
#define API_CDB0_PERIPHERAL_DEVICE_COMMAND	0xF3
#define API_CDB0_EVENT				0xF4
#define API_CDB0_DBG				0xF5
#define API_CDB0_FLASH				0xF6
/* RAID controller returned status */
#define ARCSAS_GUI_OK				0x41 /* Operation successed */
#define ARCSAS_GUI_RAIDSET_NOT_NORMAL		0x42
#define ARCSAS_GUI_VOLUMESET_NOT_NORMAL		0x43
#define ARCSAS_GUI_NO_RAIDSET			0x44
#define ARCSAS_GUI_NO_VOLUMESET			0x45
#define ARCSAS_GUI_NO_PHYSICAL_DRIVE		0x46
#define ARCSAS_GUI_PARAMETER_ERROR		0x47
#define ARCSAS_GUI_UNSUPPORTED_COMMAND		0x48
#define ARCSAS_GUI_DISK_CONFIG_CHANGED		0x49
#define ARCSAS_GUI_INVALID_PASSWORD		0x4a
#define ARCSAS_GUI_NO_DISK_SPACE		0x4b
#define ARCSAS_GUI_CHECKSUM_ERROR		0x4c
#define ARCSAS_GUI_PASSWORD_REQUIRED		0x4d
#define ARCSAS_GUI_NO_MORE_DATA			0x4e /* Internal use only */
#define ARCSAS_GUI_INTERNAL_ERROR		0x4f
#define ARCSAS_GUI_FIRMWARE_CRC_ERROR		0x50
#define ARCSAS_GUI_FIRMWARE_INVALID		0x51
#define ARCSAS_GUI_FIRMWARE_UPDATE_FAILED	0x52
#define ARCSAS_GUI_FIRMWARE_NO_HDD		0x53
#define ARCSAS_GUI_END_OF_DATA			0x54
/* 
************************************************************************
**                          API
**              ARCSAS IOCTL control code
**          main commands CDB[0] CDB definitions 
**          ------------------------------------
**  typedef struct _SRB_IO_CONTROL {
**      unsigned long HeaderLength;
**      unsigned char Signature[8];
**      unsigned long Timeout;
**      unsigned long ControlCode;
**      unsigned long ReturnCode;
**      unsigned long Length;
**  } SRB_IO_CONTROL, *PSRB_IO_CONTROL;
************************************************************************
*/
typedef struct _SRB_IO_CONTROL
{
	u32  HeaderLength;//the size of ioctldatabuffer in struct _SRB_IO_CONTROL
	u8   Signature[8];
	u32  Timeout;
	u32  ControlCode;
	u32  ReturnCode;
	u32  Length;//the size of ioctldatabuffer in struct _SRB_IOCTL_FIELD 
} SRB_IO_CONTROL, *PSRB_IO_CONTROL; 
typedef struct _SRB_IOCTL_FIELD 
{
	SRB_IO_CONTROL	srbioctl; /* ioctl header */
	u8		ioctldatabuffer;/* areca packet data */
}*PSRB_IOCTL_FIELD;

#pragma pack(push,8)
typedef struct _RequestHeader
{
	u8	version;
	u8	requestType;
	u16	startingIndexOrId;
	u16	numRequested;
	u16	numReturned;
	u16	nextStartingIndex;
	u8	reserved1[6];
} RequestHeader, *PRequestHeader;
typedef struct _Info_Request
{
	RequestHeader	header;
	u8		data[1];
} Info_Request, *PInfo_Request;
#pragma pack(pop)
#define ARCSAS_IOCTL_REQUEST_BY_RANGE			1
#define ARCSAS_IOCTL_REQUEST_BY_ID			2
#define ARCSAS_IOCTL_NO_MORE_DATA			0xFFFF
/*DeviceType*/
#define ARECA_SAS					0x90000000
/*FunctionCode*/
#define FUNCTION_SAS					0x0801
#define FUNCTION_SAY_HELLO				0x0802
#define FUNCTION_RETURN_CODE_3F				0x0803
/* ARECA IO CONTROL CODE*/
#define ARCSAS_IOCTL_SAS				ARECA_SAS | FUNCTION_SAS
#define ARCSAS_IOCTL_SAY_HELLO				ARECA_SAS | FUNCTION_SAY_HELLO
#define ARCSAS_IOCTL_RETURN_CODE_3F			ARECA_SAS | FUNCTION_RETURN_CODE_3F
/* ARECA IO CONTROL ReturnCode */
#define ARCSAS_IOCTL_RETURNCODE_OK			0x01
#define ARCSAS_IOCTL_RETURNCODE_ERROR			0x06
#define ARCSAS_IOCTL_RETURNCODE_BUS_HANG_ON		0x88
#define ARCSAS_IOCTL_RETURNCODE_3F			0x3F
/*                      */
#define API_CDB0_ADAPTER_COMMAND			0xF0
/* 
*********Adapter CDB[1] ***************
*/
#define API_CDB1_ADAPTER_GETCOUNT			0
#define API_CDB1_ADAPTER_GETINFO			(API_CDB1_ADAPTER_GETCOUNT + 1)
#define API_CDB1_ADAPTER_GETCONFIG			(API_CDB1_ADAPTER_GETCOUNT + 2)
#define API_CDB1_ADAPTER_SETCONFIG			(API_CDB1_ADAPTER_GETCOUNT + 3)
#define API_CDB1_ADAPTER_POWER_STATE_CHANGE		(API_CDB1_ADAPTER_GETCOUNT + 4)
#define API_CDB1_ADAPTER_SET_PASSWORD			(API_CDB1_ADAPTER_GETCOUNT + 5)
#define API_CDB1_ADAPTER_MAX				(API_CDB1_ADAPTER_GETCOUNT + 6)
#define API_CDB0_LOGICAL_DISK_COMMAND			0xF1
/* 
*********Logical Disk CDB[1]*************
*/
#define API_CDB1_LOGICAL_DISK_CREATE			0
#define API_CDB1_LOGICAL_DISK_GETMAXSIZE		(API_CDB1_LOGICAL_DISK_CREATE + 1)
#define API_CDB1_LOGICAL_DISK_GETINFO			(API_CDB1_LOGICAL_DISK_CREATE + 2)
#define API_CDB1_LOGICAL_DISK_GETTARGETLDINFO		(API_CDB1_LOGICAL_DISK_CREATE + 3)
#define API_CDB1_LOGICAL_DISK_DELETE			(API_CDB1_LOGICAL_DISK_CREATE + 4)
#define API_CDB1_LOGICAL_DISK_GETSTATUS			(API_CDB1_LOGICAL_DISK_CREATE + 5)
#define API_CDB1_LOGICAL_DISK_GETCONFIG			(API_CDB1_LOGICAL_DISK_CREATE + 6)
#define API_CDB1_LOGICAL_DISK_SETCONFIG			(API_CDB1_LOGICAL_DISK_CREATE + 7)
#define API_CDB1_LOGICAL_DISK_STARTREBUILD		(API_CDB1_LOGICAL_DISK_CREATE + 8)
#define API_CDB1_LOGICAL_DISK_STARTCONSISTENCYCHECK	(API_CDB1_LOGICAL_DISK_CREATE + 9)
#define API_CDB1_LOGICAL_DISK_STARTINIT			(API_CDB1_LOGICAL_DISK_CREATE + 10)
#define API_CDB1_LOGICAL_DISK_STARTMIGRATION		(API_CDB1_LOGICAL_DISK_CREATE + 11)
#define API_CDB1_LOGICAL_DISK_BGACONTROL		(API_CDB1_LOGICAL_DISK_CREATE + 12)
#define API_CDB1_LOGICAL_DISK_WIPEMDD			(API_CDB1_LOGICAL_DISK_CREATE + 13)
#define API_CDB1_LOGICAL_DISK_GETSPARESTATUS		(API_CDB1_LOGICAL_DISK_CREATE + 14)
#define API_CDB1_LOGICAL_DISK_SETGLOBALSPARE		(API_CDB1_LOGICAL_DISK_CREATE + 15)
#define API_CDB1_LOGICAL_DISK_SETLDSPARE		(API_CDB1_LOGICAL_DISK_CREATE + 16)
#define API_CDB1_LOGICAL_DISK_REMOVESPARE		(API_CDB1_LOGICAL_DISK_CREATE + 17)
#define API_CDB1_LOGICAL_DISK_HD_SETSTATUS		(API_CDB1_LOGICAL_DISK_CREATE + 18)
#define API_CDB1_LOGICAL_DISK_SHUTDOWN			(API_CDB1_LOGICAL_DISK_CREATE + 19)
#define API_CDB1_LOGICAL_DISK_HD_FREE_SPACE_INFO	(API_CDB1_LOGICAL_DISK_CREATE + 20)
#define API_CDB1_LOGICAL_DISK_HD_GETMBRINFO		(API_CDB1_LOGICAL_DISK_CREATE + 21)
#define API_CDB1_LOGICAL_DISK_SIZEOF_MIGRATE_TARGET	(API_CDB1_LOGICAL_DISK_CREATE + 22)
#define API_CDB1_LOGICAL_DISK_TARGET_LUN_TYPE		(API_CDB1_LOGICAL_DISK_CREATE + 23)
#define API_CDB1_LOGICAL_DISK_HD_MPCHECK		(API_CDB1_LOGICAL_DISK_CREATE + 24)
#define API_CDB1_LOGICAL_DISK_HD_GETMPSTATUS		(API_CDB1_LOGICAL_DISK_CREATE + 25)
#define API_CDB1_LOGICAL_DISK_HD_GET_RCT_COUNT		(API_CDB1_LOGICAL_DISK_CREATE + 26)
#define API_CDB1_LOGICAL_DISK_HD_RCT_REPORT		(API_CDB1_LOGICAL_DISK_CREATE + 27)
#define API_CDB1_LOGICAL_DISK_HD_START_DATASCRUB	(API_CDB1_LOGICAL_DISK_CREATE + 28)	// temp
#define API_CDB1_LOGICAL_DISK_HD_BGACONTROL		(API_CDB1_LOGICAL_DISK_CREATE + 29)	// temp
#define API_CDB1_LOGICAL_DISK_HD_GETBGASTATUS		(API_CDB1_LOGICAL_DISK_CREATE + 30)	// temp
// Added reserved for future expansion so that MRU/CLI can be backward compatible with drier.
#define API_CDB1_LOGICAL_DISK_RESERVED1			(API_CDB1_LOGICAL_DISK_CREATE + 31)	
#define API_CDB1_LOGICAL_DISK_RESERVED2			(API_CDB1_LOGICAL_DISK_CREATE + 32)	
#define API_CDB1_LOGICAL_DISK_RESERVED3			(API_CDB1_LOGICAL_DISK_CREATE + 33)	
#define API_CDB1_LOGICAL_DISK_RESERVED4			(API_CDB1_LOGICAL_DISK_CREATE + 34)	
#define API_CDB1_LOGICAL_DISK_RESERVED5			(API_CDB1_LOGICAL_DISK_CREATE + 35)	
#define API_CDB1_LOGICAL_DISK_MAX			(API_CDB1_LOGICAL_DISK_CREATE + 36)

#define API_CDB0_PHYSICAL_DISK_COMMAND			0xF2
/* 
**********Block device CDB[1]***********
*/
#define API_CDB1_BLOCK_GETINFO					0
#define API_CDB1_BLOCK_HD_BLOCKIDS				(API_CDB1_BLOCK_GETINFO + 1)
#define API_CDB1_BLOCK_MAX					(API_CDB1_BLOCK_GETINFO + 2)
/* 
*********for peripheral device CDB[1], CDB[2]: "device entry index".. SD_ID or EXP_ID PM_ID
*/
#define API_CDB1_PERIPHERAL_DEVICE_GET_SCSI_DEVICE_INFO		0                                           		/*CDB[2]:0...127*/
#define API_CDB1_PERIPHERAL_DEVICE_GET_EXPANDER_INFO		1/*CDB[2]:0...7  */
#define API_CDB1_PERIPHERAL_DEVICE_GET_PM_INFO			2	/*CDB[2]:0...7  */
#define API_CDB1_PERIPHERAL_DEVICE_GET_DISK_CONFIG		3
#define API_CDB1_PERIPHERAL_DEVICE_SET_DISK_CONFIG		4
/* 
*********Sub command for API_CDB1_PERIPHERAL_DEVICE_SET_DISK_CONFIG 
*/
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_WRITE_CACHE_OFF	0
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_WRITE_CACHE_ON	1
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_SMART_OFF		2
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_SMART_ON		3
#define API_CDB4_PERIPHERAL_DEVICE_SMART_RETURN			4
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_SPEED_1_5G		5
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_SPEED_3G		6
#define API_CDB4_PERIPHERAL_DEVICE_SET_DISK_SPEED_6G		7
#define API_CDB1_PERIPHERAL_DEVICE_BSL_DUMP			5
#define API_CDB1_PERIPHERAL_DEVICE_GET_LUN_DEVICE_INFO		6	// not used
#define API_CDB1_PERIPHERAL_DEVICE_RESERVED1			7	// not used
#define API_CDB1_PERIPHERAL_DEVICE_GET_SMART_STATUS		8

#define API_CDB0_EVENT_COMMAND				0xF4
/* 
*********************for event********************** 
*/
#define API_CDB1_EVENT_GET_ALL_EVENT			0
#define API_CDB1_EVENT_GET_CURRENT_EVENT_INDEX		1
#define API_CDB1_EVENT_CLEAR_ALL_EVENT			2
#define API_CDB0_DEBUG_COMMAND				0xF5
/* 
*********************for DBG ***********************
*/
#define API_CDB1_DBG_PDWR				0
#define API_CDB1_DBG_MAP				(API_CDB1_DBG_PDWR + 1)
#define API_CDB1_DBG_MAX				(API_CDB1_DBG_PDWR + 2)
#define API_CDB0_FLASH_COMMAND				0xF6
/* 
*********************for BIOS FLASH ****************
*/
#define API_CDB1_GET_BIOS_FLASH_SIZE			0
#define API_CDB1_READ_BIOS_FLASH_OFFSET			1
#define API_CDB1_ERASE_BIOS_FLASH			2
#define API_CDB1_WRITE_BIOS_FLASH_OFFSET		3
#define API_CDB1_SYNC_FLASH_DEVICES_MAP_WITH_SLOT_NUM	4
#define API_CDB1_FLASH_MAX				5
/* for SDI(HP CSMI) */
#define API_CDB0_CSMI_CORE_COMMAND			0xF7
#define API_CDB0_CSMI_RAID_COMMAND			0xF8

#define API_CDB0_PASS_THRU_SCSI_COMMAND			0xFA
#define API_CDB1_SCSI_NON_DATA				0x00
#define API_CDB1_SCSI_DATA_IN				0x01 // goes with Read Long
#define API_CDB1_SCSI_DATA_OUT				0x02 // goes with Write Long

#define API_CDB0_PASS_THRU_ATA_COMMAND			0xFB
#define API_CDB1_ATA_NON_DATA				0x00
#define API_CDB1_ATA_DATA_IN				0x01
#define API_CDB1_ATA_DATA_OUT				0x02

#define ARCSAS_SES_DEVICE_STATE_PAGE_MASK		(ARCSAS_BIT(1, SES_PG_SUPPORTED_DIAGNOSTICS) + ARCSAS_BIT(1, SES_PG_CONFIGURATION) + ARCSAS_BIT(1, SES_PG_ENCLOSURE_STATUS) + ARCSAS_BIT(1, SES_PG_ELEMENT_DESCRIPTOR) + ARCSAS_BIT(1, SES_PG_ADDITIONAL_ELEMENT_STATUS))

#define MAX_RX_FIS_NUMBER_1320				64
#define MAX_RX_FIS_NUMBER_1300				32

#define MAX_SMP_RESP_SIZE				1016
#define MAX_SSP_RESP_SENSE_SIZE				sizeof(ARCSAS_Sense_Data)

#define MAX_HBA_CMD_3G_PRD_BUFFER_SIZE			sizeof(sgd_3g_prd) * (ARCSAS_OS_SG_ENTRIES + 4)
#define MAX_HBA_CMD_6G_PRD_BUFFER_SIZE			sizeof(sgd_6g_prd) * (ARCSAS_OS_SG_ENTRIES + 7)

#define RX_FIS_SIZE					256
#define UNASSOCIATED_FIS_POOL_SIZE_1300			0x400  /*1300:1024(1k)*/
#define UNASSOCIATED_FIS_POOL_SIZE_1320			0x800  /*1320:2048(2K)*/
#define MAX_RX_FIS_POOL_SIZE_1300			(RX_FIS_SIZE * MAX_RX_FIS_NUMBER_1300 + UNASSOCIATED_FIS_POOL_SIZE_1300)
#define MAX_RX_FIS_POOL_SIZE_1320			(RX_FIS_SIZE * MAX_RX_FIS_NUMBER_1320 + UNASSOCIATED_FIS_POOL_SIZE_1320)

#define RX_FIS_DMA_REG	0x00
#define RX_FIS_PIO_REG	0x20
#define RX_FIS_RES_REG	0x34
#define RX_FIS_D2H_REG	0x40
#define RX_FIS_SDB_REG	0x58
#define RX_FIS_UNK_REG	0x60

#define SATA_RECEIVED_PIO_FIS_1300(reg_set)		(UNASSOCIATED_FIS_POOL_SIZE_1300 + 0x100 * reg_set + RX_FIS_PIO_REG)
#define SATA_RECEIVED_D2H_FIS_1300(reg_set)		(UNASSOCIATED_FIS_POOL_SIZE_1300 + 0x100 * reg_set + RX_FIS_D2H_REG)
#define SATA_RECEIVED_PIO_FIS_1320(reg_set)		(UNASSOCIATED_FIS_POOL_SIZE_1320 + 0x100 * reg_set + RX_FIS_PIO_REG) /*1300 and 1320 have the same phy number, and fis offset*/
#define SATA_RECEIVED_D2H_FIS_1320(reg_set)		(UNASSOCIATED_FIS_POOL_SIZE_1320 + 0x100 * reg_set + RX_FIS_D2H_REG) /*1300 and 1320 have the same phy number, and fis offset*/

#define SAS_SCRATCH_BUFFER_SIZE				(4096+64)
#define SMP_SCRATCH_BUFFER_SIZE				sizeof(struct _SMPRequest)
#define SES_STATUS_CONTROL_BUFFER_SIZE			SAS_SCRATCH_BUFFER_SIZE

#define REQUEST_TIME_OUT				20
#define PORT_RESET_LEVEL				1
#define DEVICE_RESET_LEVEL				PORT_RESET_LEVEL + 1
#define DEVICE_DOWN_LEVEL				PORT_RESET_LEVEL + 2
#define MAX_TaskFile_Error_Allowed			2000

#define MAX_EXPANDER_PHY_CHANGE_ALLOWED			250
#define SMART_POLLING_REQUEST_CYCLE			10  /* in sec */
#define STOP_RUNNING					0xFF
#define RUNNING						0x00
/* 
************************************************************************
**  Peripheral Device Type definitions 
************************************************************************
*/
#define SCSI_DASD		0x00	   /* Direct-access Device		*/
#define SCSI_SEQACESS		0x01	   /* Sequential-access device	*/
#define SCSI_PRINTER		0x02	   /* Printer device			*/
#define SCSI_PROCESSOR		0x03	   /* Processor device		*/
#define SCSI_WRITEONCE		0x04	   /* Write-once device		*/
#define SCSI_CDROM		0x05	   /* CD-ROM device		*/
#define SCSI_SCANNER		0x06	   /* Scanner device		*/
#define SCSI_OPTICAL		0x07	   /* Optical memory device		*/
#define SCSI_MEDCHGR		0x08	   /* Medium changer device		*/
#define SCSI_COMM		0x09	   /* Communications device		*/
#define SCSI_NODEV		0x1F	   /* Unknown or no device type	*/
/*
* Format Unit Scsi command translation
*/
#define LBA_BLOCK_COUNT					256
/**
* Self-Test Code translation for Send Diagnostic Command
**/
#define DEFAULT_SELF_TEST				0X0
#define BACKGROUND_SHORT_SELF_TEST			0x1
#define BACKGROUND_EXTENDED_SELF_TEST			0x2
#define ABORT_BACKGROUND_SELF_TEST			0x4
#define FOREGROUND_SHORT_SELF_TEST			0x5
#define FOREGROUND_EXTENDED_SELF_TEST			0x6
/* assume the maximum number of phys in an expander device is 128 */
#define MAXIMUM_EXPANDER_PHYS 				128
/* assume the maximum number of indexes per phy is 128 */
#define MAXIMUM_EXPANDER_INDEXES 			128
/* limit to 8 initiators for this example */
#define MAXIMUM_INITIATORS 				8
/* defines for SMP frame types */
#define SMP_REQUEST_FRAME 				0x40
#define SMP_RESPONSE_FRAME 				0x41
/* defines for SMP request functions */
#define REPORT_GENERAL 					0x00
#define REPORT_MANUFACTURER_INFORMATION 		0x01
#define READ_GPIO_REGISTER				0x02
#define REPORT_SELF_CONFIGURATION_STATUS		0x03
#define DISCOVER 					0x10
#define REPORT_PHY_ERROR_LOG 				0x11
#define REPORT_PHY_SATA 				0x12
#define REPORT_ROUTE_INFORMATION 			0x13
#define REPORT_PHY_EVENT_INFORMATION			0x14
#define REPORT_PHY_BROADCAST_COUNTS			0x15
#define DISCOVER_LIST					0x16
#define REPORT_EXPANDER_ROUTE_TABLE			0x17
#define CONFIGURE_GENERAL				0x80
#define ENABLE_DISABLE_ZONING				0x81
#define WRITE_GPIO_REGISTER 				0x82
#define ZONED_BROADCAST					0x85
#define CONFIGURE_ROUTE_INFORMATION 			0x90
#define PHY_CONTROL 					0x91
#define PHY_TEST 					0x92
#define CONFIGURE_PHY_EVENT_INFORMATION			0x93
/* defines for open responses, arbitrary values, not defined in the spec */
#define OPEN_ACCEPT					0
#define OPEN_REJECT_BAD_DESTINATION			1
#define OPEN_REJECT_RATE_NOT_SUPPORTED			2
#define OPEN_REJECT_NO_DESTINATION			3
#define OPEN_REJECT_PATHWAY_BLOCKED			4
#define OPEN_REJECT_PROTOCOL_NOT_SUPPORTED		5
#define OPEN_REJECT_RESERVE_ABANDON			6
#define OPEN_REJECT_RESERVE_CONTINUE			7
#define OPEN_REJECT_RESERVE_INITIALIZE			8
#define OPEN_REJECT_RESERVE_STOP			9
#define OPEN_REJECT_RETRY				10
#define OPEN_REJECT_STP_RESOURCES_BUSY			11
#define OPEN_REJECT_WRONG_DESTINATION			12
#define OPEN_REJECT_WAITING_ON_BREAK			13

/*Request Structure.*/
#define SENSE_INFO_BUFFER_SIZE				32

#define CC_CSMI_SAS_GET_DRIVER_INFO			1
#define CC_CSMI_SAS_GET_CNTLR_CONFIG			2
#define CC_CSMI_SAS_GET_CNTLR_STATUS			3
#define CC_CSMI_SAS_FIRMWARE_DOWNLOAD			4

// Control Codes requiring CSMI_RAID_SIGNATURE

#define CC_CSMI_SAS_GET_RAID_INFO			10
#define CC_CSMI_SAS_GET_RAID_CONFIG			11

// Control Codes requiring CSMI_SAS_SIGNATURE

#define CC_CSMI_SAS_GET_PHY_INFO			20
#define CC_CSMI_SAS_SET_PHY_INFO			21
#define CC_CSMI_SAS_GET_LINK_ERRORS			22
#define CC_CSMI_SAS_SMP_PASSTHRU			23
#define CC_CSMI_SAS_SSP_PASSTHRU			24
#define CC_CSMI_SAS_STP_PASSTHRU			25
#define CC_CSMI_SAS_GET_SATA_SIGNATURE			26
#define CC_CSMI_SAS_GET_SCSI_ADDRESS			27
#define CC_CSMI_SAS_GET_DEVICE_ADDRESS			28
#define CC_CSMI_SAS_TASK_MANAGEMENT			29
#define CC_CSMI_SAS_GET_CONNECTOR_INFO			30
#define CC_CSMI_SAS_GET_LOCATION			31

// Control Codes requiring CSMI_PHY_SIGNATURE

#define CC_CSMI_SAS_PHY_CONTROL				60

#define IOCTL_HEADER					SRB_IO_CONTROL
#define PIOCTL_HEADER					PSRB_IO_CONTROL

/* * * * * * * * * * Class Independent IOCTL Constants * * * * * * * * * */

// Return codes for all IOCTL's regardless of class
// (IoctlHeader.ReturnCode)

#define CSMI_SAS_STATUS_SUCCESS				0
#define CSMI_SAS_STATUS_FAILED				1
#define CSMI_SAS_STATUS_BAD_CNTL_CODE			2
#define CSMI_SAS_STATUS_INVALID_PARAMETER		3
#define CSMI_SAS_STATUS_WRITE_ATTEMPTED			4

// Signature value
// (IoctlHeader.Signature)

#define CSMI_ALL_SIGNATURE				"CSMIALL"

// Timeout value default of 60 seconds
// (IoctlHeader.Timeout)

#define CSMI_ALL_TIMEOUT				60

//  Direction values for data flow on this IOCTL
// (IoctlHeader.Direction, Linux only)
#define CSMI_SAS_DATA_READ				0
#define CSMI_SAS_DATA_WRITE				1

// I/O Bus Types
// ISA and EISA bus types are not supported
// (bIoBusType)

#define CSMI_SAS_BUS_TYPE_PCI				3
#define CSMI_SAS_BUS_TYPE_PCMCIA			4

// Offline Status Reason
// (uOfflineReason)

#define CSMI_SAS_OFFLINE_REASON_NO_REASON		0
#define CSMI_SAS_OFFLINE_REASON_INITIALIZING		1
#define CSMI_SAS_OFFLINE_REASON_BACKSIDE_BUS_DEGRADED	2
#define CSMI_SAS_OFFLINE_REASON_BACKSIDE_BUS_FAILURE	3

// Controller Class
// (bControllerClass)

#define CSMI_SAS_CNTLR_CLASS_HBA			5

// Controller Flag bits
// (uControllerFlags)

#define CSMI_SAS_CNTLR_SAS_HBA		0x00000001
#define CSMI_SAS_CNTLR_SAS_RAID		0x00000002
#define CSMI_SAS_CNTLR_SATA_HBA		0x00000004
#define CSMI_SAS_CNTLR_SATA_RAID	0x00000008

// for firmware download
#define CSMI_SAS_CNTLR_FWD_SUPPORT	0x00010000
#define CSMI_SAS_CNTLR_FWD_ONLINE	0x00020000
#define CSMI_SAS_CNTLR_FWD_SRESET	0x00040000
#define CSMI_SAS_CNTLR_FWD_HRESET	0x00080000
#define CSMI_SAS_CNTLR_FWD_RROM		0x00100000

// Download Flag bits
// (uDownloadFlags)
#define CSMI_SAS_FWD_VALIDATE		0x00000001
#define CSMI_SAS_FWD_SOFT_RESET		0x00000002
#define CSMI_SAS_FWD_HARD_RESET		0x00000004

// Firmware Download Status
// (usStatus)
#define CSMI_SAS_FWD_SUCCESS		0
#define CSMI_SAS_FWD_FAILED		1
#define CSMI_SAS_FWD_USING_RROM		2
#define CSMI_SAS_FWD_REJECT		3
#define CSMI_SAS_FWD_DOWNREV		4

// Firmware Download Severity
// (usSeverity>
#define CSMI_SAS_FWD_INFORMATION	0
#define CSMI_SAS_FWD_WARNING		1
#define CSMI_SAS_FWD_ERROR		2
#define CSMI_SAS_FWD_FATAL		3

/* * * * * * * * * * SAS RAID Class IOCTL Constants  * * * * * * * * */

// Return codes for the RAID IOCTL's regardless of class
// (IoctlHeader.ControlCode)

#define CSMI_SAS_RAID_SET_OUT_OF_RANGE	1000

// Signature value
// (IoctlHeader.Signature)

#define CSMI_RAID_SIGNATURE		"CSMIARY"

// Timeout value default of 60 seconds
// (IoctlHeader.Timeout)

#define CSMI_RAID_TIMEOUT		60

// RAID Types
// (bRaidType)
#define CSMI_SAS_RAID_TYPE_NONE		0
#define CSMI_SAS_RAID_TYPE_0		1
#define CSMI_SAS_RAID_TYPE_1		2
#define CSMI_SAS_RAID_TYPE_10		3
#define CSMI_SAS_RAID_TYPE_5		4
#define CSMI_SAS_RAID_TYPE_15		5
#define CSMI_SAS_RAID_TYPE_OTHER	255

// RAID Status
// (bStatus)
#define CSMI_SAS_RAID_SET_STATUS_OK		0
#define CSMI_SAS_RAID_SET_STATUS_DEGRADED	1
#define CSMI_SAS_RAID_SET_STATUS_REBUILDING	2
#define CSMI_SAS_RAID_SET_STATUS_FAILED		3

// RAID Drive Status
// (bDriveStatus)
#define CSMI_SAS_DRIVE_STATUS_OK		0
#define CSMI_SAS_DRIVE_STATUS_REBUILDING	1
#define CSMI_SAS_DRIVE_STATUS_FAILED		2
#define CSMI_SAS_DRIVE_STATUS_DEGRADED		3

// RAID Drive Usage
// (bDriveUsage)
#define CSMI_SAS_DRIVE_CONFIG_NOT_USED		0
#define CSMI_SAS_DRIVE_CONFIG_MEMBER		1
#define CSMI_SAS_DRIVE_CONFIG_SPARE		2

/* * * * * * * * * * SAS HBA Class IOCTL Constants * * * * * * * * * */

// Return codes for SAS IOCTL's
// (IoctlHeader.ReturnCode)

#define CSMI_SAS_PHY_INFO_CHANGED		CSMI_SAS_STATUS_SUCCESS
#define CSMI_SAS_PHY_INFO_NOT_CHANGEABLE	2000
#define CSMI_SAS_LINK_RATE_OUT_OF_RANGE		2001

#define CSMI_SAS_PHY_DOES_NOT_EXIST		2002
#define CSMI_SAS_PHY_DOES_NOT_MATCH_PORT	2003
#define CSMI_SAS_PHY_CANNOT_BE_SELECTED		2004
#define CSMI_SAS_SELECT_PHY_OR_PORT		2005
#define CSMI_SAS_PORT_DOES_NOT_EXIST		2006
#define CSMI_SAS_PORT_CANNOT_BE_SELECTED	2007
#define CSMI_SAS_CONNECTION_FAILED		2008

#define CSMI_SAS_NO_SATA_DEVICE			2009
#define CSMI_SAS_NO_SATA_SIGNATURE		2010
#define CSMI_SAS_SCSI_EMULATION			2011
#define CSMI_SAS_NOT_AN_END_DEVICE		2012
#define CSMI_SAS_NO_SCSI_ADDRESS		2013
#define CSMI_SAS_NO_DEVICE_ADDRESS		2014

// Signature value
// (IoctlHeader.Signature)

#define CSMI_SAS_SIGNATURE    "CSMISAS"

// Timeout value default of 60 seconds
// (IoctlHeader.Timeout)

#define CSMI_SAS_TIMEOUT			60

// Device types
// (bDeviceType)

#define CSMI_SAS_PHY_UNUSED			0x00
#define CSMI_SAS_NO_DEVICE_ATTACHED		0x00
#define CSMI_SAS_END_DEVICE			0x10
#define CSMI_SAS_EDGE_EXPANDER_DEVICE		0x20
#define CSMI_SAS_FANOUT_EXPANDER_DEVICE	0x30

// Protocol options
// (bInitiatorPortProtocol, bTargetPortProtocol)

#define CSMI_SAS_PROTOCOL_SATA			0x01
#define CSMI_SAS_PROTOCOL_SMP			0x02
#define CSMI_SAS_PROTOCOL_STP			0x04
#define CSMI_SAS_PROTOCOL_SSP			0x08

// Negotiated and hardware link rates
// (bNegotiatedLinkRate, bMinimumLinkRate, bMaximumLinkRate)

#define CSMI_SAS_LINK_RATE_UNKNOWN		0x00
#define CSMI_SAS_PHY_DISABLED			0x01
#define CSMI_SAS_LINK_RATE_FAILED		0x02
#define CSMI_SAS_SATA_SPINUP_HOLD		0x03
#define CSMI_SAS_SATA_PORT_SELECTOR		0x04
#define CSMI_SAS_LINK_RATE_1_5_GBPS		0x08
#define CSMI_SAS_LINK_RATE_3_0_GBPS		0x09
#define CSMI_SAS_LINK_VIRTUAL			0x10

// Discover state
// (bAutoDiscover)

#define CSMI_SAS_DISCOVER_NOT_SUPPORTED		0x00
#define CSMI_SAS_DISCOVER_NOT_STARTED		0x01
#define CSMI_SAS_DISCOVER_IN_PROGRESS		0x02
#define CSMI_SAS_DISCOVER_COMPLETE		0x03
#define CSMI_SAS_DISCOVER_ERROR			0x04

// Programmed link rates
// (bMinimumLinkRate, bMaximumLinkRate)
// (bProgrammedMinimumLinkRate, bProgrammedMaximumLinkRate)

#define CSMI_SAS_PROGRAMMED_LINK_RATE_UNCHANGED	0x00
#define CSMI_SAS_PROGRAMMED_LINK_RATE_1_5_GBPS	0x08
#define CSMI_SAS_PROGRAMMED_LINK_RATE_3_0_GBPS	0x09

// Link rate
// (bNegotiatedLinkRate in CSMI_SAS_SET_PHY_INFO)

#define CSMI_SAS_LINK_RATE_NEGOTIATE		0x00
#define CSMI_SAS_LINK_RATE_PHY_DISABLED		0x01

// Signal class
// (bSignalClass in CSMI_SAS_SET_PHY_INFO)

#define CSMI_SAS_SIGNAL_CLASS_UNKNOWN		0x00
#define CSMI_SAS_SIGNAL_CLASS_DIRECT		0x01
#define CSMI_SAS_SIGNAL_CLASS_SERVER		0x02
#define CSMI_SAS_SIGNAL_CLASS_ENCLOSURE		0x03

// Link error reset
// (bResetCounts)

#define CSMI_SAS_LINK_ERROR_DONT_RESET_COUNTS	0x00
#define CSMI_SAS_LINK_ERROR_RESET_COUNTS	0x01

// Phy identifier
// (bPhyIdentifier)

#define CSMI_SAS_USE_PORT_IDENTIFIER		0xFF

// Port identifier
// (bPortIdentifier)

#define CSMI_SAS_IGNORE_PORT			0xFF

// Programmed link rates
// (bConnectionRate)

#define CSMI_SAS_LINK_RATE_NEGOTIATED		0x00
// Connection status
// (bConnectionStatus)

#define CSMI_SAS_OPEN_ACCEPT				0
#define CSMI_SAS_OPEN_REJECT_BAD_DESTINATION		1
#define CSMI_SAS_OPEN_REJECT_RATE_NOT_SUPPORTED		2
#define CSMI_SAS_OPEN_REJECT_NO_DESTINATION		3
#define CSMI_SAS_OPEN_REJECT_PATHWAY_BLOCKED		4
#define CSMI_SAS_OPEN_REJECT_PROTOCOL_NOT_SUPPORTED	5
#define CSMI_SAS_OPEN_REJECT_RESERVE_ABANDON		6
#define CSMI_SAS_OPEN_REJECT_RESERVE_CONTINUE		7
#define CSMI_SAS_OPEN_REJECT_RESERVE_INITIALIZE		8
#define CSMI_SAS_OPEN_REJECT_RESERVE_STOP		9
#define CSMI_SAS_OPEN_REJECT_RETRY                    	10
#define CSMI_SAS_OPEN_REJECT_STP_RESOURCES_BUSY		11
#define CSMI_SAS_OPEN_REJECT_WRONG_DESTINATION		12

// SSP Status
// (bSSPStatus)

#define CSMI_SAS_SSP_STATUS_UNKNOWN		0x00
#define CSMI_SAS_SSP_STATUS_WAITING		0x01
#define CSMI_SAS_SSP_STATUS_COMPLETED		0x02
#define CSMI_SAS_SSP_STATUS_FATAL_ERROR		0x03
#define CSMI_SAS_SSP_STATUS_RETRY		0x04
#define CSMI_SAS_SSP_STATUS_NO_TAG		0x05

// SSP Flags
// (uFlags)

#define CSMI_SAS_SSP_READ				0x00000001
#define CSMI_SAS_SSP_WRITE				0x00000002
#define CSMI_SAS_SSP_UNSPECIFIED			0x00000004

#define CSMI_SAS_SSP_TASK_ATTRIBUTE_SIMPLE		0x00000000
#define CSMI_SAS_SSP_TASK_ATTRIBUTE_HEAD_OF_QUEUE	0x00000010
#define CSMI_SAS_SSP_TASK_ATTRIBUTE_ORDERED		0x00000020
#define CSMI_SAS_SSP_TASK_ATTRIBUTE_ACA			0x00000040

// SSP Data present
// (bDataPresent)

#define CSMI_SAS_SSP_NO_DATA_PRESENT		0x00
#define CSMI_SAS_SSP_RESPONSE_DATA_PRESENT	0x01
#define CSMI_SAS_SSP_SENSE_DATA_PRESENT		0x02

// STP Flags
// (uFlags)

#define CSMI_SAS_STP_READ		0x00000001
#define CSMI_SAS_STP_WRITE		0x00000002
#define CSMI_SAS_STP_UNSPECIFIED	0x00000004
#define CSMI_SAS_STP_PIO		0x00000010
#define CSMI_SAS_STP_DMA		0x00000020
#define CSMI_SAS_STP_PACKET		0x00000040
#define CSMI_SAS_STP_DMA_QUEUED		0x00000080
#define CSMI_SAS_STP_EXECUTE_DIAG	0x00000100
#define CSMI_SAS_STP_RESET_DEVICE	0x00000200

// Task Management Flags
// (uFlags)

#define CSMI_SAS_TASK_IU		0x00000001
#define CSMI_SAS_HARD_RESET_SEQUENCE	0x00000002
#define CSMI_SAS_SUPPRESS_RESULT	0x00000004

// Task Management Functions
// (bTaskManagement)

#define CSMI_SAS_SSP_ABORT_TASK			0x01
#define CSMI_SAS_SSP_ABORT_TASK_SET		0x02
#define CSMI_SAS_SSP_CLEAR_TASK_SET		0x04
#define CSMI_SAS_SSP_LOGICAL_UNIT_RESET		0x08
#define CSMI_SAS_SSP_CLEAR_ACA			0x40
#define CSMI_SAS_SSP_QUERY_TASK			0x80

// Task Management Information
// (uInformation)

#define CSMI_SAS_SSP_TEST		1
#define CSMI_SAS_SSP_EXCEEDED		2
#define CSMI_SAS_SSP_DEMAND		3
#define CSMI_SAS_SSP_TRIGGER		4

// Connector Pinout Information
// (uPinout)

#define CSMI_SAS_CON_UNKNOWN		0x00000001
#define CSMI_SAS_CON_SFF_8482		0x00000002
#define CSMI_SAS_CON_SFF_8470_LANE_1	0x00000100
#define CSMI_SAS_CON_SFF_8470_LANE_2	0x00000200
#define CSMI_SAS_CON_SFF_8470_LANE_3	0x00000400
#define CSMI_SAS_CON_SFF_8470_LANE_4	0x00000800
#define CSMI_SAS_CON_SFF_8484_LANE_1	0x00010000
#define CSMI_SAS_CON_SFF_8484_LANE_2	0x00020000
#define CSMI_SAS_CON_SFF_8484_LANE_3	0x00040000
#define CSMI_SAS_CON_SFF_8484_LANE_4	0x00080000

// Connector Location Information
// (bLocation)

// same as uPinout above...
// #define CSMI_SAS_CON_UNKNOWN		0x01
#define CSMI_SAS_CON_INTERNAL		0x02
#define CSMI_SAS_CON_EXTERNAL		0x04
#define CSMI_SAS_CON_SWITCHABLE		0x08
#define CSMI_SAS_CON_AUTO		0x10
#define CSMI_SAS_CON_NOT_PRESENT	0x20
#define CSMI_SAS_CON_NOT_CONNECTED	0x80

// Device location identification
// (bIdentify)

#define CSMI_SAS_LOCATE_UNKNOWN		0x00
#define CSMI_SAS_LOCATE_FORCE_OFF	0x01
#define CSMI_SAS_LOCATE_FORCE_ON	0x02

// Location Valid flags
// (uLocationFlags)

#define CSMI_SAS_LOCATE_SAS_ADDRESS_VALID			0x00000001
#define CSMI_SAS_LOCATE_SAS_LUN_VALID				0x00000002
#define CSMI_SAS_LOCATE_ENCLOSURE_IDENTIFIER_VALID		0x00000004
#define CSMI_SAS_LOCATE_ENCLOSURE_NAME_VALID			0x00000008
#define CSMI_SAS_LOCATE_BAY_PREFIX_VALID			0x00000010
#define CSMI_SAS_LOCATE_BAY_IDENTIFIER_VALID			0x00000020
#define CSMI_SAS_LOCATE_LOCATION_STATE_VALID			0x00000040

/* * * * * * * * SAS Phy Control Class IOCTL Constants * * * * * * * * */

// Return codes for SAS Phy Control IOCTL's
// (IoctlHeader.ReturnCode)

// Signature value
// (IoctlHeader.Signature)

#define CSMI_PHY_SIGNATURE			"CSMIPHY"

// Phy Control Functions
// (bFunction)

// values 0x00 to 0xFF are consistent in definition with the SMP PHY CONTROL 
// function defined in the SAS spec
#define CSMI_SAS_PC_NOP				0x00000000
#define CSMI_SAS_PC_LINK_RESET			0x00000001
#define CSMI_SAS_PC_HARD_RESET			0x00000002
#define CSMI_SAS_PC_PHY_DISABLE			0x00000003
// 0x04 to 0xFF reserved...
#define CSMI_SAS_PC_GET_PHY_SETTINGS		0x00000100

// Link Flags
#define CSMI_SAS_PHY_ACTIVATE_CONTROL		0x00000001
#define CSMI_SAS_PHY_UPDATE_SPINUP_RATE		0x00000002
#define CSMI_SAS_PHY_AUTO_COMWAKE		0x00000004

// Device Types for Phy Settings
// (bType)
#define CSMI_SAS_UNDEFINED			0x00
#define CSMI_SAS_SATA				0x01
#define CSMI_SAS_SAS				0x02

// Transmitter Flags
// (uTransmitterFlags)
#define CSMI_SAS_PHY_PREEMPHASIS_DISABLED	0x00000001

// Receiver Flags
// (uReceiverFlags)
#define CSMI_SAS_PHY_EQUALIZATION_DISABLED	0x00000001

// Pattern Flags
// (uPatternFlags)
// #define CSMI_SAS_PHY_ACTIVATE_CONTROL	0x00000001
#define CSMI_SAS_PHY_DISABLE_SCRAMBLING		0x00000002
#define CSMI_SAS_PHY_DISABLE_ALIGN		0x00000004
#define CSMI_SAS_PHY_DISABLE_SSC		0x00000008

#define CSMI_SAS_PHY_FIXED_PATTERN		0x00000010
#define CSMI_SAS_PHY_USER_PATTERN		0x00000020

// Fixed Patterns
// (bFixedPattern)
#define CSMI_SAS_PHY_CJPAT			0x00000001
#define CSMI_SAS_PHY_ALIGN			0x00000002

// Type Flags
// (bTypeFlags)
#define CSMI_SAS_PHY_POSITIVE_DISPARITY		0x01
#define CSMI_SAS_PHY_NEGATIVE_DISPARITY		0x02
#define CSMI_SAS_PHY_CONTROL_CHARACTER		0x04

#define ARCSAS_EVENT_TYPE_MESSAGE_ONLY		0
#define ARCSAS_EVENT_TYPE_PM_SLOT		1
#define ARCSAS_EVENT_TYPE_EXPANDER_SLOT		2
#define ARCSAS_EVENT_TYPE_HBA_SLOT		3
#define ARCSAS_EVENT_TYPE_PM			4
#define ARCSAS_EVENT_TYPE_EXPANDER		5
#define ARCSAS_EVENT_TYPE_ENCLOSURE		6
#define ARCSAS_EVENT_TYPE_HBA			7

/*
************************************************************************
**          Peripherals Registers
**          ARC1320 SPI
************************************************************************
*/
#define SPI_CTRL_SpiStart_1320		ARCSAS_BIT(1, 0)
#define SPI_ADDR_VLD_1320		ARCSAS_BIT(1, 1)
#define SPI_CTRL_READ_1320		ARCSAS_BIT(1, 2)

#define SPI_CTRL_REG_1320		0xc800
#define SPI_ADDR_REG_1320            	0xc804
#define SPI_WRITE_DATA_REG_1320		0xc808
#define SPI_READ_DATA_REG_1320         	0xc80c
/*
************************************************************************
**          ARC1300 SPI
************************************************************************
*/
#define SPI_CTRL_SpiStart_1300		ARCSAS_BIT(1,  20 )
#define SPI_CTRL_SpiRdy_1300		ARCSAS_BIT(1,  22 )
#define SPI_CTRL_VenderEnable_1300	ARCSAS_BIT(1,  29 )

#define SPI_CTRL_REG_1300		0x10
#define SPI_CMD_REG_1300		0x14
#define SPI_WRITE_DATA_REG_1300		0x18/*same address as read*/
#define SPI_READ_DATA_REG_1300		0x18/*same address as write*/
/*
************************************************************************
**
************************************************************************
*/
#define SPI_INS_WREN				0 /* spi instruction write enable          */
#define SPI_INS_WRDI				1 /* spi instruction write disable         */
#define SPI_INS_RDSR				2 /* spi instruction read status register  */
#define SPI_INS_WRSR				3 /* spi instruction write status register,The WRSR instruction also allows the user to enable or disable the Write Protect */
#define SPI_INS_READ				4 /* spi instruction read                  */
#define SPI_INS_PROG				5 /* spi instruction program               */
#define SPI_INS_SERASE				6 /* spi instruction sector erase, Erase One Sector in Memory Array */
#define SPI_INS_CERASE				7 /* spi instruction chip erase, Erase All Sectors in Memory Array  */
#define SPI_INS_RDID				8 /* spi instruction read flash id         */
#define SPI_INS_PTSEC				9 /* spi instruction protect sector        */
#define SPI_INS_UPTSEC				10 /* spi instruction unprotect sector      */
#define SPI_INS_RDPT 				11 /* spi instruction read protect          */
#define SPI_MAX_CMD				12
/*ATMEL*/
#define AT25F2048				0x0101
#define AT25DF041A				0x0102
#define AT25DF021				0x0103
/*MXIC*/
#define MX25L2005				0x0201
#define MX25L4005				0x0202
#define MX25L8005				0x0203
/*WINBOND*/
#define WINBOND					0x0301
/*SST*/
#define SST25VF040B				0x0501
/*CHINGIS*/
#define PM25LV010A				0x0601
#define PM25LV020				0x0602
#define PM25LV040				0x0603
#define PM25LD010C				0x0604
#define PM25LD020C				0x0605
/* SES2 Page Code */
#define SES_PG_SUPPORTED_DIAGNOSTICS		0x00
#define SES_PG_CONFIGURATION			0x01   
#define SES_PG_ENCLOSURE_STATUS			0x02
#define SES_PG_HELP_TEXT			0x03   
#define SES_PG_STRING_IN			0x04   
#define SES_PG_THRESHOLD_IN			0x05   
#define SES_PG_ELEMENT_DESCRIPTOR		0x07
#define SES_PG_SHORT_ENCLOSURE_STATUS		0x08
#define SES_PG_ENCLOSURE_BUSY			0x09
#define SES_PG_ADDITIONAL_ELEMENT_STATUS	0x0a
#define SES_PG_SUBENCLOSURE_HELP_TEXT		0x0b
#define SES_PG_SUBENCLOSURE_STRING_IN		0x0c
#define SES_PG_SUPPORTED_SES_DIAGNOSTICS	0x0d
#define SES_PG_VENDOR_SPECIFIC_SES_PAGE		0x10
#define SES_PG_VENDOR_PAGEA0			0xa0
/* Element Type */
#define SES_TYPE_DEVICE				0x01
#define CMD_SSP					0x01
#define CMD_SMP					0x02
#define CMD_STP					0x03
#define CMD_SSP_TGT				0x04
#define CMD_SLOT_RESET				0x07
/*
************************************************************************
**                  I2C
************************************************************************
*/
#define REQ_CMD_HOST_ID_I2CWRITE	0x0010
#define REQ_CMD_HOST_ID_I2CREAD		0x0011
#define REQ_CMD_CRC32_I2CWRITE		0x8010
#define REQ_CMD_CRC32_I2CREAD		0x8011
#define I2C_DATA_WRITE			0x00
#define I2C_DATA_READ			0x01
/*
************************************************************************
**                  I2C_1300
************************************************************************
*/
#define I2C_SOFTWARE_CONTROL_1300		0x1c
#define I2C_HARDWARE_CONTROL_1300		0x20
#define I2C_STATUS_DATA_1300			0x24
/*
************************************************************************
**                  I2C_1320
************************************************************************
*/
#define I2C_SOFTWARE_CONTROL_1320_A		0x0C51C
#define I2C_HARDWARE_CONTROL_1320_A		0x0C520
#define I2C_STATUS_DATA_1320_A			0x0C524
#define I2C_SOFTWARE_CONTROL_1320_B		0x0C61C
#define I2C_HARDWARE_CONTROL_1320_B		0x0C620
#define I2C_STATUS_DATA_1320_B			0x0C624
#define I2C_SOFTWARE_CONTROL_1320_C		0x1C71C
#define I2C_HARDWARE_CONTROL_1320_C		0x1C720
#define I2C_STATUS_DATA_1320_C			0x1C724
/* 
************************************************************************
** BUS_PTL value
************************************************************************
*/
#define BUS_PTL_WRITE_QUICK			0x00 /* Write quick. */
#define BUS_PTL_READ_QUICK			0x01 /* Read quick. */
#define BUS_PTL_SEND_BYTE			0x02 /* Send byte. */
#define BUS_PTL_RCV_BYTE			0x03 /* Receive byte. */
#define BUS_PTL_WRITE_BYTE			0x04 /* Write byte. */
#define BUS_PTL_READ_BYTE			0x05 /* Read byte. */
#define BUS_PTL_WRITE_WORD			0x06 /* Write word. */
#define BUS_PTL_READ_WORD			0x07 /* Read word. */
#define BUS_PTL_RCV_WORD			0x0F /* Receive word. */
/*  SAS/SATA Vendor Specific Port Registers */
enum sas_sata_vsp_regs 
{
	ARC1300_VSR_PHY_STAT			= 0x00, /* Phy Status */
	ARC1300_VSR_PHY_MODE1		= 0x01, /* phy tx */
	ARC1300_VSR_PHY_MODE2		= 0x02, /* tx scc */
	ARC1300_VSR_PHY_MODE3		= 0x03, /* pll */
	ARC1300_VSR_PHY_MODE4		= 0x04, /* VCO */
	ARC1300_VSR_PHY_MODE5		= 0x05, /* Rx */
	ARC1300_VSR_PHY_MODE6		= 0x06, /* CDR */
	ARC1300_VSR_PHY_MODE7		= 0x07, /* Impedance */
	ARC1300_VSR_PHY_MODE8		= 0x08, /* Voltage */
	ARC1300_VSR_PHY_MODE9		= 0x09, /* Test */
	ARC1300_VSR_PHY_MODE10		= 0x0A, /* Power */
	ARC1300_VSR_PHY_MODE11		= 0x0B, /* Phy Mode */
	ARC1300_VSR_PHY_VS0			= 0x0C, /* Vednor Specific 0 */
	ARC1300_VSR_PHY_VS1			= 0x0D, /* Vednor Specific 1 */
	ARC1320_VSR_IRQ_STATUS			= 0x00,
	ARC1320_VSR_IRQ_MASK			= 0x04,
	ARC1320_VSR_PHY_CONFIG			= 0x08,
	ARC1320_VSR_PHY_STATUS			= 0x0c,
	ARC1320_VSR_PHY_MODE_REG_1		= 0x064,
	ARC1320_VSR_PHY_FFE_CONTROL		= 0x10C,
	ARC1320_VSR_PHY_DFE_UPDATE_CRTL		= 0x110,
	ARC1320_VSR_REF_CLOCK_CRTL		= 0x1A0,
	ARC1320_VSR_PHY_GENERATION_1_SETTING	= 0x118,
	ARC1320_VSR_PHY_GENERATION_1_2_SETTING	= 0x11C,
	ARC1320_VSR_PHY_GENERATION_2_3_SETTING	= 0x120,
	ARC1320_VSR_PHY_GENERATION_3_4_SETTING	= 0x124,
	ARC1320_VSR_PHY_STAT			= 0x00 * 4, /* Phy Status */
	ARC1320_VSR_PHY_MODE1			= 0x01 * 4, /* phy tx */
	ARC1320_VSR_PHY_MODE2			= 0x02 * 4, /* tx scc */
	ARC1320_VSR_PHY_MODE3			= 0x03 * 4, /* pll */
	ARC1320_VSR_PHY_MODE4			= 0x04 * 4, /* VCO */
	ARC1320_VSR_PHY_MODE5			= 0x05 * 4, /* Rx */
	ARC1320_VSR_PHY_MODE6			= 0x06 * 4, /* CDR */
	ARC1320_VSR_PHY_MODE7			= 0x07 * 4, /* Impedance */
	ARC1320_VSR_PHY_MODE8			= 0x08 * 4, /* Voltage */
	ARC1320_VSR_PHY_MODE9			= 0x09 * 4, /* Test */
	ARC1320_VSR_PHY_MODE10			= 0x0A * 4, /* Power */
	ARC1320_VSR_PHY_MODE11			= 0x0B * 4, /* Phy Mode */
	ARC1320_VSR_PHY_VS0			= 0x0C * 4, /* Vednor Specific 0 */
	ARC1320_VSR_PHY_VS1			= 0x0D * 4, /* Vednor Specific 1 */
};
typedef struct _ReceiveDiagnostic
{
	unsigned char OperationCode;

	unsigned char PCV:1;
	unsigned char Reserved:7;

	unsigned char PageCode;

	unsigned char AllocationLength[2];

	unsigned char Control;
}ReceiveDiagnostic, *PReceiveDiagnostic;

typedef struct _SendDiagnostic
{
	unsigned char OperationCode;

	unsigned char UnitOffL:1;
	unsigned char DevOffL:1;
	unsigned char SelfTest:1;
	unsigned char Reserved:1;
	unsigned char PF:1;
	unsigned char SelfTestCode:3;

	unsigned char ParamLength[2];

	unsigned char Control;
} SendDiagnostic, *PSendDiagnostic;

typedef struct _SupportedPageHeader
{
	unsigned char PageCode;
	unsigned char Reserved;
	unsigned char PageLength[2];
    /*
    ** The PAGE LENGTH field specifies the length in bytes of the following supported page list
    */
} SupportedPageHeader, *PSupportedPageHeader;

/* SES_PG_CONFIGURATION */
typedef struct _ConfigurationPageHeader
{
	unsigned char PageCode;
	unsigned char SubEnclosureCount;
	unsigned char PageLength[2];
	unsigned char GenerationCode[4];
} ConfigurationPageHeader, *PConfigurationPageHeader;

typedef struct _EnclosureDescriptorHeader
{
	unsigned char EnclosureServiceProcessesCount:3;
	unsigned char Reserved1:1;
	unsigned char RelativeEnclosureServiceProcessID:3;
	unsigned char Reserved2:1;

	unsigned char SubEnclosureID;
	unsigned char NumberOfElementTypesSupported;
	unsigned char EnclosureDescriptorLength;
   
} EnclosureDescriptorHeader, *PEnclosureDescriptorHeader;

typedef struct _EnclosureDescriptor
{
	unsigned char EnclosureLogicalID[8];
	unsigned char EnclosureVendorID[8];
	unsigned char ProductID[16];
	unsigned char ProductRevisionLevel[4];
	unsigned char VendorSpecific; 
} EnclosureDescriptor, *PEnclosureDescriptor;

typedef struct _TypeDescriptorHeader
{
	unsigned char ElementType;              /*SesElementType*/
	unsigned char NumberOfPossibleElements;
	unsigned char SubEnclosureID;
	unsigned char TypeDescriptorTextLength;
} TypeDescriptorHeader, *PTypeDescriptorHeader;
/* SES_PG_ADDITIONAL_ELEMENT_STATUS */
typedef struct _DeviceElementStatusPage
{
	unsigned char PageCode;
	unsigned char Reserved;
	unsigned char PageLength[2];  
	unsigned char GenerationCode[4];
} DeviceElementStatusPage, *PDeviceElementStatusPage;

typedef struct _DeviceElementStatusDescriptorEIP0
{
	unsigned char ProtocolID:4;
	unsigned char bEIP:1;
	unsigned char Reserved1:2;
	unsigned char Invalid:1;

	unsigned char DeviceElementStatusDescriptorLength;
	unsigned char PhyDescriptorCount;

	unsigned char NotAllPhys:1;
	unsigned char Reserved2:5;
	unsigned char DescriptorType:2;
} DeviceElementStatusDescriptorEIP0, *PDeviceElementStatusDescriptorEIP0;

typedef struct _DeviceElementStatusDescriptorEIP1
{
	unsigned char ProtocolID:4;
	unsigned char bEIP:1;
	unsigned char Reserved1:2;
	unsigned char Invalid:1;

	unsigned char DeviceElementStatusDescriptorLength;
	unsigned char Reserved2;
	unsigned char ElementIndex;
	unsigned char PhyDescriptorCount;

	unsigned char NotAllPhys:1;
	unsigned char Reserved3:5;
	unsigned char DescriptorType:2;

	unsigned char Reserved4;
	unsigned char BayNumber;
} DeviceElementStatusDescriptorEIP1, *PDeviceElementStatusDescriptorEIP1;

typedef union 
{
	DeviceElementStatusDescriptorEIP0	EIP0;
	DeviceElementStatusDescriptorEIP1	EIP1;	
}DeviceElementStatusDescriptor, *PDeviceElementStatusDescriptor;
/*
************************************************************************
** I2C_SOFTWARE_CONTROL bits 
************************************************************************
*/
typedef struct _SASPhyDescriptor
{
	unsigned char Reserved1:4;
	unsigned char DeviceType:3;
	unsigned char Reserved2:1;

	unsigned char Reserved3;

	unsigned char Reserved4:1;
	unsigned char SMPInitiatorPort:1;
	unsigned char STPInitiatorPort:1;
	unsigned char SSPInitiatorPort:1;
	unsigned char Reserved5:4;

	unsigned char Reserved6:1;
	unsigned char SMPTargetPort:1;
	unsigned char STPTargetPort:1;
	unsigned char SSPTargetPort:1;
	unsigned char Reserved7:4;

	unsigned char AttachedSASAddress[8];
	unsigned char SASAddress[8];
	unsigned char PhyIdentifier;
	unsigned char Reserved8[7];
} SASPhyDescriptor, *PSASPhyDescriptor;
/* SES_PG_ELEMENT_DESCRIPTOR */
typedef struct _ElementDescriptorPage
{
	unsigned char PageCode;
	unsigned char Reserved1;
	unsigned char PageLength[2]; 
	unsigned char GenerationCode[4];
} ElementDescriptorPage, *PElementDescriptorPage;

typedef struct _ElementDescriptor
{
	unsigned char Reserved1;
	unsigned char Reserved2;
	unsigned char DescriptorLength[2];  
	unsigned char Descriptor[1];
} ElementDescriptor, *PElementDescriptor;

/* SES_PG_ENCLOSURE_STATUS_CONTROL */
typedef struct _OverallEnclosureStatus
{
	unsigned char UnrecoverableCondition:1;
	unsigned char CriticalCondition:1;
	unsigned char NoncriticalCondition:1;
	unsigned char InformationCondition:1;
	unsigned char InvalidOperationRequested:1;
	unsigned char Reserved1:3;
    /*byte0*/
} OverallEnclosureStatus, *pOverallEnclosureStatus;

typedef struct _EnclosureStatusPage
{
	unsigned char PageCode;
    /*byte0*/
	OverallEnclosureStatus Status;
    /*byte1*/
	unsigned char PageLength[2];  
    /*byte2,3*/
	unsigned char GenerationCode[4];
    /*byte4,5,6,7*/
} EnclosureStatusPage, *PEnclosureStatusPage;

typedef struct _DeviceElementForEnclosureStatusPage
{
	unsigned char ElementStatusCode:4;
	unsigned char Swap:1;
	unsigned char Reserved1:1;
	unsigned char Prdfail:1;
	unsigned char Reserved2:1;
    /*byte0*/
	unsigned char SlotAddress;
    /*byte1*/
	unsigned char Report:1;
	unsigned char Ident:1;
	unsigned char Rmv:1;
	unsigned char ReadyToInsert:1;
	unsigned char EnclosureBypassedB:1;
	unsigned char EnclosureBypassedA:1;
	unsigned char DoNotRemove:1;
	unsigned char AppClientBypassedA:1;
    /*byte2*/
	unsigned char DeviceBypassedB:1;
	unsigned char DeviceBypassedA:1;
	unsigned char BypassedB:1;
	unsigned char BypassedA:1;
	unsigned char DeviceOff:1;
	unsigned char FaultReqstd:1;
	unsigned char FaultSensed:1;
	unsigned char AppClientBypassedB:1;
    /*byte3*/
}DeviceElementForEnclosureStatusPage, *PDeviceElementForEnclosureStatusPage;

/* Element Status, Summary SES Status Defines, Common Status Codes */
typedef struct _ArrayElementForEnclosureStatusPage
{
	unsigned char StatusCode:4;
	unsigned char Swap:1;
	unsigned char Reserved1:1;
	unsigned char PredictedFailure:1;
	unsigned char Reserved2:1;
    /*byte0*/
	unsigned char RebuildRemapAbort:1;
	unsigned char RebuildRemap:1;
	unsigned char InFailedArray:1;
	unsigned char InCriticalArray:1;
	unsigned char ConsistencyCheck:1;
	unsigned char HotSpare:1;
	unsigned char ReservedDevice:1;
	unsigned char OK:1;
    /*byte1*/
	unsigned char Report:1;
	unsigned char Identify:1;
	unsigned char Rmv:1;
	unsigned char ReadyToInsert:1;
	unsigned char EnclosureBypassedB:1;
	unsigned char EnclosureBypassedA:1;
	unsigned char DoNotRemove:1;
	unsigned char AppBypassedA:1;
    /*byte2*/
	unsigned char DeviceBypassedB:1;
	unsigned char DeviceBypassedA:1;
	unsigned char BypassedB:1;
	unsigned char BypassedA:1;
	unsigned char DeviceOff:1;
	unsigned char FaultRequested:1;
	unsigned char FaultSensed:1;
	unsigned char AppBypassedB:1;
    /*byte3*/
} ArrayElementForEnclosureStatusPage, *PArrayElementForEnclosureStatusPage;

/* SES_PG_ENCLOSURE_CONTROL */
typedef struct _EnclosureControlPage
{
	unsigned char PageCode;
	unsigned char UnrecoverableCondition:1;
	unsigned char CriticalCondition:1;
	unsigned char NoncriticalCondition:1;
	unsigned char InformationCondition:1;
	unsigned char Reserved1:4;
	unsigned char PageLength[2];  
	unsigned char GenerationCode[4];
} EnclosureControlPage, *PEnclosureControlPage;

typedef struct _DeviceElementForEnclosureControlPage
{
	unsigned char Reserved1:4;
	unsigned char ResetSwap:1;
	unsigned char Disable:1;
	unsigned char PredictedFailure:1;
	unsigned char Select:1;
    /*byte0*/
	unsigned char Reserved2;
    /*byte1*/
	unsigned char Reserved3:1;
	unsigned char RequestIdentify:1;
	unsigned char RequestRemove:1;
	unsigned char RequestInsert:1;
	unsigned char Reserved4:2;
	unsigned char DoNotRemove:1;
	unsigned char Active:1;
    /*byte2*/
	unsigned char Reserved5:2;
	unsigned char EnableBypassB:1;
	unsigned char EnableBypassA:1;
	unsigned char DeviceOff:1;
	unsigned char RequestFault:1;
	unsigned char Reserved6:2;
    /*byte3*/
} DeviceElementForEnclosureControlPage, *PDeviceElementForEnclosureControlPage;

typedef struct _ArrayDeviceSlotControlElement
{
	u8 Reserved0_0123:4;
	u8  ResetSwap:1;
	u8  Disable:1;
	u8  PredictedFailure:1;
	u8  Select:1;
    /*byte0*/
	u8  RequestRebuildRemapAbort:1;
	u8  RequestRebuildRemap:1;
	u8  RequestInFailedArray:1;
	u8  RequestInCriticalArray:1;
	u8  RequestConsistencyCheck:1;
	u8  RequestHotSpare:1;
	u8  RequestReservedDevice:1;
	u8  RequestOK:1;
    /*byte1*/
	u8  Reserved2:1;
	u8  RequestIdentify:1;
	u8  RequestRemove:1;
	u8  RequestInsert:1;
	u8  Reserved3:2;
	u8  DoNotRemove:1;
	u8  Active:1;
    /*byte2*/
	u8  Reserved4:2;
	u8  EnableBypassB:1;
	u8  EnableBypassA:1;
	u8  DeviceOff:1;
	u8  RequestFault:1;
	u8  Reserved5:2;
    /*byte3*/
} ArrayElementForEnclosureControlPage, *PArrayElementForEnclosureControlPage;

typedef union 
{
	DeviceElementForEnclosureStatusPage  Status;    /*4 bytes*/
	DeviceElementForEnclosureControlPage Control;	/*4 bytes*/
} DeviceEnclosureElement, *PDeviceEnclosureElement;

typedef union 
{
	ArrayElementForEnclosureStatusPage  Status;     /*4 bytes*/
	ArrayElementForEnclosureControlPage Control;	/*4 bytes*/
} ArrayEnclosureElement, *PArrayEnclosureElement;

typedef union 
{
	DeviceEnclosureElement DiskType;  /*8 bytes*/
	ArrayEnclosureElement  ArrayType;	/*8 bytes such as ARC8060 in ENCLOSURE */
} SesEnclosureElement, *PSesEnclosureElement;

typedef struct _REG_I2C_SOFTWARE_CONTROL
{
	u32	DATA_PORT:8;    /*7:0   */
	u32	ADRS_PORT:3;    /*10:8  */
	u32	Reserved1:1;    /*11    */
	u32	WRITE_EN:1;     /*12    */
	u32	READ_EN:1;      /*13    */
	u32	Reserved2:18;   /*31:14 */
} REG_I2C_SOFTWARE_CONTROL, *PREG_I2C_SOFTWARE_CONTROL;
/* 
********************************************
** ADRS_PORT values,  MI2C register address 
********************************************
*/
#define MI2C_SLAVE_ADDRESS		0x0
#define MI2C_X_SLAVE_ADDRESS		0x4
#define MI2C_DATA			0x1
#define MI2C_CONTROL			0x2
#define MI2C_STATUS			0x3 /* READ */
#define MI2C_CLOCK_CONTROL		0x3 /* WRITE */
#define MI2C_SOFT_RESET			0x7
/*
********************************************
** MI2C_SLAVE_ADDRESS bits 
********************************************
*/
typedef struct _REG_MI2C_SLAVE_ADDRESS
{
	unsigned char	GCE:1; /* general call address enable */
	unsigned char	SLA:7; /* Slave address 0-6 */
} REG_MI2C_SLAVE_ADDRESS, *PREG_MI2C_SLAVE_ADDRESS;
/* 
********************************************
** MI2C_CONTROL bits 
********************************************
*/
typedef struct _REG_MI2C_CONTROL
{
	unsigned char	Reserved:2;
	unsigned char	AAK:1; /* Assert AcKnowledge */
	unsigned char	IFLG:1; /* Interrupt FLaG */
	unsigned char	STP:1; /* Master Stop */
	unsigned char	STA:1; /* Master Start */
	unsigned char	ENAB:1; /* Bus Enable */
	unsigned char	IEN:1; /* Interrupt Enable */
} REG_MI2C_CONTROL, *PREG_MI2C_CONTROL;
/* 
********************************************
** MI2C_STATUS bits 
********************************************
*/
#define MI2C_STATUS_BUS_ERROR				0x00       /*bus error*/
#define MI2C_STATUS_START_TX				0x08       /*START condition trans*/
#define MI2C_STATUS_REP_START_TX			0x10
#define MI2C_STATUS_ADDR_W_TX_ACK			0x18
#define MI2C_STATUS_ADDR_W_TX_NAK			0x20
#define MI2C_STATUS_MDATA_TX_ACK			0x28
#define MI2C_STATUS_MDATA_TX_NAK			0x30
#define MI2C_STATUS_ARBLST_ADDR_DATA			0x38
#define MI2C_STATUS_ADDR_R_TX_ACK			0x40
#define MI2C_STATUS_ADDR_R_TX_NAK			0x48
#define MI2C_STATUS_MDATA_RX_ACK			0x50
#define MI2C_STATUS_MDATA_RX_NAK			0x58
#define MI2C_STATUS_SADDR_W_RX_ACK			0x60
#define MI2C_STATUS_ARBLST_SADDR_W_RX_ACK		0x68
#define MI2C_STATUS_GC_ADDR_RX_ACK			0x70
#define MI2C_STATUS_ARBLST_GC_RX_ACK			0x78
#define MI2C_STATUS_SADDR_DATA_RX_ACK			0x80
#define MI2C_STATUS_SADDR_DATA_RX_NAK			0x88
#define MI2C_STATUS_GC_DATA_RX_ACK			0x90
#define MI2C_STATUS_GC_DATA_RX_NAK			0x98
#define MI2C_STATUS_STOP_RX				0xA0
#define MI2C_STATUS_SADDR_R_RX_ACK			0xA8
#define MI2C_STATUS_ARBLST_SADDR_R_RX_ACK		0xB0
#define MI2C_STATUS_SDATA_TX_ACK			0xB8
#define MI2C_STATUS_SDATA_TX_NAK			0xC0
#define MI2C_STATUS_LAST_SDATA_TX_ACK			0xC8
#define MI2C_STATUS_ADDR2_W_TX_ACK			0xD0
#define MI2C_STATUS_ADDR2_W_TX_NAK			0xD8
#define MI2C_STATUS_ADDR2_R_TX_ACK			0xE0
#define MI2C_STATUS_ADDR2_R_TX_NAK			0xE8
#define MI2C_STATUS_NONE				0xF8
/* 
************************************************************************
** I2C_State in Core_Extension
************************************************************************
*/
enum 
{
	I2C_STATE_IDLE=0,
	I2C_STATE_CMD,
	I2C_STATE_RESPONSE,
	I2C_STATE_ERROR
};
/*
************************************************************************
** I2C_HARDWARE_CONTROL_1300 bits 
************************************************************************
*/
typedef struct _REG_I2C_HARDWARE_CONTROL
{
#ifdef _BIG_ENDIAN_BITFIELD__
	u32	INT:1;          /*31    */
	u32	Reserved:6;     /*30:25 */
	u32	BUS_PTL_START:1;/*24    */
	u32	CMD_CODE:8;     /*23:16 */
	u32	BUS_PTL:8;      /*15:8  */
	u32	SLV_ADRS:8;     /*7:0   */
#else
	u32	SLV_ADRS:8;     /*7:0   */
	u32	BUS_PTL:8;      /*15:8  */
	u32	CMD_CODE:8;     /*23:16 */
	u32	BUS_PTL_START:1;/*24    */
	u32	Reserved:6;     /*30:25 */
	u32	INT:1;          /*31    */
#endif
} REG_I2C_HARDWARE_CONTROL, *PREG_I2C_HARDWARE_CONTROL;
/*
************************************************************************
** I2C_STATUS_DATA bits 
************************************************************************
*/
typedef struct _REG_I2C_STATUS_DATA
{
	u32	INT_ST:8;           /*7:0 for INT_ST value, see MI2C_STATUS */
	u32	Reserved:8;         /*15:8  */
	u32	DATA_LO:8;          /*23:16 */
	u32	DATA_HI:8;          /*31:24 */
} REG_I2C_STATUS_DATA, *PREG_I2C_STATUS_DATA;
/* 
************************************************************************
** 150MHz/10 
************************************************************************
*/
#define I2C_SYS_CLOCK	150000000/10
/* 
************************************************************************
** I2C link rate definitions 
************************************************************************
*/
#define  I2C_LINK_RATE	100*1000          /* 100 kbps   */
//#define  I2C_LINK_RATE        400*1000          /* 400 kbps   */
/*
************************************************************************
** Definition for SES commands over I2C (to SEP)
** Key components:
**       [SEP address] - [R/W direction] - [Command Type] - [Checksum] - [SEMB address] - [Seq number] - [SEP/SES commands] - [Data...
************************************************************************
*/
/* 
************************************************************************
** [SEP address] 
************************************************************************
*/
#define I2C_SEP_ADDR0	0xC0 /* i2c address for SEP 0           */
#define I2C_SEP_ADDR1	0xC2 /* i2c address for SEP 1           */
/* 
************************************************************************
** [SEMB address] 
************************************************************************
*/
#define I2C_SEMB_ADDR	0xD0 /* i2c address for SEMB            */

/*
************************************************************************ 
** To be backward compatible, keep the original Adapter_Config 
************************************************************************
*/
typedef struct _Adapter_Config 
{
	u32	BIOS_Flag;/*4 bytes*/
	u32	Reserved;
} Adapter_Config, *PAdapter_Config;
/* 
************************************************************************
** [ARC100 address] 
************************************************************************
*/
#define I2C_ARC100_ADDR	0x10 /* ARC100 i2c slave address        */
/* 
************************************************************************
** SEP ATA command code 
************************************************************************
*/
#define SATA_SEP_ATTN	0x67 /* for all SEP command             */
/*
************************************************************************
** I2C command type for SEP device
************************************************************************
*/
#define I2C_CT_D2H_SAFTE	0x00 /* SEP-to-Host SAFTE data transfer */
#define I2C_CT_H2D_SAFTE	0x80 /* Host-to-SEP SAFTE data transfer */
#define I2C_CT_D2H_SES		0x02 /* SEP-to-Host SES data transfer   */
#define I2C_CT_H2D_SES		0x82 /* Host-to-SEP SES data transfer   */
/* 
************************************************************************
** command type flag 
************************************************************************
*/
#define I2C_CT_H2D		0x80
/* 
************************************************************************
macro for judge H2D/D2H command type
************************************************************************
*/
#define mis_i2c_ct_d2h(ct)    (!((ct) & I2C_CT_H2D))
/* 
************************************************************************
SEP command 
************************************************************************
*/
#define I2C_CMD_IDENTIFY_SEP 0xEC /* identify sep */
/*
************************************************************************
** IDENDIFY SEP data page definitions 
************************************************************************
*/
typedef struct IDENTIFY_SEP_DATA_
{
	unsigned char       dataLength;
	unsigned char       subenclosureID;
	unsigned char       logicalID[8];
	unsigned char       vendorID[8];
	unsigned char       productID[16];
	unsigned char       revisionLevel[4];
	unsigned char       channelID;
	unsigned char       firmwareRevisionLevel[4];
	unsigned char       interfaceIDstring[6];
	unsigned char       interfaceRevisionLevel[4];
	unsigned char       vendorSpecific[11];
} IDENTIFY_SEP_DATA, *PIDENTIFY_SEP_DATA;
/* 
************************************************************************
** Product device id 
************************************************************************
*/
//#define VENDOR_ID				0x11AB
#define DEVICE_ID_1300				0x1300
#define DEVICE_ID_1320				0x1320
#define DEVICE_ID_UNKNOWN			0xFFFF
/*
************************************************************************
 * When you plug-in this command consolidate sub-module to some module
 * Please define the following the definition.
 * This is maintained by caller.
************************************************************************
*/
/*
 * ATA IDE Command definition 
*/
/* SATA SSD TRIM COMMAND */
#define ATA_CMD_DATA_SET_MANAGEMENT			0x06
#define ATA_DSM_TRIM					0x01
/* PIO command */
#define ATA_CMD_READ_PIO				0x20
#define ATA_CMD_READ_PIO_MULTIPLE			0xC4
#define ATA_CMD_READ_PIO_EXT				0x24
#define ATA_CMD_READ_PIO_MULTIPLE_EXT			0x29
#define ATA_CMD_WRITE_PIO				0x30
#define ATA_CMD_WRITE_PIO_MULTIPLE			0xC5
#define ATA_CMD_WRITE_PIO_EXT				0x34
#define ATA_CMD_WRITE_PIO_MULTIPLE_EXT			0x39
#define ATA_CMD_WRITE_PIO_MULTIPLE_FUA_EXT		0xCE

/* DMA read write command */
#define ATA_CMD_READ_DMA				0xC8	/* 24 bit DMA read */
#define ATA_CMD_READ_DMA_QUEUED				0xC7	/* 24 bit TCQ DMA read */
#define ATA_CMD_READ_DMA_EXT				0x25	/* 48 bit DMA read */
#define ATA_CMD_READ_DMA_QUEUED_EXT			0x26	/* 48 bit TCQ DMA read */
#define ATA_CMD_READ_FPDMA_QUEUED			0x60	/* NCQ DMA read: SATA only. Always 48 bit */

#define ATA_CMD_WRITE_DMA				0xCA	
#define ATA_CMD_WRITE_DMA_QUEUED			0xCC
#define ATA_CMD_WRITE_DMA_EXT  				0x35
#define ATA_CMD_WRITE_DMA_QUEUED_EXT			0x36
#define ATA_CMD_WRITE_FPDMA_QUEUED			0x61

/* Identify command */
#define ATA_CMD_IDENTIFY_ATA				0xEC
#define ATA_CMD_IDENTIFY_ATAPI				0xA1

#define ATA_CMD_VERIFY					0x40	/* 24 bit read verifty */
#define ATA_CMD_VERIFY_EXT				0x42	/* 48 bit read verify */

#define ATA_CMD_FLUSH					0xE7	/* 24 bit flush */
#define ATA_CMD_FLUSH_EXT				0xEA	/* 48 bit flush */

#define ATA_CMD_PACKET					0xA0
#define ATA_CMD_SMART					0xB0
#define ATA_CMD_SMART_READ_VALUES			0xD0
#define ATA_CMD_SMART_READ_THRESHOLDS			0xD1
#define ATA_CMD_SMART_ATTRIBUTE_AUTOSAVE		0xD2
#define ATA_CMD_SMART_SAVE				0xD3
#define ATA_CMD_SMART_IMMEDIATE_OFFLINE			0xD4
#define ATA_CMD_SMART_READ_LOG				0xD5
#define ATA_CMD_SMART_WRITE_LOG 			0xD6
#define ATA_CMD_SMART_WRITE_THRESHOLDS              	0xD7
#define ATA_CMD_SMART_ENABLE				0xD8
#define ATA_CMD_SMART_DISABLE				0xD9
#define ATA_CMD_SMART_RETURN_STATUS			0xDA
#define ATA_CMD_SMART_AUTO_OFFLINE			0xDB
#define ATA_CMD_SET_FEATURES				0xEF

#define ATA_FEATURE_ENABLE_WRITE_CACHE			0x02
#define ATA_FEATURE_SET_TRANSFER_MODE			0x03
#define ATA_FEATURE_PUP_STBY_SPIN_UP			0x07 /* this ata command will spin up sata disk */
#define ATA_FEATURE_DISABLE_READ_LOOK_AHEAD		0x55
#define ATA_FEATURE_DISABLE_WRITE_CACHE			0x82
#define ATA_FEATURE_ENABLE_READ_LOOK_AHEAD		0xAA
/*	security commmand */
#define ATA_CMD_SEC_PASSWORD				0xF1
#define ATA_CMD_SEC_UNLOCK				0xF2
#define ATA_CMD_SEC_ERASE_PRE				0xF3
#define ATA_CMD_SEC_ERASE_UNIT				0xF4
#define ATA_CMD_SEC_FREEZE_LOCK				0xF5
#define ATA_CMD_SEC_DISABLE_PASSWORD			0xF6


#define ATA_CMD_STANDBY_IMMEDIATE			0xE0
#define ATA_CMD_CHECK_POWER_MODE			0xE5
#define ATA_CMD_GET_MEDIA_STATUS			0xDA

#define ATA_CMD_SEEK					0x70
#define ATA_CMD_READ_LOG_EXT				0x2F

/* PM commands & registers */
#define ATA_CMD_PM_READ					0xE4
#define ATA_CMD_PM_WRITE				0xE8
#define ATA_CMD_PM_CHECK				0xE5

#define ATA_CMD_EXECUTE_DEVICE_DIAGNOSTIC		0x90
#define ATA_CMD_DOWNLOAD_MICROCODE			0x92
/* Some macros to make device type easier to use */
#define IS_SSP(pDevice)					((pDevice->Connection & DC_SCSI)  && !(pDevice->Connection & DC_ATA))
#define IS_STP(pDevice)					((pDevice->Connection & DC_SCSI)  && (pDevice->Connection & DC_ATA))
#define IS_NATIVE_SATA(pDevice)				(!(pDevice->Connection & DC_SCSI) && (pDevice->Connection & DC_ATA))
#define IS_STP_OR_SATA_ATAPI(pDevice)			(pDevice->Connection & DC_ATA)
#define IS_SATAPI(pDevice)				((pDevice->Connection & DC_SCSI) && (pDevice->Connection & DC_ATAPI) && (pDevice->Connection & DC_ATA))
#define IS_ATAPI(pDevice)				((pDevice->Connection & DC_ATAPI) && (pDevice->Connection & DC_ATA))
#define IS_SGPIO(pDevice)				(pDevice->Connection & DC_SGPIO)
#define IS_PM_SATA(pDevice)				((pDevice->pPM != NULL) && (pDevice->Connection & DC_ATA))
#define IS_DISK(pDevice)				(pDevice->Dev_Type == DT_DIRECT_ACCESS_BLOCK)
#define IS_TAPE(pDevice)				(pDevice->Dev_Type == DT_SEQ_ACCESS)
#define IS_MEDIA_CHANGER(pDevice)			(pDevice->Dev_Type == DT_MEDIA_CHANGER)
#define IS_ENCLOSURE(pDevice)				(pDevice->Dev_Type == DT_ENCLOSURE) || (pDevice->Dev_Type == DT_SES_DEVICE)
/* Device status */
#define DEVICE_STATUS_NO_DEVICE				ARCSAS_BIT(1, 0)
#define DEVICE_STATUS_EXISTING				ARCSAS_BIT(1, 1)
#define DEVICE_STATUS_FUNCTIONAL			ARCSAS_BIT(1, 2)
#define DEVICE_STATUS_SHOWED_ONCE			ARCSAS_BIT(1, 3)
#define DEVICE_STATE_REPORT2_MLDRIVER			ARCSAS_BIT(1, 4)
#define DEVICE_STATUS_CHECK_CONDITION			ARCSAS_BIT(1, 5)
#define DEVICE_STATUS_VALID_SLOT_NUMBER			ARCSAS_BIT(1, 6)
#define DEVICE_STATUS_IO_LOCK_UNLOCK			ARCSAS_BIT(1, 7)
/* Device Setting */
#define DEVICE_SETTING_SMART_ENABLED			ARCSAS_BIT(1, 0)
#define DEVICE_SETTING_WRITECACHE_ENABLED		ARCSAS_BIT(1, 1)
#define DEVICE_SETTING_PROTECT_INFO_ENABLED		ARCSAS_BIT(1, 2)
#define DEVICE_SETTING_READ_LOOK_AHEAD			ARCSAS_BIT(1, 3)
#define DEVICE_SETTING_WRITE_PROTECT			ARCSAS_BIT(1, 4)
#define DEVICE_SETTING_NCQ_SUPPORTED			ARCSAS_BIT(1, 5)
#define DEVICE_SETTING_MAX_LINK_SPEED			(ARCSAS_BIT(1, 10) | ARCSAS_BIT(1, 9) | ARCSAS_BIT(1, 8)) /* came from BIOS_Flag */

#define Get_Ports_EntryIndex(pCore, ID)			(pCore->Ports_EntryIndex[ID] )
/*To idnetify the EntryIndex belongs to which port, which could be 0 ~7.*/
#define Get_Port_Index(pACB, ID)			((Get_DevicesExpandersPMs_EntryIndex(pACB, ID) == ID_NOT_MAPPED) ? ID_NOT_MAPPED : (((ID & 0xff) < ARCSAS_MIN_EXPANDER_ID) ? (pACB->Devices[pACB->DevicesExpandersPMs_EntryIndex[(ID & 0xff)]].pPort->Port_Index) : (((ID & 0xff) < ARCSAS_MIN_PM_ID) ? (pACB->Expanders[pACB->DevicesExpandersPMs_EntryIndex[ID]].pPort->Port_Index) : (pACB->PMs[pACB->DevicesExpandersPMs_EntryIndex[ID]].pPort->Port_Index))))
#define Get_DevicesExpandersPMs_EntryIndex(pACB, entry_index)	(((entry_index & 0xff) <= ARCSAS_MAX_PM_ID) ? pACB->DevicesExpandersPMs_EntryIndex[(entry_index & 0xff)] : ID_NOT_MAPPED)

#define MODE_DSP_FUA_SUPPORTED            0x10
#define MODE_DSP_WRITE_PROTECT            0x80

/*
************************************************************************
64xx chip registers
************************************************************************
*/

#define ARCSAS_PCI_BAR                         	5
#define GEN_HOST_CONTROL_REG_START		0x0
#define VENDOR_SPECIFIC_REG_START		0xa0
#define SAS_PORT_REG_START			0x100

/* Power Mode PM_CNTRL/PMODE */
#define PMODE_PARTIAL				0x10
#define PMODE_SLUMBER				0x01

/* CONFIG_R_ERR_COUNT/CONFIG_CRC_ERR_COUNT bits */
#define ERR_COUNT_MASK				0x0ffff

/* SSP_FrameType */
#define SSP_FRAME_TYPE_COMMAND			0x00
#define SSP_FRAME_TYPE_TASK			0x01
#define SSP_FRAME_TYPE_XFER_RDY			0x04 /* Target mode */
#define SSP_FRAME_TYPE_RESPONSE			0x05 /* Target mode */
#define SSP_FRAME_TYPE_RD_DATA			0x06 /* Target mode */
#define SSP_FRAME_TYPE_RD_DATA_RESPONSE		0x07 /* Target mode, read data after response frame */
/* Tag */
#define SSP_I_HEADR_TAG_MASK			0xfe00   /* lower 9 bits from HW command slot */
#define SSP_T_HEADR_TAG_MASK			0xffff
#define STP_HEADR_TAG_MASK			0x001f   /* NCQ uses lower 5 bits  */
/* TargetTag */
#define SSP_T_HEADR_TGTTAG_MASK			0xfe00   /* lower 9 bits from HW command slot */
/*
************************************************************************
**          Task Management Functions.
************************************************************************
*/
#define ABORT_TASK_TMF				0x01
#define ABORT_TASK_SET_TMF			0x02
#define CLEAR_TASK_SET_TMF			0x04
#define LOGICAL_UNIT_RESET_TMF			0x08
#define CLEAR_ACA_TMF				0x40
#define QUERY_TASK_TMF				0x80


/* DataPres */
#define NO_SENSE_RESPONSE			0x0
#define RESPONSE_ONLY				0x1
#define SENSE_ONLY				0x2
#define RESERVED				0x3

#define CORE_CONTEXT_TYPE_NONE		0		/* No valid context */
#define CORE_CONTEXT_TYPE_CDB		1		/* Context is the CDB */
#define CORE_CONTEXT_TYPE_BIG_REQUEST	2		/* Context for large request */

/*
 * Here is the definition for the command consolidate sub module
 * This is only changed when we modify the consolidate algorithm.
 */
//#define CONS_MAX_INTERNAL_REQUEST_COUNT	CORE_MAX_REQUEST_NUMBER

#define CONS_MAX_EXTERNAL_REQUEST_SIZE		(1024 * 128)
#define CONS_SEQUENTIAL_MAX			0x7FFF		/* Avoid overflow. It's determined by Sequential variable size */
#define CONS_SEQUENTIAL_THRESHOLD		64

#define CONS_MAX_INTERNAL_REQUEST_SIZE	(1024 * 128)	/* The maximum request size hardware can handle. */
#define CONS_MIN_INTERNAL_REQUEST_SIZE	(1024 * 128)	/* We'll accumulate the request to this size and then fire. */

/*Work around free disks suporting temporarily*/
#define RESTORE_FREE_DISK_TID			1

/*
 * Command flag is the flag for the CDB command itself 
 */
/* The first 16 bit can be determined by the initiator. */
#define CMD_FLAG_NON_DATA			ARCSAS_BIT(1, 0)	/*1-non data; 
0-data command */
#define CMD_FLAG_DMA				ARCSAS_BIT(1, 1)	/* 1-DMA */
#define CMD_FLAG_PIO				ARCSAS_BIT(1, 2)	/* 1-PIO */
#define CMD_FLAG_DATA_IN			ARCSAS_BIT(1, 3)	/* 1-host read data */
#define CMD_FLAG_DATA_OUT			ARCSAS_BIT(1, 4)	/* 1-host write data */
#define CMD_FLAG_SMART				ARCSAS_BIT(1, 5)	/* 1-SMART command; 
0-non SMART command*/
#define CMD_FLAG_SMART_ATA_12			ARCSAS_BIT(1, 6)	/* SMART ATA_12  */
#define CMD_FLAG_SMART_ATA_16			ARCSAS_BIT(1, 7)	/* SMART ATA_16; */
#define CMD_FLAG_SMART_COMMAND_TRANSPORT	ARCSAS_BIT(1, 8)
/*
 * The last 16 bit only can be set by the target. Only core driver knows the device characteristic. 
 */
#define CMD_FLAG_NCQ				ARCSAS_BIT(1, 16)
#define CMD_FLAG_TCQ				ARCSAS_BIT(1, 17)
#define CMD_FLAG_48BIT				ARCSAS_BIT(1, 18)
#define CMD_FLAG_PACKET				ARCSAS_BIT(1, 19)	/* ATAPI packet cmd */
#define CMD_FLAG_SCSI_PASS_THRU			ARCSAS_BIT(1, 20)
#define CMD_FLAG_ATA_PASS_THRU			ARCSAS_BIT(1, 21)

#define ARCSAS_MAX(x,y)				(((x) > (y)) ? (x) : (y))
#define ARCSAS_MIN(x,y)				(((x) < (y)) ? (x) : (y))

#define ARCSAS_MAX_U64(x, y)			((((x).value) > ((y).value)) ? (x) : (y))
#define ARCSAS_MIN_U64(x, y)			((((x).value) < ((y).value)) ? (x) : (y))
#define ARCSAS_ALIGN(x, a)			(((x) + (a) - 1) & ~((a) - 1))
#define U64_ASSIGN_VALUE(x,y)			((x).value = (y))
#define U64_ASSIGN_U64(x,y)			((x).value = (y).value)
#define U64_COMPARE_U64(x,y)			((x) == (y).value)
#define U64_COMPARE_U64_VALUE(x,y)		((x).value == (y).value)
#define U32_ASSIGN_U64(v64, v32)		((v64).value = (v32))
#define U64_SHIFT_LEFT(v64, v32)		((v64).value = (v64).value << (v32))
#define U64_SHIFT_RIGHT(v64, v32)		((v64).value = (v64).value >> (v32))
#define U64_ZERO_VALUE(v64)			((v64).value = 0)

#define FILO_TAG			0x00
#define FIFO_TAG			0x01

#define SERVICE_ACTION_IN	0x9e
/* values for service action in */
#define SAI_READ_CAPACITY_16	0x10
#define SAI_GET_LBA_STATUS	0x12
#define SAI_REPORT_REFERRALS	0x13

#define SGD_DOMAIN_MASK	0xF0000000L
#define SGD_EOT			(1L << 27)	/* End of table */
#define SGD_COMPACT		(1L << 26)	/* Compact (12 bytes) SG format, not verified yet */
#define SGD_WIDE		(1L << 25)	/* 32 byte SG format */
#define SGD_X64			(1L << 24)	/* the 2nd part of SGD_WIDE */
#define SGD_NEXT_TBL		(1L << 23)	/* Next SG table format */
#define SGD_VIRTUAL		(1L << 22)	/* Virtual SG format, either 32 or 64 bit is determined during compile time. */
#define SGD_REFTBL		(1L << 21)	/* sg table reference format, either 32 or 64 bit is determined during compile time. */
#define SGD_REFSGD		(1L << 20)	/* sg item reference format */
#define SGD_VP			(1L << 19)	/* virtual and physical, not verified yet */
#define SGD_VWOXCTX		(1L << 18)	/* virtual without translation context */
#define SGD_PCTX		(1L << 17)	/* sgd_pctx_t, 64 bit only */

#define SIZEOF_COMPACT_SGD	12
/*Inquiry defines. Used to interpret data returned from target as result of inquiry command.*/
#define LOGICAL_UNIT_NOT_PRESENT_DEVICE		0x0C /* Odin Linux defines it as 0x0C, Windows define it as 0x7C */

#define SCSI_TAPE_READ_BIT_SILI		ARCSAS_BIT(1, 1)

/*
 * SCSI command
 */
#define SCSI_CMD_INQUIRY				0x12
#define SCSI_CMD_START_STOP_UNIT			0x1B
#define SCSI_CMD_TEST_UNIT_READY			0x00
#define SCSI_CMD_REZERO_UNIT				0x01
#define SCSI_CMD_RESERVE_6				0x16
#define SCSI_CMD_RELEASE_6				0x17
#define SCSI_CMD_WRITE_FILEMARKS			0x10
#define SCSI_CMD_SPACE					0x11
#define SCSI_CMD_READ_6					0x08
#define SCSI_CMD_READ_10				0x28
#define SCSI_CMD_READ_12				0xA8
#define SCSI_CMD_READ_16				0x88
#define SCSI_CMD_READ_LONG_10				0x3E
#define SCSI_CMD_WRITE_6				0x0A
#define SCSI_CMD_WRITE_10				0x2A
#define SCSI_CMD_WRITE_12				0xAA
#define SCSI_CMD_WRITE_16				0x8A
#define SCSI_CMD_WRITE_LONG_10				0x3F
#define SCSI_CMD_READ_CAPACITY_10			0x25
#define SCSI_CMD_SERVICE_ACTION_IN			0x9E    /* 9Eh/10h */
#define SCSI_CMD_VERIFY_10				0x2F
#define SCSI_CMD_VERIFY_12				0xAF
#define SCSI_CMD_VERIFY_16				0x8F
#define SCSI_CMD_REQUEST_SENSE				0x03
#define SCSI_CMD_MODE_SENSE_6				0x1A
#define SCSI_CMD_MODE_SENSE_10				0x5A
#define SCSI_CMD_MODE_SELECT_6				0x15
#define SCSI_CMD_MODE_SELECT_10				0x55
#define SCSI_CMD_LOG_SELECT				0x4C
#define SCSI_CMD_LOG_SENSE				0x4D
#define SCSI_CMD_WRITE_VERIFY_10			0x2E
#define SCSI_CMD_WRITE_VERIFY_12			0xAE
#define SCSI_CMD_WRITE_VERIFY_16			0x8E
#define SCSI_CMD_SYNCHRONIZE_CACHE_10			0x35
#define SCSI_CMD_SYNCHRONIZE_CACHE_16			0x91
#define SCSI_CMD_WRITE_SAME_10				0x41
#define SCSI_CMD_UNMAP					0x42
#define SCSI_CMD_WRITE_SAME_16				0x93
#define SCSI_CMD_XDWRITE_10				0x50
#define SCSI_CMD_XPWRITE_10				0x51
#define SCSI_CMD_XDREAD_10				0x52
#define SCSI_CMD_XDWRITEREAD_10				0x53
#define SCSI_CMD_FORMAT_UNIT				0x04
#define SCSI_CMD_RCV_DIAG_RSLT				0x1C
#define SCSI_CMD_SND_DIAG				0x1D
#define SCSI_CMD_PERSISTENT_RESERVE_IN			0x5E
#define SCSI_CMD_PERSISTENT_RESERVE_OUT			0x5F
#define SCSI_CMD_ATA_PASSTHRU_12			0xA1
#define SCSI_CMD_ATA_PASSTHRU_16			0x85
#define SCSI_CMD_READ_BUFFER				0x3C
#define SCSI_CMD_WRITE_BUFFER				0x3B
#define SCSI_CMD_READ_BLOCK_LIMITS 			0x05

/* MMC */
#define SCSI_CMD_REPORT_LUNS				0xA0
#define SCSI_CMD_PREVENT_MEDIUM_REMOVAL			0x1E
#define SCSI_CMD_READ_SUB_CHANNEL			0x42
#define SCSI_CMD_READ_TOC				0x43
#define SCSI_CMD_READ_DISC_STRUCTURE			0xAD
#define SCSI_CMD_READ_CD				0xBE
#define SCSI_CMD_GET_EVENT_STATUS_NOTIFICATION		0x4A
#define SCSI_CMD_SECURITY_PROTOCOL_IN			0xA2
#define SCSI_CMD_SECURITY_PROTOCOL_OUT			0xB5
#define SCSI_CMD_READ_DISC_INFO				0x51
#define SCSI_CMD_ARECA_SPECIFIC				0xE1
#define SCSI_CMD_READ_DEFECT_DATA_10			0x37
#define SCSI_CMD_READ_DEFECT_DATA_12			0xb7
#define SCSI_CMD_VARIABLE_LENGTH_CMD			0x7f

#define CDB_CORE_ATA_SLEEP				0x20
#define CDB_CORE_ATA_IDLE				0x21
#define CDB_CORE_ATA_STANDBY				0x22
#define CDB_CORE_ATA_IDLE_IMMEDIATE			0x23
#define CDB_CORE_ATA_STANDBY_IMMEDIATE			0x24
#define CDB_CORE_ATA_CHECK_POWER_MODE			0x25

#define CDB_CORE_MODULE					0x1
#define CDB_CORE_SOFT_RESET_1				0x1
#define CDB_CORE_SOFT_RESET_0				0x2
#define CDB_CORE_IDENTIFY				0x3
#define CDB_CORE_SET_UDMA_MODE				0x4
#define CDB_CORE_SET_PIO_MODE				0x5
#define CDB_CORE_ENABLE_WRITE_CACHE			0x6
#define CDB_CORE_DISABLE_WRITE_CACHE			0x7
#define CDB_CORE_ENABLE_SMART				0x8
#define CDB_CORE_DISABLE_SMART				0x9
#define CDB_CORE_SMART_RETURN_STATUS			0xA
#define CDB_CORE_SHUTDOWN				0xB
#define CDB_CORE_ENABLE_READ_AHEAD			0xC
#define CDB_CORE_DISABLE_READ_AHEAD			0xD
#define CDB_CORE_READ_LOG_EXT				0xE
#define CDB_CORE_TASK_MGMT				0xF
#define CDB_CORE_SMP					0x10
#define CDB_CORE_PM_READ_REG				0x11
#define CDB_CORE_PM_WRITE_REG				0x12
#define CDB_CORE_RESET_DEVICE				0x13
#define CDB_CORE_RESET_PORT				0x14
#define CDB_CORE_OS_SMART_CMD				0x15
#define CDB_CORE_ATA_IDENTIFY_ATA			0x16
#define CDB_CORE_ATA_IDENTIFY_ATAPI			0x17
#define CDB_CORE_ATA_SMART_READ_VALUES			0x18
#define CDB_CORE_ATA_SMART_READ_THRESHOLDS		0x19
#define CDB_CORE_ATA_SMART_READ_LOG			0x1A
#define CDB_CORE_ATA_SMART_WRITE_LOG			0x1B
#define CDB_CORE_ATA_SMART_AUTO_OFFLINE			0x1C
#define CDB_CORE_ATA_SMART_AUTOSAVE			0x1D
#define CDB_CORE_ATA_SMART_IMMEDIATE_OFFLINE		0x1E
#define CDB_CORE_SMP_VIRTUAL_DISCOVER			0x30
#define CDB_CORE_SMP_VIRTUAL_CONFIG_ROUTE		0x31
#define CDB_CORE_STP_VIRTUAL_PHY_RESET			0x32
#define CDB_CORE_STP_VIRTUAL_REPORT_SATA_PHY		0x33
#define CDB_CORE_SSP_VIRTUAL_PHY_RESET			0x34
#define CDB_CORE_SSP_VIRTUAL_REPORT_SATA_PHY		0x35
#define CDB_CORE_SMP_VIRTUAL_RESET_SATA_PHY		0x36
#define CDB_CORE_SMP_VIRTUAL_CLEAR_AFFILIATION_ALL	0x37
#define CDB_CORE_SET_FEATURE_SPINUP			0x40
#define SMP_CDB_USE_ADDRESS				0x01

/*
************************************************************************
** Log Sense Scsi command translation
************************************************************************
*/
/* ANSI SCSI-3 Log Pages retrieved by LOG SENSE. */
#define SUPPORTED_LOG_PAGES_LOG_PAGE				0x00
#define BUFFER_OVERRUN_LPAGE					0x01
#define WRITE_ERROR_COUNTER_LOG_PAGE				0x02
#define READ_ERROR_COUNTER_LOG_PAGE				0x03
#define READ_REVERSE_ERROR_COUNTER_LOG_PAGE			0x04
#define VERIFY_ERROR_COUNTER_LOG_PAGE				0x05
#define NON_MEDIUM_ERROR_LOG_PAGE				0x06
#define LAST_N_ERROR_LOG_PAGE					0x07
#define FORMAT_STATUS_LOG_PAGE					0x08
#define TEMPERATURE_LOG_PAGE					0x0d
#define STARTSTOP_CYCLE_COUNTER_LOG_PAGE			0x0e
#define APPLICATION_CLIENT_LOG_PAGE				0x0f
#define SELF_TEST_RESULTS_LOG_PAGE				0x10
#define BACKGROUND_RESULTS_LOG_PAGE				0x15   /* SBC-3 */
#define INFORMATIONAL_EXCEPTIONS_LOG_PAGE 			0x2f
#define VENDOR_SPECIFIC_LOG_PAGE				0x31
/*
 * ATA status
 */
#define ATA_S_OK							0x00 /* ok */
#define ATA_S_ERROR						0x01 /* error */
#define ATA_S_INDEX						0x02 /* index */
#define ATA_S_CORR						0x04 /* data corrected */
#define ATA_S_DRQ						0x08 /* data request */
#define ATA_S_DSC						0x10 /* drive seek completed */
#define ATA_S_SERVICE						0x10 /* drive needs service */
#define ATA_S_DWF						0x20 /* drive write fault */
#define ATA_S_DMA						0x20 /* DMA ready */
#define ATA_S_READY						0x40 /* drive ready */
#define ATA_S_BUSY						0x80 /* busy */
/* SCSI status */
#define SCSI_STATUS_GOOD					0x00
#define SCSI_STATUS_CHECK_CONDITION				0x02
#define SCSI_STATUS_CONDITION_MET				0x04
#define SCSI_STATUS_BUSY					0x08
#define SCSI_STATUS_INTERMEDIATE				0x10
#define SCSI_STATUS_INTERMEDIATE_MET				0x14
#define SCSI_STATUS_RESERVATION_CONFLICT			0x18
#define SCSI_STATUS_FULL					0x28
#define SCSI_STATUS_ACA_ACTIVE					0x30
#define SCSI_STATUS_ABORTED					0x40
/* SCSI sense key */
#define SCSI_SK_NO_SENSE					0x00
#define SCSI_SK_RECOVERED_ERROR					0x01
#define SCSI_SK_NOT_READY					0x02
#define SCSI_SK_MEDIUM_ERROR					0x03
#define SCSI_SK_HARDWARE_ERROR					0x04
#define SCSI_SK_ILLEGAL_REQUEST					0x05
#define SCSI_SK_UNIT_ATTENTION					0x06
#define SCSI_SK_DATA_PROTECT					0x07 
#define SCSI_SK_BLANK_CHECK					0x08
#define SCSI_SK_VENDOR_SPECIFIC					0x09
#define SCSI_SK_COPY_ABORTED					0x0A
#define SCSI_SK_ABORTED_COMMAND					0x0B
#define SCSI_SK_VOLUME_OVERFLOW					0x0D
#define SCSI_SK_MISCOMPARE					0x0E
/* SCSI additional sense code */
#define SCSI_ASC_NO_ASC					0x00
#define SCSI_ASC_LUN_NOT_READY				0x04
#define SCSI_ASC_LOGICAL_UNIT_NOT_RESP_TO_SEL		0x05
#define SCSI_ASC_ECC_ERROR				0x10
#define SCSI_ASC_ID_ADDR_MARK_NOT_FOUND			0x12
#define SCSI_ASC_RECORD_NOT_FOUND			0x14
#define SCSI_ASC_PARAMETER_LIST_LENGTH_ERROR		0x1A
#define SCSI_ASC_MISCOMPARE_DURING_VERIFY		0x1D
#define SCSI_ASC_INVALID_OPCODE				0x20
#define SCSI_ASC_LBA_OUT_OF_RANGE			0x21
#define SCSI_ASC_INVALID_FEILD_IN_CDB			0x24
#define SCSI_ASC_LOGICAL_UNIT_NOT_SUPPORTED		0x25
#define SCSI_ASC_INVALID_FIELD_IN_PARAMETER		0x26
#define SCSI_ASC_WRITE_PROTECTED			0x27
#define SCSI_ASC_MEDIA_CHANGED				0x28
#define SCSI_ASC_CMD_SEQUENCE_ERROR			0x2C
#define SCSI_ASC_MEDIUM_FORMAT_CORRUPTED		0x31
#define SCSI_ASC_SAVING_PARAMETERS_NOT_SUPPORT		0x39
#define SCSI_ASC_MEDIUM_NOT_PRESENT			0x3A
#define SCSI_ASC_LOGICAL_UNIT_FAILURE			0x3E
#define SCSI_ASC_INTERNAL_TARGET_FAILURE		0x44
#define SCSI_ASC_SCSI_PARITY_ERROR			0x47
#define SCSI_ASC_MEDIA_LOAD_EJECT_FAILURE		0x53
#define SCSI_ASC_SYSTEM_RESOURCE_FAILURE		0x55
#define SCSI_ASC_OPERATOR_MEDIUM_REMOVAL_REQUEST	0x5A
#define SCSI_ASC_FAILURE_PREDICTION_THRESHOLD_EXCEEDED	0x5D
#define SCSI_ASC_CONFIGURATION_FAILURE			0x67
#define SCSI_ADSENSE_NO_SENSE					0x98 //TBD: Definition: Don't use windows
#define SCSI_ADSENSE_INVALID_CDB				0x99 //TBD: Definition: Don't use windows
/* SCSI additional sense code qualifier */
#define SCSI_ASCQ_NO_ASCQ					0x00
#define SCSI_ASCQ_FILEMARK_DETECTED				0x01
#define SCSI_ASCQ_INSUFFICIENT_RESERVATION_RESOURCES		0x02
#define SCSI_ASCQ_INTERVENTION_REQUIRED				0x03
#define SCSI_ASCQ_FORMAT_IN_PROGRESS				0x04
#define SCSI_ASCQ_EOD_DETECTED					0x05
#define SCSI_ASCQ_MAINTENANCE_IN_PROGRESS			0x80
#define SCSI_ASCQ_HIF_GENERAL_HD_FAILURE			0x10
#define SCSI_ASCQ_ATA_PASSTHRU_INFO				0x1D
#define SCSI_ASCQ_INSUFFICIENT_RESERVATION_RESOURCES		0x02
/* SCSI sense ResponseCode */
#define ARCSAS_SCSI_RESPONSE_CODE				0x70
#define ARCSAS_SCSI_DIRECT_ACCESS_DEVICE			0x00
/* Virtual Device Inquiry Related */
#define VIRTUALD_INQUIRY_DATA_SIZE			36
#define VPD_PAGE0_VIRTUALD_SIZE				7
#define VPD_PAGE80_VIRTUALD_SIZE			12
#define VPD_PAGE83_VIRTUALD_SIZE			24
/* HBA_FLAG_XX */
#define BIOS_FLAG_INT13_ENABLE				ARCSAS_BIT(1, 0)	//int 13h enable/disable
#define BIOS_FLAG_SILENT_MODE_ENABLE			ARCSAS_BIT(1, 1)	//silent mode enable/disable
#define BIOS_FLAG_PERSISTENT_MAP_ENABLE  		ARCSAS_BIT(1, 2)	//flash device map function
#define BIOS_FLAG_PERSISTENT_MAP_CLEAR			ARCSAS_BIT(1, 3)	//clear device persistent map
#define BIOS_FLAG_ALL_DISKS_WRITE_PROTECT		ARCSAS_BIT(1, 4)	//all disks write protect
#define BIOS_FLAG_SATA_NCQ_ENABLE			ARCSAS_BIT(1, 5)	//SATA drive NCQ enable/disable
#define BIOS_FLAG_SMART_STATUS_POLL_ENABLE		ARCSAS_BIT(1, 6)	//SMART status polling enable/disable
#define BIOS_FLAG_DISK_WRITE_CACHE_ENABLE		ARCSAS_BIT(1, 7)	//disk write cache enable/disable
#define BIOS_FLAG_MPL_AUTO				0		//maximum phy link rate = auto - no bit set
#define BIOS_FLAG_MPL_15G				ARCSAS_BIT(1, 8)	//maximum phy link rate = 1.5G - bit8
#define BIOS_FLAG_MPL_30G				ARCSAS_BIT(1, 9)	//maximum phy link rate = 3.0G - bit9
#define BIOS_FLAG_MPL_60G				(ARCSAS_BIT(1, 8) | ARCSAS_BIT(1, 9))	// maximum phy link rate = 6.0G - bit8|9
#define BIOS_FLAG_DISK_MAX_LINK_SPEED			(ARCSAS_BIT(1, 10) | ARCSAS_BIT(1, 9) | ARCSAS_BIT(1, 8))  
#define BIOS_FLAG_MAX_CMDXFER_LENGTH			(ARCSAS_BIT(1, 13) | ARCSAS_BIT(1, 12) | ARCSAS_BIT(1, 11))              

#define BIOS_FLAG_MCXL_1M				0		// maximum command xfer length = 1MB - no bit set
#define BIOS_FLAG_MCXL_4M				ARCSAS_BIT(1, 11)	// maximum command xfer length = 4MB - bit11

#define DEFAULT_PARAMETERS				(BIOS_FLAG_INT13_ENABLE | BIOS_FLAG_SATA_NCQ_ENABLE | BIOS_FLAG_DISK_WRITE_CACHE_ENABLE |BIOS_FLAG_MPL_AUTO |BIOS_FLAG_MCXL_4M)


#define NVRAM_DATA_MAJOR_VERSION			0
#define NVRAM_DATA_MINOR_VERSION			1

#define ARCSAS_FLASH_ROM_SIZE				0x40000			/* rom region 0x00000....0x40000 256k bytes */
#define ARCSAS_FLASH_SECTOR_SIZE			(64L * 1024)		/* 64K,0x10000 */
#define ARCSAS_FLASH_BIOS_PARAM_SIZE			(sizeof(BIOS_Info_Page))	/* 256 bytes */
#define ARCSAS_FLASH_DEVICES_MAP_SIZE			(sizeof(ARCSAS_U64) * ARCSAS_DISK_SUPPORTED)	/* 1K */

/* rom region 0x20000....0x20400 (1K), 8 bytes pattern for each device 8*128 = 1024 */
#define ARCSAS_FLASH_DEVICES_MAP_OFFSET     		ARCSAS_FLASH_ROM_SIZE - 0x20000
 /* rom region 0x20500....0x21500 (4K), 32 bytes one event, 128 events 32*128 = 4096 */
#define ARCSAS_FLASH_HBA_EVENT_OFFSET       		ARCSAS_FLASH_ROM_SIZE - 0x1FB00
 /* rom region 0x3FF00....0x40000 last 256 bytes */
#define ARCSAS_FLASH_BIOS_PARAM_OFFSET      		ARCSAS_FLASH_ROM_SIZE - 0x00100 

#define DRIVER_STATUS_IDLE				1    /* The first status */
#define DRIVER_STATUS_STARTING				2    /* Begin to start all modules */
#define DRIVER_STATUS_STARTED				3    /* All modules are all settled. */
#define HBA_REQ_TIMER_AFTER_RESET			20
#define HBA_REQ_TIMER					10
#define HBA_REQ_TIMER_IOCTL				(HBA_REQ_TIMER_AFTER_RESET + 3)
#define MSG_QUEUE_DEPTH					32
#define MAX_PASS_THRU_DATA_BUFFER_SIZE			(ARCSAS_SECTOR_SIZE + 128)
#define BASE_ADDRESS_MAX_NUM				6
#define SUPPORT_LD_MODE_RAID0				ARCSAS_BIT(1, 0)
#define SUPPORT_LD_MODE_RAID1				ARCSAS_BIT(1, 1)
#define SUPPORT_LD_MODE_RAID10				ARCSAS_BIT(1, 2)
#define SUPPORT_LD_MODE_RAID1E				ARCSAS_BIT(1, 3)
#define SUPPORT_LD_MODE_RAID5				ARCSAS_BIT(1, 4)
#define SUPPORT_LD_MODE_RAID6				ARCSAS_BIT(1, 5)
#define SUPPORT_LD_MODE_RAID50				ARCSAS_BIT(1, 6)
#define SUPPORT_LD_MODE_JBOD				ARCSAS_BIT(1, 7)
#define FEATURE_BGA_MEDIAPATROL_SUPPORT			ARCSAS_BIT(1, 4)

/* PTR_INTEGER is necessary to convert between pointer and integer. */

#define ERR_GENERIC					2
#define ERR_RAID					50
#define ERR_CORE					100
#define ERR_API						150
#define ERR_NONE					0
#define ERR_FAIL					1
/* generic error */
#define ERR_UNKNOWN					(ERR_GENERIC + 1)
#define ERR_NO_RESOURCE					(ERR_GENERIC + 2)
#define ERR_REQ_OUT_OF_RANGE				(ERR_GENERIC + 3)
#define ERR_INVALID_REQUEST				(ERR_GENERIC + 4)
#define ERR_INVALID_PARAMETER				(ERR_GENERIC + 5)
#define ERR_INVALID_LOGICAL_DISK_ID			(ERR_GENERIC + 6)
#define ERR_INVALID_PHYSICAL_DISK_ID			(ERR_GENERIC + 7)
#define ERR_INVALID_EXP_ID				(ERR_GENERIC + 8)
#define ERR_INVALID_PM_ID				(ERR_GENERIC + 9)
#define ERR_INVALID_BLOCK_ID				(ERR_GENERIC + 10)
#define ERR_INVALID_ADAPTER_ID				(ERR_GENERIC + 11)
#define ERR_INVALID_RAID_MODE				(ERR_GENERIC + 12)
#define ERR_INVALID_ENC_ID				(ERR_GENERIC + 13)
#define ERR_INVALID_BU_ID				(ERR_GENERIC + 14)
#define ERR_INVALID_DG_ID				(ERR_GENERIC + 15)
#define ERR_INVALID_ENC_ELEMENT_ID			(ERR_GENERIC + 16)
#define ERR_NOT_SUPPORTED				(ERR_GENERIC + 17)
#define ERR_DRIVER_SENSOR				(ERR_GENERIC + 18)
/* RAID errors */
#define ERR_TARGET_IN_LD_FUNCTIONAL			(ERR_RAID + 1)
#define ERR_TARGET_NO_ENOUGH_SPACE			(ERR_RAID + 2)
#define ERR_HD_IS_NOT_SPARE				(ERR_RAID + 3)
#define ERR_HD_IS_SPARE					(ERR_RAID + 4)
#define ERR_HD_NOT_EXIST				(ERR_RAID + 5)
#define ERR_HD_IS_ASSIGNED_ALREADY			(ERR_RAID + 6)
#define ERR_INVALID_HD_COUNT				(ERR_RAID + 7)
#define ERR_LD_NOT_READY				(ERR_RAID + 8)
#define ERR_LD_NOT_EXIST				(ERR_RAID + 9)
#define ERR_LD_IS_FUNCTIONAL				(ERR_RAID + 10)
#define ERR_HAS_BGA_ACTIVITY				(ERR_RAID + 11)
#define ERR_NO_BGA_ACTIVITY				(ERR_RAID + 12)
#define ERR_BGA_RUNNING 				(ERR_RAID + 13)
#define ERR_RAID_NO_AVAILABLE_ID			(ERR_RAID + 14)
#define ERR_LD_NO_ATAPI					(ERR_RAID + 15)
#define ERR_INVALID_RAID6_PARITY_DISK_COUNT		(ERR_RAID + 16)
#define ERR_INVALID_BLOCK_SIZE				(ERR_RAID + 17)
#define ERR_MIGRATION_NOT_NEED				(ERR_RAID + 18)
#define ERR_STRIPE_BLOCK_SIZE_MISMATCH			(ERR_RAID + 19)
#define ERR_MIGRATION_NOT_SUPPORT			(ERR_RAID + 20)
#define ERR_LD_NOT_FULLY_INITED				(ERR_RAID + 21)
#define ERR_LD_NAME_INVALID				(ERR_RAID + 22)

/* API errors */
#define ERR_INVALID_MATCH_ID				(ERR_API + 1)    
#define ERR_INVALID_HDCOUNT				(ERR_API + 2)
#define ERR_INVALID_BGA_ACTION				(ERR_API + 3)
#define ERR_HD_IN_DIFF_CARD				(ERR_API + 4)
#define ERR_INVALID_FLASH_TYPE				(ERR_API + 5)
#define ERR_INVALID_FLASH_ACTION			(ERR_API + 6)
#define ERR_TOO_FEW_EVENT				(ERR_API + 7)
#define ERR_VD_HAS_RUNNING_OS				(ERR_API + 8)
#define ERR_DISK_HAS_RUNNING_OS				(ERR_API + 9)
#define ERR_COMMAND_NOT_SUPPURT				(ERR_API + 10)

//The Flags of Eroor Handling
#define EH_READ_VERIFY_REQ_ERROR			ARCSAS_BIT(1, 0)
#define EH_WRITE_REQ_ERROR				ARCSAS_BIT(1, 1)
#define EH_WRITE_RECOVERY_ERROR				ARCSAS_BIT(1, 2)
#define EH_MEDIA_ERROR					ARCSAS_BIT(1, 3)
#define EH_TIMEOUT_ERROR				ARCSAS_BIT(1, 4)

#define CONSISTENCYCHECK_ONLY				0
#define CONSISTENCYCHECK_FIX				1

#define INIT_QUICK					0    //Just initialize first part size of LD
#define INIT_FULLFOREGROUND				1    //Initialize full LD size
#define INIT_FULLBACKGROUND				2    //Initialize full LD size background
#define INIT_NONE					3

#define INIT_QUICK_WITHOUT_EVENT			0xf	 // Used for QUICK INIT but set cdb[5]to 0xf so driver won't send event. 
	
#define BGA_CONTROL_START				0
#define BGA_CONTROL_RESTART				1
#define BGA_CONTROL_PAUSE				2
#define BGA_CONTROL_RESUME				3
#define BGA_CONTROL_ABORT				4
#define BGA_CONTROL_COMPLETE				5
#define BGA_CONTROL_IN_PROCESS				6
#define BGA_CONTROL_TERMINATE_IMMEDIATE			7
#define BGA_CONTROL_AUTO_PAUSE				8

#define DEVICE_TYPE_NONE				0
#define DEVICE_TYPE_HD					1	//  DT_DIRECT_ACCESS_BLOCK
#define DEVICE_TYPE_PM					2
#define DEVICE_TYPE_EXPANDER				3	// DT_EXPANDER
#define DEVICE_TYPE_TAPE				4	// DT_SEQ_ACCESS
#define DEVICE_TYPE_PRINTER				5	// DT_PRINTER
#define DEVICE_TYPE_PROCESSO				6	// DT_PROCESSOR
#define DEVICE_TYPE_WRITE_ONCE				7 	// DT_WRITE_ONCE
#define DEVICE_TYPE_CD_DVD				8	// DT_CD_DVD
#define DEVICE_TYPE_OPTICAL_MEMORY			9 	// DT_OPTICAL_MEMORY
#define DEVICE_TYPE_MEDIA_CHANGER			10	// DT_MEDIA_CHANGER
#define DEVICE_TYPE_ENCLOSURE				11	// DT_ENCLOSURE
#define DEVICE_TYPE_PORT				0xFF	// DT_STORAGE_ARRAY_CTRL

#define ARCSAS_LOGICAL_DISK_STATUS_FUNCTIONAL		0
#define ARCSAS_LOGICAL_DISK_STATUS_DEGRADE		1
#define ARCSAS_LOGICAL_DISK_STATUS_DELETED		2
#define ARCSAS_LOGICAL_DISK_STATUS_MISSING		3 /* LD missing in system. */
#define ARCSAS_LOGICAL_DISK_STATUS_OFFLINE		4
#define ARCSAS_LOGICAL_DISK_STATUS_PARTIALLYOPTIMAL	5 /* r6 w/ 2 pd, 1 hd drops */
#define ARCSAS_LOGICAL_DISK_STATUS_INVALID		0xFF

#define ARCSAS_LOGICAL_DISK_BGA_NONE			0
#define ARCSAS_LOGICAL_DISK_BGA_REBUILD			ARCSAS_BIT(1, 0)
#define ARCSAS_LOGICAL_DISK_BGA_CONSISTENCY_FIX		ARCSAS_BIT(1, 1)
#define ARCSAS_LOGICAL_DISK_BGA_CONSISTENCY_CHECK	ARCSAS_BIT(1, 2)
#define ARCSAS_LOGICAL_DISK_BGA_INIT_QUICK		ARCSAS_BIT(1, 3)
#define ARCSAS_LOGICAL_DISK_BGA_INIT_BACK		ARCSAS_BIT(1, 4)
#define ARCSAS_LOGICAL_DISK_BGA_MIGRATION		ARCSAS_BIT(1, 5)

#define ARCSAS_LOGICAL_DISK_MODE_RAID0		0x0
#define ARCSAS_LOGICAL_DISK_MODE_RAID1		0x1
#define ARCSAS_LOGICAL_DISK_MODE_RAID5		0x5
#define ARCSAS_LOGICAL_DISK_MODE_RAID6		0x6
#define ARCSAS_LOGICAL_DISK_MODE_JBOD		0x0f
#define ARCSAS_LOGICAL_DISK_MODE_RAID10		0x10
#define ARCSAS_LOGICAL_DISK_MODE_RAID1E		0x11
#define ARCSAS_LOGICAL_DISK_MODE_RAID50		0x50
#define ARCSAS_LOGICAL_DISK_MODE_RAID60		0x60
#define ARCSAS_LOGICAL_DISK_MODE_UNKNOWN	0xFF

#define SD_SPEED_1_5G					1 // current 
#define SD_SPEED_3G					2
#define SD_SPEED_6G					3

#define FIS_REG_H2D_SIZE_IN_DWORD			5

#define ARCSAS_SATA_GSCR_ID_REG_NUM			0
#define ARCSAS_SATA_GSCR_REVISION_REG_NUM		1
#define ARCSAS_SATA_GSCR_INFO_REG_NUM			2
#define ARCSAS_SATA_GSCR_ERROR_REG_NUM			32
#define ARCSAS_SATA_GSCR_ERROR_ENABLE_REG_NUM		33
#define ARCSAS_SATA_GSCR_FEATURES_REG_NUM		64
#define ARCSAS_SATA_GSCR_FEATURES_ENABLE_REG_NUM	96
#define ARCSAS_SATA_PSCR_SSTATUS_REG_NUM		0
#define ARCSAS_SATA_PSCR_SERROR_REG_NUM			1
#define ARCSAS_SATA_PSCR_SCONTROL_REG_NUM		2
#define ARCSAS_SATA_PSCR_SACTIVE_REG_NUM		3

#define SERROR_EXCHANGED				ARCSAS_BIT(1, 26)
#define SERROR_COMM_WAKE				ARCSAS_BIT(1, 18)
#define SERROR_PHYRDY_CHANGE				ARCSAS_BIT(1, 16)

#define CONFIG_ROUTE_INDEX_VALID			0xff
#define CONFIG_ROUTE_INDEX_LOADED			0xf0
#define CONFIG_ROUTE_INDEX_INVALID			0x00

/* 
Proposed Interface Between Core Driver and Discover Engine (IBCD)


Objective
Looking for an interface to help extend the flexibility, since Discover Engine 
is planned to be used for different applications while the core driver has limited 
resource and dedicated function need.
*/

#define SAS_ADDR_SIZE				8
#define MAXIMUM_DISCOVER_REQUESTS		12
#define CONTINUEREMAINDER			0xe0
/* FIS type definition */
#define SATA_FIS_TYPE_REG_H2D			0x27	/* Register FIS - Host to Device */
#define SATA_FIS_TYPE_REG_D2H			0x34	/* Register FIS - Device to Host */
#define SATA_FIS_TYPE_DMA_ACTIVATE		0x39	/* DMA Activate FIS - Device to Host */
#define SATA_FIS_TYPE_DMA_SETUP 		0x41	/* DMA Setup FIS - Bi-directional */
#define SATA_FIS_TYPE_DATA			0x46	/* Data FIS - Bi-directional */
#define SATA_FIS_TYPE_BIST_ACTIVATE		0x58	/* BIST Activate FIS - Bi-directional */
#define SATA_FIS_TYPE_PIO_SETUP 		0x5F	/* PIO Setup FIS - Device to Host */
#define SATA_FIS_TYPE_SET_DEVICE_BITS		0xA1	/* Set Device Bits FIS - Device to Host */	
#define PM_STATUS_IDLE				0
#define PM_STATUS_INITIATING			ARCSAS_BIT(1, 0)
#define PM_STATUS_STARTED			ARCSAS_BIT(1, 1)
#define PM_STATUS_RESET 			ARCSAS_BIT(1, 2)
#define PM_STATUS_DEVICES_INITIATING		ARCSAS_BIT(1, 3)
#define PM_STATUS_DEVICE_REMOVING		ARCSAS_BIT(1, 4)
#define PM_STATUS_DEVICE_HOTPLUG		ARCSAS_BIT(1, 5)
#define PM_STATUS_DEVICE_PHY_CHANGE		ARCSAS_BIT(1, 6)
#define PM_STATUS_GSCR_CHANGE			ARCSAS_BIT(1, 7)
#define PM_STATUS_LINK_RATE_RESET		ARCSAS_BIT(1, 8)

	/* PM State definitions */
#define PM_STATE_IDLE				0x00
#define PM_STATE_INIT				0x01
#define PM_STATE_READ_INFO_DONE 		0x02
#define PM_STATE_READ_ID_DONE			0x03
#define PM_STATE_READ_REV_DONE			0x04
#define PM_STATE_ENABLE_ASYNOTIFY		0x05
#define PM_STATE_CLEAR_ERROR_INFO		0x06
#define PM_STATE_PORT_ENABLE_1			0x07	
#define PM_STATE_PORT_ENABLE_0			0x08
#define PM_STATE_PORT_ENABLED			0x09
#define PM_STATE_READ_PORT_ERROR		0x0A
#define PM_STATE_CLEAR_PORT_ERROR		0x0B
#define PM_STATE_PORT_FINISHED			0x0C
#define PM_STATE_CHECK_HOTPLUG			0x0D
#define PM_STATE_READ_PERROR			0x0E
#define PM_STATE_CLEAR_PERROR			0x0F
#define PM_STATE_READ_PSTATUS			0x10
#define PM_STATE_FLUSH_GERROR			0x11
#define PM_STATE_DEVICE_PLUG_IN 		0x12
#define PM_STATE_DEVICE_PLUG_OUT		0x13
#define PM_STATE_PORT_ENABLE_FLUSHED		0x14
#define PM_STATE_PORT_WAITING_PHY_READY		0x15
#define PM_STATE_PORT_SPIN_UP_DEVICE		0x16
#define PM_STATE_PORT_SPIN_UP_DEVICE_DONE	0x17
	/* Device initialization state */
#define DEVICE_STATE_IDLE			0x0
#define DEVICE_STATE_PM_ENABLE_1		0x1
#define DEVICE_STATE_PM_ENABLE_0		0x2
#define DEVICE_STATE_SRST_1			0x3
#define DEVICE_STATE_SRST_0			0x4
#define DEVICE_STATE_RESET_DONE 		0x5
	/* SATA device states */
#define DEVICE_STATE_IDENTIFY_DONE		0x6
#define DEVICE_STATE_SET_UDMA_DONE		0x7
#define DEVICE_STATE_SET_PIO_DONE		0x8
#define DEVICE_STATE_ENABLE_WRITE_CACHE_DONE	0x9
#define DEVICE_STATE_ENABLE_READ_AHEAD_DONE	0xA
	/* SAS device states */
#define DEVICE_STATE_INQUIRY_DONE		0xB /* issue read capacity next */
#define DEVICE_STATE_INQUIRY_16_DONE		0xC /* issue read capacity 16 next*/
#define DEVICE_STATE_INQUIRY_EVPD_DONE		0xD /*	*/
#define DEVICE_STATE_INQUIRY_16_EVPD_DONE	0xE /*	*/
#define DEVICE_STATE_READ_CAPACITY_DONE 	0xF
#define DEVICE_STATE_MODE_SENSE_DONE		0x10
#define DEVICE_STATE_MODE_SELECT_DONE		0x11
#define DEVICE_STATE_LOG_SENSE_DONE		0x12
	/* Device states */
#define DEVICE_STATE_FORMAT_DONE		0x10
#define DEVICE_STATE_FORMAT_WRITE		0x11
#define DEVICE_STATE_FORMAT_VERIFY		0x12
#define DEVICE_STATE_PM_SOFTRESET		0x13
#define DEVICE_STATE_STARTSTOP_DONE		0x14
#define DEVICE_STATE_REPORT_LUN_DONE		0x15
#define DEVICE_STATE_INIT_FAILED		0xFE
#define DEVICE_STATE_INIT_DONE			0xFF
	/* SES device states */
#define DEVICE_STATE_SUPPORTED_DIAGNOSTICS_DONE	0x59
#define DEVICE_STATE_CONFIGURATION_DONE		0x5A    
#define DEVICE_STATE_ENCLOSURE_STATUS_DONE		0x5B
#define DEVICE_STATE_ELEMENT_DESCRIPTOR_DONE		0x5C
#define DEVICE_STATE_ADDITIONAL_ELEMENT_STATUS_DONE	0x5D
#define DEVICE_STATE_SES_STATE_MACHINE_START		0x50
#define DEVICE_STATE_SES_STATE_MACHINE_DONE		0x5F

#define DC_ATA						ARCSAS_BIT(1, 0)
#define DC_SCSI 					ARCSAS_BIT(1, 1)
#define DC_SERIAL					ARCSAS_BIT(1, 2)
#define DC_PARALLEL					ARCSAS_BIT(1, 3)
#define DC_ATAPI					ARCSAS_BIT(1, 4)
#define DC_SGPIO					ARCSAS_BIT(1, 5)
/* 3G and TCQ */
#define DEVICE_ABILITY_48BIT_SUPPORTED			ARCSAS_BIT(1, 0)
#define DEVICE_ABILITY_SMART_SUPPORTED			ARCSAS_BIT(1, 1)
#define DEVICE_ABILITY_WRITECACHE_SUPPORTED		ARCSAS_BIT(1, 2)
#define DEVICE_ABILITY_NCQ_SUPPORTED			ARCSAS_BIT(1, 3)
#define DEVICE_ABILITY_RATE_1_5G			ARCSAS_BIT(1, 4)
#define DEVICE_ABILITY_RATE_3G				ARCSAS_BIT(1, 5)
#define DEVICE_ABILITY_RATE_6G                      	ARCSAS_BIT(1, 6) 
#define DEVICE_ABILITY_OSM_READOGEXT_SUPPORTED		ARCSAS_BIT(1, 7)
#define DEVICE_ABILITY_READ_LOOK_AHEAD_SUPPORTED	ARCSAS_BIT(1, 8)
#define DEVICE_ABILITY_SMART_SELF_TEST_SUPPORTED	ARCSAS_BIT(1, 9)
#define DEVICE_ABILITY_HAD_FORMAT_UNIT			ARCSAS_BIT(1, 10)
#define DEVICE_ABILITY_POIS_SUPPORTED			ARCSAS_BIT(1, 11)
#define DEVICE_ABILITY_TRIM_SUPPORTED			ARCSAS_BIT(1, 12)
#define DEVICE_ABILITY_SSD				ARCSAS_BIT(1, 13)
#define DEVICE_ABILITY_SCT_FEATURE_SET_SUPPORTED	ARCSAS_BIT(1, 14)
#define DEVICE_ABILITY_DATA_SET_MANAGEMENT_SUPPORTED	ARCSAS_BIT(1, 15)
#define	SOLID_STATE_ROTATING_RATE		1
#define	WORLD_WIDE_NAME_SUPPORTED		ARCSAS_BIT(1, 8)

#define SCT_COMMAND_TRANSPORT			ARCSAS_BIT(1, 0)
#define SCT_WRITE_SAME				ARCSAS_BIT(1, 2)
#define SCT_ERROR_RECOVERY_CONTROL		ARCSAS_BIT(1, 3)
#define SCT_FEATURE_CONTROL			ARCSAS_BIT(1, 4)
#define SCT_DATA_TABLES				ARCSAS_BIT(1, 5)

/* PD's Device type defined in SCSI-III specification (used by new driver) */
#define DT_DIRECT_ACCESS_BLOCK			0x00
#define DT_SEQ_ACCESS				0x01
#define DT_PRINTER				0x02
#define DT_PROCESSOR				0x03
#define DT_WRITE_ONCE				0x04
#define DT_CD_DVD				0x05
#define DT_OPTICAL_MEMORY			0x07
#define DT_MEDIA_CHANGER			0x08
#define DT_STORAGE_ARRAY_CTRL			0x0C
	/* an actual enclosure */
#define DT_ENCLOSURE				0x0D
	/* The following are defined by Marvell */
#define DT_EXPANDER				0x20
#define DT_PM					0x21
	/* a device that provides some SES services (ie, Luigi) */
#define DT_SES_DEVICE				0x22
#define DT_LOGICAL_UNIT_NOT_PRESENT_DEVICE	0x7F
#define ARCSAS_DEVICE_NO_NEED_NOTIFY		0x00
#define ARCSAS_DEVICE_NEED_NOTIFY		0x01
#define ARCSAS_DEVICE_FAILED_NO_NEED_NOTIFY	0x02
	/* Element Type */
#define SES_TYPE_UNSPECIFIED			0x00
#define SES_TYPE_DISK_DEVICE			0x01
#define SES_TYPE_POWER_SUPPLY			0x02
#define SES_TYPE_FAN				0x03
#define SES_TYPE_TEMPERATURE_SENSOR		0x04
#define SES_TYPE_DOOR_LOCK			0x05
#define SES_TYPE_SPEAKER			0x06
#define SES_TYPE_ES_CONTROLLER			0x07/* Enclosure SCC */
#define SES_TYPE_SCC_CONTROLLER 		0x08/* SCC */
#define SES_TYPE_NONVOLATILE_CACHE		0x09
#define SES_TYPE_UPS				0x0B
#define SES_TYPE_DISPLAY			0x0C
#define SES_TYPE_KEYPAD 			0x0D
#define SES_TYPE_ENCLOSURE			0x0E
#define SES_TYPE_SCSI_TRANSCEIVER		0x0F
#define SES_TYPE_LANGUAGE			0x10
#define SES_TYPE_COMM_PORT			0x11
#define SES_TYPE_VOLTAGE_SENSOR 		0x12
#define SES_TYPE_CURRENT_SENSOR 		0x13
#define SES_TYPE_SCSI_TARGET_PORT		0x14
#define SES_TYPE_SCSI_INITIATOR_PORT		0x15
#define SES_TYPE_SIMPLE_SUBENCLOSURE		0x16
#define SES_TYPE_ARRAY_DEVICE			0x17 /*such as ARC8060 in Enclosure */
#define SES_TYPE_VENDOR_SPECIFIC		0x80
#define SD_FEATURE_NCQ				ARCSAS_BIT(1, 0)	/* Capability */
#define SD_FEATURE_TCQ				ARCSAS_BIT(1, 1)
#define SD_FEATURE_1_5G				ARCSAS_BIT(1, 2)
#define SD_FEATURE_3G				ARCSAS_BIT(1, 3)
#define SD_FEATURE_6G				ARCSAS_BIT(1, 4)
#define SD_FEATURE_WRITE_CACHE			ARCSAS_BIT(1, 5)
#define SD_FEATURE_48BITS			ARCSAS_BIT(1, 6)
#define SD_FEATURE_SMART			ARCSAS_BIT(1, 7)
#define SD_FEATURE_DATA_SET_MANAGEMENT          ARCSAS_BIT(1, 8)          
#define CCB_FLAG_LBA_VALID			ARCSAS_BIT(1, 0)
#define CCB_FLAG_CMD_FLAG_VALID			ARCSAS_BIT(1, 1)
#define CCB_FLAG_RETRY				ARCSAS_BIT(1, 2)
#define CCB_FLAG_INTERNAL_SG			ARCSAS_BIT(1, 3)
#define CCB_FLAG_FLUSH				ARCSAS_BIT(1, 6)
#define CCB_FLAG_CONSOLIDATE			ARCSAS_BIT(1, 8)
#define CCB_FLAG_NO_CONSOLIDATE			ARCSAS_BIT(1, 9)
#define CCB_FLAG_EXTERNAL			ARCSAS_BIT(1, 10)
#define CCB_FLAG_CORE_SUB			ARCSAS_BIT(1, 11)
#define CCB_FLAG_BYPASS_HYBRID			ARCSAS_BIT(1, 12)	// hybrid disk simulation
#define CCB_FLAG_REQUEUE			ARCSAS_BIT(1, 13)

#define CORE_STATE_IDLE 			ARCSAS_BIT(1, 0)
#define CORE_STATE_STARTED			ARCSAS_BIT(1, 1)
#define CORE_STATE_OS_BUS_RESET_EH		ARCSAS_BIT(1, 2)
#define CORE_STATE_DEVICE_CHANGE_INTERRUPT	ARCSAS_BIT(1, 3)

#define DISCOVERY_STATE_CONTINUE_REMAINDER	0xe0
#define DISCOVERY_STATE_FINDBOUNDARY		0xe1
#define DISCOVERY_STATE_END			0xe2
#define DISCOVERY_STATE_FREE			0xfc
#define DISCOVERY_STATE_ABORT			0xfd
#define DISCOVERY_STATE_EXPANDER_NODE_DONE	0xfe
#define DISCOVERY_STATE_START			0xff
#define TARGET_NOT_YET_DISCOVER			0xff

#define CCB_STATUS_SUCCESS			0x0
#define CCB_STATUS_NOT_READY			0x1
#define CCB_STATUS_MEDIA_ERROR			0x2
#define CCB_STATUS_BUSY 			0x3
#define CCB_STATUS_INVALID_REQUEST		0x4
#define CCB_STATUS_INVALID_PARAMETER		0x5
#define CCB_STATUS_NO_DEVICE			0x6
/* Sense data structure is the SCSI "Fixed format sense datat" format. */
#define CCB_STATUS_HAS_SENSE			0x7
#define CCB_STATUS_ERROR_WITH_SENSE		0x8
#define CCB_STATUS_SUCCESS_WITH_SENSE		0x9
#define CCB_STATUS_CHECK_CONDITION		0x0A
#define CCB_STATUS_REQUEST_SENSE		0x0B
#define CCB_STATUS_ERROR			0x0C
#define CCB_STATUS_DATA_OVER_RUN                0x0D
/* Request initiator must set the status to CCB_STATUS_PENDING. */
#define CCB_STATUS_PENDING			0x80
#define CCB_STATUS_RETRY			0x81
#define CCB_STATUS_DID_ABORT			0x83
#define CCB_STATUS_DID_RESET			0x84
#define CCB_STATUS_DID_TIMEOUT			0x85


#define ACB_FLAG_HIBERNATE_WAKEUP		ARCSAS_BIT(1, 0)	
#define ACB_FLAG_CORES_STATE_STARTED		ARCSAS_BIT(1, 1) 
#define ACB_FLAG_PERSISTENT_MAP_ENABLE		ARCSAS_BIT(1, 2)	/*flash device map function*/
#define ACB_FLAG_DUAL_CORE			ARCSAS_BIT(1, 3)
#define ACB_FLAG_IS_DUMP			ARCSAS_BIT(1, 4)
#define ACB_FLAG_FLASH_NEED_UPDATE		ARCSAS_BIT(1, 5)
#define ACB_FLAG_DONE_HBA_SHUTDOWN		ARCSAS_BIT(1, 6)
#define ACB_FLAG_SMART_ON			ARCSAS_BIT(1, 7)
#define ACB_FLAG_BUS_RESET			ARCSAS_BIT(1, 8)
#define ACB_FLAG_TIMER_RUNNING			ARCSAS_BIT(1, 9)
#define ACB_FLAG_ENTRY_INDEX_UPDATED		ARCSAS_BIT(1, 10)
#define ACB_FLAG_SYNC_DEVICE_ID			ARCSAS_BIT(1, 11)
#define ACB_FLAG_TYPE_SGPIO_HBA			ARCSAS_BIT(1, 12)
#define ACB_FLAG_RESET_CNTRLR			ARCSAS_BIT(1, 13)
#define ACB_FLAG_MSI_ENABLED			ARCSAS_BIT(1, 14)


#define SD_TYPE_SATA				ARCSAS_BIT(1, 0)
#define SD_TYPE_PATA				ARCSAS_BIT(1, 1)
#define SD_TYPE_SAS				ARCSAS_BIT(1, 2)
#define SD_TYPE_ATAPI				ARCSAS_BIT(1, 3)
#define SD_TYPE_TAPE				ARCSAS_BIT(1, 4)
#define SD_TYPE_SES 				ARCSAS_BIT(1, 5)
#define SD_TYPE_MEDIA_CHANGER			ARCSAS_BIT(1, 6)

#define CMD_PORT_ISS0_1300			0x00

#define CMD_PORT_ACTIVE0_1300			0x40
#define CMD_PORT_ACTIVE0_1320			0x300

/*
************************************************************************
************************************************************************
*/
typedef enum _ARCSAS_QUEUE_COMMAND_RESULT
{
	ARCSAS_QUEUE_COMMAND_RESULT_FINISHED = 0,
	ARCSAS_QUEUE_COMMAND_RESULT_FULL,
	ARCSAS_QUEUE_COMMAND_RESULT_NO_RESOURCE,
	ARCSAS_QUEUE_COMMAND_RESULT_SENT,       /* Request is sent to the hardware and not finished yet. */
	ARCSAS_QUEUE_COMMAND_RESULT_WAIT_REG_SET,  
} ARCSAS_QUEUE_COMMAND_RESULT;
/*
************************************************************************
Element Status
************************************************************************
*/
enum
{
	SES_STATUS_UNSUPPORTED = 0x00,
	SES_STATUS_OK,
	SES_STATUS_CRITICAL,
	SES_STATUS_NONCRITICAL,
	SES_STATUS_UNRECOVERABLE,
	SES_STATUS_NOTINSTALLED,
	SES_STATUS_UNKNOWN,
	SES_STATUS_NOTAVAILABLE,
	SES_STATUS_RESERVED
};
/*
************************************************************************
************************************************************************
*/
enum 
{
        /* COMMON_DONE_Q_CONFIG (R134h) bits */
	DoneQUEUE_SIZE_MASK	= (0xFFF << 0),
	DoneQUEUE_ENABLE	= ARCSAS_BIT(1, 16),
};
/*
************************************************************************
************************************************************************
*/
enum _tag_hba_msg_state
{
	MSG_QUEUE_IDLE = 0,
	MSG_QUEUE_PROC,
};
enum 
{
    /* COMMON_DONE_Q_CONFIG (R134h) bits */
	DONE_QUEUE_SIZE_MASK	= (0xFFF << 0),
	DONE_QUEUE_ENABLE	= ARCSAS_BIT(1, 16),
};
enum 
{
	ARCSAS_MOD_VOID   = 0,
	ARCSAS_MOD_UNINIT,
	ARCSAS_MOD_REGISTERED,  /* module ops pointer registered */
	ARCSAS_MOD_INITED,      /* resource assigned */ 
	ARCSAS_MOD_FUNCTIONAL,
	ARCSAS_MOD_STARTED,
	ARCSAS_MOD_DEINIT,      /* extension released, be gone soon */
	ARCSAS_MOD_GONE,
};
/*
************************************************************************
Resource type
************************************************************************
*/
enum Resource_Type
{
	RESOURCE_CACHED_MEMORY = 0,
	RESOURCE_DISCARDABLE_MEMORY,
	RESOURCE_UNCACHED_MEMORY
};
/*
************************************************************************
Module event type
************************************************************************
*/
enum Module_Event
{
	ARCSAS_EVENT_MODULE_ALL_STARTED = 0,
	ARCSAS_EVENT_DEVICE_CACHE_MODE_CHANGED,
	ARCSAS_EVENT_SPECIFY_RUNTIME_DEVICE,
	ARCSAS_EVENT_DISCARD_RESOURCE,
	ARCSAS_EVENT_DEVICE_PLUG,
	ARCSAS_EVENT_DEVICE_UNPLUG,
	ARCSAS_EVENT_LOG_GENERATED,
};
/*
************************************************************************
**Error_Handling_State
************************************************************************
*/
enum EH_State
{
	EH_NONE = 0,
	EH_REQUEST_ABORT,
	EH_LUN_RESET,
	EH_LINK_RESET,
	EH_PORT_RESET,
	EH_CHIP_RESET,
	EH_SET_DISK_RESET,
	EH_SET_DISK_DOWN,
	EH_PHY_RESET,
	EH_SCSI_ABORT_CMND
};
typedef enum
{
	EH_REQ_NOP = 0,
	EH_REQ_ABORT_REQUEST,
	EH_REQ_HANDLE_TIMEOUT,
	EH_REQ_RESET_BUS,
	EH_REQ_RESET_CHANNEL,
	EH_REQ_RESET_DEVICE,
	EH_REQ_RESET_ADAPTER
}eh_req_type_t;
enum 
{
	CCB_TYPE_OS = 2,
	CCB_TYPE_CACHE,
	CCB_TYPE_RAID,
	CCB_TYPE_MP,
	CCB_TYPE_INTERNAL,
};
/*
************************************************************************
**  SAS/SATA configuration port registers, aka phy registers
************************************************************************
*/
enum 
{
	/* Port Config Registers PORT_CONFIG_ADDR, PORT_CONFIG_DATA */
	CONFIG_DEV_INFO			= 0x00, /* port device info register for IDENTIFY frame*/
	CONFIG_DEV_ADDR_LO		= 0x04, /* HBA Port device SAS addr lo register */
	CONFIG_DEV_ADDR_HI		= 0x08, /* HBA Port device SAS addr hi register */
	CONFIG_ATT_DEV_INFO		= 0x0c, /* attached device info register */
	CONFIG_ATT_DEV_ADDR_LO		= 0x10, /* attached device SAS addr lo reg */
	CONFIG_ATT_DEV_ADDR_HI		= 0x14, /* attached device SAS addr hi reg */
	CONFIG_SATA_CONTROL		= 0x18, /* port SATA control register */
	CONFIG_PHY_STATUS		= 0x1c, /* port phy status register */
	CONFIG_SATA_SIG0		= 0x20, /* port SATA signature FIS(Byte 0-3) */
	CONFIG_SATA_SIG1		= 0x24, /* port SATA signature FIS(Byte 4-7) */
	CONFIG_SATA_SIG2       		= 0x28, /* port SATA signature FIS(Byte 8-11) */
	CONFIG_SATA_SIG3		= 0x2c, /* port SATA signature FIS(Byte 12-15)*/
	CONFIG_R_ERR_COUNT		= 0x30, /* port R_ERR count register */
	CONFIG_CRC_ERR_COUNT		= 0x34, /* port CRC error count register */
	CONFIG_WIDE_PORT		= 0x38, /* wide port participating register */
	CONFIG_PHYR_CURRENT0		= 0x80, /* current connection info 0 */
	CONFIG_PHYR_CURRENT1		= 0x84, /* current connection info 1 */
	CONFIG_PHYR_CURRENT2		= 0x88, /* current connection info 2 */
	CONFIG_ID_FRAME0		= 0x100, /* Port device ID frame register DWORD0 */
	CONFIG_ID_FRAME1		= 0x104, /* Port device ID frame register DWORD1 */
	CONFIG_ID_FRAME2		= 0x108, /* Port device ID frame register DWORD2 */
	CONFIG_ID_FRAME3		= 0x10c, /* Port device ID frame register DWORD3 */
	CONFIG_ID_FRAME4		= 0x110, /* Port device ID frame register DWORD4 */
	CONFIG_ID_FRAME5		= 0x114, /* Port device ID frame register DWORD5 */
	CONFIG_ID_FRAME6		= 0x118, /* Port device ID frame register DWORD6 */
	CONFIG_ATT_ID_FRAME0		= 0x11c, /* attached device ID frame register DWORD0 */
	CONFIG_ATT_ID_FRAME1		= 0x120, /* attached device ID frame register DWORD1 */
	CONFIG_ATT_ID_FRAME2		= 0x124, /* attached device ID frame register DWORD2 */
	CONFIG_ATT_ID_FRAME3		= 0x128, /* attached device ID frame register DWORD3 */
	CONFIG_ATT_ID_FRAME4		= 0x12c, /* attached device ID frame register DWORD4 */
	CONFIG_ATT_ID_FRAME5		= 0x130, /* attached device ID frame register DWORD5 */
	CONFIG_ATT_ID_FRAME6		= 0x134, /* attached device ID frame register DWORD6 */
	CONFIG_BRDCST_RCV_TYPE		= 0x140, /* broadcast received type */
	/* CONFIG_DEV_INFO/CONFIG_ATT_DEV_INFO (0x00) bits */
	PORT_DEV_TYPE_MASK		= (0x7 << 0),
	PORT_DEV_INIT_MASK		= (0x7 << 9),
	PORT_DEV_TRGT_MASK		= (0x7 << 17),
	PORT_DEV_STP_INIT		= ARCSAS_BIT(1, 9),
	PORT_DEV_SMP_INIT		= ARCSAS_BIT(1, 10),
	PORT_DEV_SSP_INIT		= ARCSAS_BIT(1, 11),
	PORT_DEV_STP_TRGT		= ARCSAS_BIT(1, 17),
	PORT_DEV_SMP_TRGT		= ARCSAS_BIT(1, 18),
	PORT_DEV_SSP_TRGT		= ARCSAS_BIT(1, 19),
	PORT_PHY_ID_MASK		= (0xFF << 24),
};
/*
************************************************************************
************************************************************************
*/
enum 
{
    /* PORT_IRQ_STATE/MASK (R160h) bits */
	IRQ_PHY_RDY_CHNG_MASK			= ARCSAS_BIT(1, 0),
	HRD_RES_DONE_MASK			= ARCSAS_BIT(1, 1),
	PHY_ID_DONE_MASK			= ARCSAS_BIT(1, 2),
	PHY_ID_FAIL_MASK			= ARCSAS_BIT(1, 3),
	PHY_ID_TIMEOUT				= ARCSAS_BIT(1, 4),
	HARD_RESET_RCVD_MASK			= ARCSAS_BIT(1, 5),
	PORT_SEL_PRESENT_MASK			= ARCSAS_BIT(1, 6),
	COMWAKE_RCVD_MASK			= ARCSAS_BIT(1, 7),
	IRQ_BRDCST_CHNG_RCVD_MASK		= ARCSAS_BIT(1, 8),
	UNKNOWN_TAG_ERR				= ARCSAS_BIT(1, 9),
	IU_TOO_SHRT_ERR				= ARCSAS_BIT(1, 10),
	IU_TOO_LNG_ERR				= ARCSAS_BIT(1, 11),
	PHY_RDY_CHNG_1_TO_0			= ARCSAS_BIT(1, 12),
	IRQ_SIG_FIS_RCVD_MASK			= ARCSAS_BIT(1, 16),
	BIST_ACTVT_FIS_RCVD_MASK		= ARCSAS_BIT(1, 17),
	ASYNC_NTFCN_RCVD_MASK			= ARCSAS_BIT(1, 18),
	IRQ_UNASSOC_FIS_RCVD_MASK		= ARCSAS_BIT(1, 19),
	STP_SATA_RX_ERR_MASK			= ARCSAS_BIT(1, 20),
	STP_SATA_TX_ERR_MASK			= ARCSAS_BIT(1, 21),
	STP_SATA_CRC_ERR_MASK			= ARCSAS_BIT(1, 22),
	STP_SATA_DCDR_ERR_MASK			= ARCSAS_BIT(1, 23),
	STP_SATA_PHY_DEC_ERR_MASK		= ARCSAS_BIT(1, 24),
	STP_SATA_SYNC_ERR_MASK			= ARCSAS_BIT(1, 25),

	/* PORT_PHY_CONTROL/STATUS bits */
	ARCSAS_PHY_RESET			= ARCSAS_BIT(1, 0),/* phy reset */
	ARCSAS_PHY_HARD_RESET_SEQ		= ARCSAS_BIT(1, 1),/* hard reset + phy reset */
	ARCSAS_PHY_BRDCST_CHNG_NOTIFY		= ARCSAS_BIT(1, 2),
	ARCSAS_SSP_RESET			= ARCSAS_BIT(1, 3),
	ARCSAS_SAS_LINK_EN			= ARCSAS_BIT(1, 4),
	ARCSAS_PHY_READY_MASK			= ARCSAS_BIT(1, 20),
	ARCSAS_MIN_SPP_PHYS_LINK_RATE_MASK_1300	= (0xF << 8),
	ARCSAS_MAX_SPP_PHYS_LINK_RATE_MASK_1300	= (0xF << 12),
	ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_OFFSET_1300	= 16,
	ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_1300	= (0xF << ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_OFFSET_1300),

	ARCSAS_MIN_SPP_PHYS_LINK_RATE_MASK_1320	= (0x7 << 8),
	ARCSAS_MAX_SPP_PHYS_LINK_RATE_MASK_1320	= (0x7 << 12),
	ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_OFFSET_1320 = 16,
	ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_1320	= (0x3 << ARCSAS_NEG_SPP_PHYS_LINK_RATE_MASK_OFFSET_1320),


	/* Command Port Registers COMMON_CMD_ADDR, COMMON_CMD_DATA */
	/* command port issue register $i (slot 0-31) to (slot 480-511) */
	CMD_PORT_ISS0					= 0x00,
	CMD_PORT_ISS1					= 0x04,
	CMD_PORT_ISS2					= 0x08,
	CMD_PORT_ISS3					= 0x0c,
	CMD_PORT_ISS4					= 0x10,
	CMD_PORT_ISS5					= 0x14,
	CMD_PORT_ISS6					= 0x18,
	CMD_PORT_ISS7					= 0x1c,
	CMD_PORT_ISS8					= 0x20,
	CMD_PORT_ISS9					= 0x24,
	CMD_PORT_ISS10					= 0x28,
	CMD_PORT_ISS11					= 0x2c,
	CMD_PORT_ISS12					= 0x30,
	CMD_PORT_ISS13					= 0x34,
	CMD_PORT_ISS14					= 0x38,
	CMD_PORT_ISS15					= 0x3c,

	/* cmd port active register $i (slot 0-31) to (slot 480-511) */
	CMD_PORT_ACTIVE0				= 0x40, 
	CMD_PORT_ACTIVE1				= 0x44,
	CMD_PORT_ACTIVE2				= 0x48,
	CMD_PORT_ACTIVE3				= 0x4c,
	CMD_PORT_ACTIVE4				= 0x50,
	CMD_PORT_ACTIVE5				= 0x54,
	CMD_PORT_ACTIVE6				= 0x58,
	CMD_PORT_ACTIVE7				= 0x5c,
	CMD_PORT_ACTIVE8				= 0x60,
	CMD_PORT_ACTIVE9				= 0x64,
	CMD_PORT_ACTIVE10				= 0x68,
	CMD_PORT_ACTIVE11				= 0x6c,
	CMD_PORT_ACTIVE12				= 0x70,
	CMD_PORT_ACTIVE13				= 0x74,
	CMD_PORT_ACTIVE14				= 0x78,
	CMD_PORT_ACTIVE15				= 0x7c,

	/* SATA register set $i task file data register */
	CMD_SATA_TFDATA0				= 0x80,
	CMD_SATA_TFDATA1				= 0x84,
	CMD_SATA_TFDATA2				= 0x88,
	CMD_SATA_TFDATA3				= 0x8c,
	CMD_SATA_TFDATA4				= 0x90,
	CMD_SATA_TFDATA5				= 0x94,
	CMD_SATA_TFDATA6				= 0x98,
	CMD_SATA_TFDATA7				= 0x9c,
	CMD_SATA_TFDATA8				= 0xa0,
	CMD_SATA_TFDATA9				= 0xa4,
	CMD_SATA_TFDATA10				= 0xa8,
	CMD_SATA_TFDATA11				= 0xac,
	CMD_SATA_TFDATA12				= 0xb0,
	CMD_SATA_TFDATA13				= 0xb4,
	CMD_SATA_TFDATA14				= 0xb8,
	CMD_SATA_TFDATA15				= 0xbc,

	/* SATA register set $i association reg */
	CMD_SATA_ASSOC0				= 0xc0,
	CMD_SATA_ASSOC1				= 0xc4,
	CMD_SATA_ASSOC2				= 0xc8,
	CMD_SATA_ASSOC3				= 0xcc,
	CMD_SATA_ASSOC4				= 0xd0,
	CMD_SATA_ASSOC5				= 0xd4,
	CMD_SATA_ASSOC6				= 0xd8,
	CMD_SATA_ASSOC7				= 0xdc,
	CMD_SATA_ASSOC8				= 0xe0,
	CMD_SATA_ASSOC9				= 0xe4,
	CMD_SATA_ASSOC10			= 0xe8,
	CMD_SATA_ASSOC11			= 0xec,
	CMD_SATA_ASSOC12			= 0xf0,
	CMD_SATA_ASSOC13			= 0xf4,
	CMD_SATA_ASSOC14			= 0xf8,
	CMD_SATA_ASSOC15			= 0xfc,

	CMD_CMRST_OOB_DET			= 0x100, /* COMRESET OOB detect register */
	CMD_CMWK_OOB_DET			= 0x104, /* COMWAKE OOB detect register */
	CMD_CMSAS_OOB_DET			= 0x108, /* COMSAS OOB detect register */
	CMD_BRST_OOB_DET			= 0x10c, /* burst OOB detect register */
	CMD_OOB_SPACE				= 0x110, /* OOB space control register */
	CMD_OOB_BURST				= 0x114, /* OOB burst control register */
	CMD_PHY_TIMER				= 0x118, /* PHY timer control register */
	CMD_PHY_CONFIG0				= 0x11c, /* PHY config register 0 */
	CMD_PHY_CONFIG1				= 0x120, /* PHY config register 1 */
	CMD_SAS_CONTROL0			= 0x124, /* SAS control register 0 */
	CMD_SAS_CONTROL1			= 0x128, /* SAS control register 1 */
	CMD_SAS_CONTROL2			= 0x12c, /* SAS control register 2 */
	CMD_SAS_CONTROL3			= 0x130, /* SAS control register 3 */
	CMD_ID_TEST				= 0x134, /* ID test register */
	CMD_PL_TIMER				= 0x138, /* PL timer register */
	CMD_WD_TIMER				= 0x13c, /* WD timer register */
	CMD_PORT_SEL_COUNT			= 0x140, /* port selector count register */
	CMD_APP_MEM_CONTROL			= 0x144, /* Application Memory Control */
	CMD_XOR_MEM_CONTROL			= 0x148, /* XOR Block Memory Control */
	CMD_DMA_MEM_CONTROL			= 0x14c, /* DMA Block Memory Control */
	CMD_PORT_MEM_CONTROL0			= 0x150, /* Port Memory Control 0 */
	CMD_PORT_MEM_CONTROL1			= 0x154, /* Port Memory Control 1 */
	CMD_SATA_PORT_MEM_CONTROL0		= 0x158, /* SATA Port Memory Control 0 */
	CMD_SATA_PORT_MEM_CONTROL1		= 0x15c, /* SATA Port Memory Control 1 */
	CMD_XOR_MEM_BIST_CONTROL		= 0x160, /* XOR Memory BIST Control */
	CMD_XOR_MEM_BIST_STATUS			= 0x164, /* XOR Memroy BIST Status */
	CMD_DMA_MEM_BIST_CONTROL		= 0x168, /* DMA Memory BIST Control */
	CMD_DMA_MEM_BIST_STATUS			= 0x16c, /* DMA Memory BIST Status */
	CMD_PORT_MEM_BIST_CONTROL		= 0x170, /* Port Memory BIST Control */
	CMD_PORT_MEM_BIST_STATUS0		= 0x174, /* Port Memory BIST Status 0 */
	CMD_PORT_MEM_BIST_STATUS1		= 0x178, /* Port Memory BIST Status 1 */
	CMD_STP_MEM_BIST_CONTROL		= 0x17c, /* STP Memory BIST Control */
	CMD_STP_MEM_BIST_STATUS0		= 0x180, /* STP Memory BIST Status 0 */
	CMD_STP_MEM_BIST_STATUS1		= 0x184, /* STP Memory BIST Status 1 */
	CMD_RESET_COUNT				= 0x188, /* Reset Count */
	CMD_MONTR_DATA_SEL			= 0x18C, /* Monitor Data/Select */
	CMD_PLL_PHY_CONFIG			= 0x190, /* PLL/PHY Configuration */
	CMD_PHY_CONTROL				= 0x194, /* PHY Control and Status */
	CMD_PHY_TEST_COUNT0			= 0x198, /* Phy Test Count 0 */
	CMD_PHY_TEST_COUNT1			= 0x19C, /* Phy Test Count 1 */
	CMD_PHY_TEST_COUNT2			= 0x1A0, /* Phy Test Count 2 */
	CMD_APP_ERR_CONFIG			= 0x1A4, /* Application Error Configuration */
	CMD_PND_FIFO_CONTROL0			= 0x1A8, /* Pending FIFO Control 0 */
	CMD_HOST_CONTROL			= 0x1AC, /* Host Control Status */
	CMD_HOST_WR_DATA			= 0x1B0, /* Host Write Data */
	CMD_HOST_RD_DATA			= 0x1B4, /* Host Read Data */
	CMD_PHY_MODE_21				= 0x1B8, /* Phy Mode 21 */
	CMD_SL_MODE0 				= 0x1BC, /* SL Mode 0 */
	CMD_SL_MODE1				= 0x1C0, /* SL Mode 1 */
	CMD_PND_FIFO_CONTROL1 			= 0x1C4, /* Pending FIFO Control 1 */
	CMD_PORT_ACTIVE16				= 0x240, /* Command Active 16 */
	CMD_PORT_ACTIVE17				= 0x244, /* Command Active 17 */
	CMD_PORT_ACTIVE18				= 0x248, /* Command Active 18 */
	CMD_PORT_ACTIVE19				= 0x24C, /* Command Active 19 */
	CMD_PORT_ACTIVE20				= 0x250, /* Command Active 20 */
	CMD_PORT_ACTIVE21				= 0x254, /* Command Active 21 */
	CMD_PORT_ACTIVE22				= 0x258, /* Command Active 22 */
	CMD_PORT_ACTIVE23				= 0x25C, /* Command Active 23 */
	CMD_PORT_ACTIVE24				= 0x260, /* Command Active 24 */
	CMD_PORT_ACTIVE25				= 0x264, /* Command Active 25 */
	CMD_PORT_ACTIVE26				= 0x268, /* Command Active 26 */
	CMD_PORT_ACTIVE27				= 0x26C, /* Command Active 27 */
	CMD_PORT_ACTIVE28				= 0x270, /* Command Active 28 */
	CMD_PORT_ACTIVE29				= 0x274, /* Command Active 29 */
	CMD_PORT_ACTIVE30				= 0x278, /* Command Active 30 */
	CMD_PORT_ACTIVE31				= 0x27C, /* Command Active 31 */
	CMD_SATA_TFDATA16				= 0x280, /* SATA Register Set 16 Task File Data */
	CMD_SATA_TFDATA17				= 0x284, /* SATA Register Set 17 Task File Data */
	CMD_SATA_TFDATA18				= 0x288, /* SATA Register Set 18 Task File Data */
	CMD_SATA_TFDATA19				= 0x28C, /* SATA Register Set 19 Task File Data */
	CMD_SATA_TFDATA20				= 0x290, /* SATA Register Set 20 Task File Data */
	CMD_SATA_TFDATA21				= 0x294, /* SATA Register Set 21 Task File Data */
	CMD_SATA_TFDATA22				= 0x298, /* SATA Register Set 22 Task File Data */
	CMD_SATA_TFDATA23				= 0x29C, /* SATA Register Set 23 Task File Data */
	CMD_SATA_TFDATA24				= 0x2A0, /* SATA Register Set 24 Task File Data */
	CMD_SATA_TFDATA25				= 0x2A4, /* SATA Register Set 25 Task File Data */
	CMD_SATA_TFDATA26				= 0x2A8, /* SATA Register Set 26 Task File Data */
	CMD_SATA_TFDATA27				= 0x2AC, /* SATA Register Set 27 Task File Data */
	CMD_SATA_TFDATA28				= 0x2B0, /* SATA Register Set 28 Task File Data */
	CMD_SATA_TFDATA29				= 0x2B4, /* SATA Register Set 29 Task File Data */
	CMD_SATA_TFDATA30				= 0x2B8, /* SATA Register Set 30 Task File Data */
	CMD_SATA_TFDATA31				= 0x2BC, /* SATA Register Set 31 Task File Data */
	CMD_SATA_ASSOC16				= 0x2C0, /* SATA Reg Set 16 Association */
	CMD_SATA_ASSOC17				= 0x2C4, /* SATA Reg Set 17 Association */
	CMD_SATA_ASSOC18				= 0x2C8, /* SATA Reg Set 18 Association */
	CMD_SATA_ASSOC19				= 0x2CC, /* SATA Reg Set 19 Association */
	CMD_SATA_ASSOC20				= 0x2D0, /* SATA Reg Set 20 Association */
	CMD_SATA_ASSOC21				= 0x2D4, /* SATA Reg Set 21 Association */
	CMD_SATA_ASSOC22				= 0x2D8, /* SATA Reg Set 22 Association */
	CMD_SATA_ASSOC23				= 0x2DC, /* SATA Reg Set 23 Association */
	CMD_SATA_ASSOC24				= 0x2E0, /* SATA Reg Set 24 Association */
	CMD_SATA_ASSOC25				= 0x2E4, /* SATA Reg Set 25 Association */
	CMD_SATA_ASSOC26				= 0x2E8, /* SATA Reg Set 26 Association */
	CMD_SATA_ASSOC27				= 0x2EC, /* SATA Reg Set 27 Association */
	CMD_SATA_ASSOC28				= 0x2F0, /* SATA Reg Set 28 Association */
	CMD_SATA_ASSOC29				= 0x2F4, /* SATA Reg Set 29 Association */
	CMD_SATA_ASSOC30				= 0x2F8, /* SATA Reg Set 30 Association */
	CMD_SATA_ASSOC31				= 0x2FC, /* SATA Reg Set 31 Association */
};
/*
************************************************************************
************************************************************************
*/
/*ATA Protocols*/
enum _ATA_PROTOCOL {
        ATA_PROTOCOL_HARD_RESET		= 0x00,
        ATA_PROTOCOL_SRST		= 0x01,
        ATA_PROTOCOL_NON_DATA		= 0x03,
        ATA_PROTOCOL_PIO_IN		= 0x04,
        ATA_PROTOCOL_PIO_OUT		= 0x05,
        ATA_PROTOCOL_DMA		= 0x06,
        ATA_PROTOCOL_DMA_QUEUED		= 0x07,
        ATA_PROTOCOL_DEVICE_DIAG	= 0x08,
        ATA_PROTOCOL_DEVICE_RESET	= 0x09,
        ATA_PROTOCOL_UDMA_IN		= 0x0A,
        ATA_PROTOCOL_UDMA_OUT		= 0x0B,
        ATA_PROTOCOL_FPDMA		= 0x0C,
        ATA_PROTOCOL_RTN_INFO		= 0x0F,
};
/*
************************************************************************
************************************************************************
*/
enum 
{
	/* COMMON_COAL_CONFIG (R148h) bits */
	INT_COAL_COUNT_MASK		= (0x1FF << 0),
	INT_COAL_ENABLE			= ARCSAS_BIT(1, 16),

	/* COMMON_COAL_TIMEOUT (R14Ch) bits ,(R2014Ch)*/
	COAL_TIMER_MASK			= (0xFFFF << 0),
	COAL_TIMER_UNIT_1MS		= ARCSAS_BIT(1, 16),   /* 0: 6.67 ns, 1: 1 us */

	/* COMMON_IRQ_STATE/MASK (R150h) bits */
	INT_CMD_CMPL			= ARCSAS_BIT(1, 0), /* cmd completion */
	INT_CMD_CMPL_MASK		= ARCSAS_BIT(1, 0),
	INT_CMD_ISSUE_STOPPED		= ARCSAS_BIT(1, 1), /* cmd issue stopped */
	INT_SRS_EVENT			= ARCSAS_BIT(1, 3), /* SRS event */
	INT_PORT_MASK_OFFSET		= 8,
	INT_PORT_MASK			= (0xFF << INT_PORT_MASK_OFFSET),
	INT_PORT_STOP_MASK_OFFSET	= 16,
	INT_PORT_STOP_MASK_1300		= (0xF << INT_PORT_STOP_MASK_OFFSET),
	INT_PORT_STOP_MASK_1320		= (0xFF << INT_PORT_STOP_MASK_OFFSET),
	INT_PHY_MASK_OFFSET		= 4,
	INT_PHY_MASK			= (0xF << INT_PHY_MASK_OFFSET),
	INT_NON_SPCFC_NCQ_ERR		= ARCSAS_BIT(1, 24), /* Nos-specific NCQ error */
	INT_I2C_SLAVE_EVENT		= ARCSAS_BIT(1, 25), /* slave I2C event */
	INT_MEM_PAR_ERR			= ARCSAS_BIT(1, 26), /* int mem parity err */
	INT_DMA_PCIE_TIMEOUT		= ARCSAS_BIT(1, 27), /* DMA to PCIE timeout */
	INT_PRD_BC_READ_ERROR		= ARCSAS_BIT(1, 28), /* PRD BC err for read cmd */
	INT_SOFTWARE1_EVENT		= ARCSAS_BIT(1, 29), /* software event 1 */
	INT_SOFTWARE0_EVENT		= ARCSAS_BIT(1, 30), /* software event 0 */
};
/*
************************************************************************
************************************************************************
*/
enum 
{
	/* global controller registers */
	HBA_CAP_REG			= 0x00,		/* host capabilities */
	/* HOST_CAP_REG bits */
	ARCSAS_HBA_CAP_64		= ARCSAS_BIT(1, 31),	/* PCI DAC (64-bit DMA) support */

	HBA_CONTROL_1300_REG		= 0x04,		/* global host control */
	HBA_CONTROL_1320_REG		= 0x10204,	/* global host control */
 	HBA_CHIP_CONFIG_1320_REG	= 0x10104,	/* R10104h Chip Configuration*/
	/* HBA_CONTROL bits */
	ARCSAS_HBA_RESET		= ARCSAS_BIT(1, 0),    /* reset controller; self-clear */
	ARCSAS_HBA_INT_ENABLE		= ARCSAS_BIT(1, 1),    /* global IRQ enable */
	ARCSAS_HBA_DONEQ_ENABLE		= ARCSAS_BIT(1, 2),    /* global done queue enable */

	HBA_IRQ_STAT_1300_REG		= 0x08,		/* global irq status, interrupt status */
	HBA_IRQ_STAT_1320_REG		= 0x10200,	/* main interrupt cause */
	CPU_MAIN_IRQ_MASK_1320_REG	= 0x1020C,
	IRQ_COM_IN_I2O_IOP0_1320	= ARCSAS_BIT(1, 0),
	IRQ_COM_IN_I2O_IOP1_1320	= ARCSAS_BIT(1, 1),
	IRQ_COM_IN_I2O_IOP2_1320	= ARCSAS_BIT(1, 2),
	IRQ_COM_IN_I2O_IOP3_1320	= ARCSAS_BIT(1, 3),
	IRQ_COM_OUT_I2O_HOS0_1320	= ARCSAS_BIT(1, 4),
	IRQ_COM_OUT_I2O_HOS1_1320	= ARCSAS_BIT(1, 5),
	IRQ_COM_OUT_I2O_HOS2_1320	= ARCSAS_BIT(1, 6),
	IRQ_COM_OUT_I2O_HOS3_1320	= ARCSAS_BIT(1, 7),
	IRQ_PCIF_TO_CPU_DRBL0_1320	= ARCSAS_BIT(1, 8),
	IRQ_PCIF_TO_CPU_DRBL1_1320	= ARCSAS_BIT(1, 9),
	IRQ_PCIF_TO_CPU_DRBL2_1320	= ARCSAS_BIT(1, 10),
	IRQ_PCIF_TO_CPU_DRBL3_1320	= ARCSAS_BIT(1, 11),
	IRQ_PCIF_DRBL0_1320		= ARCSAS_BIT(1, 12),
	IRQ_PCIF_DRBL1_1320		= ARCSAS_BIT(1, 13),
	IRQ_PCIF_DRBL2_1320		= ARCSAS_BIT(1, 14),
	IRQ_PCIF_DRBL3_1320		= ARCSAS_BIT(1, 15),
	IRQ_XOR_A_1320			= ARCSAS_BIT(1, 16),
	IRQ_XOR_B_1320			= ARCSAS_BIT(1, 17),
	IRQ_SAS_A_1320			= ARCSAS_BIT(1, 18),
	IRQ_SAS_B_1320			= ARCSAS_BIT(1, 19),
	IRQ_CPU_CNTRL_1320		= ARCSAS_BIT(1, 20),
	IRQ_GPIO_1320			= ARCSAS_BIT(1, 21),
	IRQ_UART_1320			= ARCSAS_BIT(1, 22),
	IRQ_SPI_1320			= ARCSAS_BIT(1, 23),
	IRQ_I2C_1320			= ARCSAS_BIT(1, 24),
	IRQ_SGPIO_1320			= ARCSAS_BIT(1, 25),
	IRQ_COM_ERR_1320		= ARCSAS_BIT(1, 29),
	IRQ_I2O_ERR_1320		= ARCSAS_BIT(1, 30),
	IRQ_PCIE_ERR_1320		= ARCSAS_BIT(1, 31),
	/* HBA_IRQ_STAT_1300 (0x08) bits */
	ARCSAS_SAS_SATA_IRQ_PENDING	= ARCSAS_BIT(1, 0),
	ARCSAS_XOR_IRQ_PENDING		= ARCSAS_BIT(1, 4),

	HBA_PORTS_IMPL_REG		= 0x0C,    /* bitmap of implemented ports ,ports implemented bitmask*/
	HBA_SOC_PHY_CTL_REG		= 0x40,    /* SOC PHY Control */
	HBA_SOC_PORTS_IMP_REG		= 0x9C,    /* SOC Port Implemented */
	HBA_GBL_PORT_TYPE_REG		= 0xA0,    /* port type HBA_GBL_PORT_TYPE_REG=VENDOR_SPECIFIC_REG_START=VSR_PORT_TYPE_REG*/

	HOST_PORTS_IMPL0		= ARCSAS_BIT(1, 0),  /* HOST port0 existing  */
	HOST_PORTS_IMPL1		= ARCSAS_BIT(1, 1),  /* HOST port1 existing  */
	HOST_PORTS_IMPL2		= ARCSAS_BIT(1, 2),  /* HOST port2 existing  */
	HOST_PORTS_IMPL3		= ARCSAS_BIT(1, 3),  /* HOST port3 existing  */
	HOST_PORTS_IMPL4		= ARCSAS_BIT(1, 4),  /* HOST port0 existing  */
	HOST_PORTS_IMPL5		= ARCSAS_BIT(1, 5),  /* HOST port1 existing  */
	HOST_PORTS_IMPL6		= ARCSAS_BIT(1, 6),  /* HOST port2 existing  */
	HOST_PORTS_IMPL7		= ARCSAS_BIT(1, 7),  /* HOST port3 existing  */
	HOST_XOR_IMPL			= ARCSAS_BIT(1, 8),  /* HOST XOR port existing  */
	/* VENDOR_SPECIFIC */
	/* vendor specific registers */
	VSR_PORT_TYPE_REG		= VENDOR_SPECIFIC_REG_START,/* port type register */

	/* VSR_PORT_TYPE bits */
	MODE_SAS_PORT0_MASK		= ARCSAS_BIT(1, 0),  /* SATA if 0 is set */
	MODE_SAS_PORT1_MASK		= ARCSAS_BIT(1, 1),
	MODE_SAS_PORT2_MASK		= ARCSAS_BIT(1, 2),
	MODE_SAS_PORT3_MASK		= ARCSAS_BIT(1, 3),
	MODE_SAS_PORT4_MASK		= ARCSAS_BIT(1, 4),
	MODE_SAS_PORT5_MASK		= ARCSAS_BIT(1, 5),
	MODE_SAS_PORT6_MASK		= ARCSAS_BIT(1, 6),
	MODE_SAS_PORT7_MASK		= ARCSAS_BIT(1, 7),
	MODE_SAS_SATA			= MODE_SAS_PORT0_MASK | MODE_SAS_PORT1_MASK | MODE_SAS_PORT2_MASK | MODE_SAS_PORT3_MASK |	MODE_SAS_PORT4_MASK | MODE_SAS_PORT5_MASK |	MODE_SAS_PORT6_MASK | MODE_SAS_PORT7_MASK,

	MODE_AUTO_DET_PORT0		= ARCSAS_BIT(1, 8),
	MODE_AUTO_DET_PORT1		= ARCSAS_BIT(1, 9),
	MODE_AUTO_DET_PORT2		= ARCSAS_BIT(1, 10),
	MODE_AUTO_DET_PORT3		= ARCSAS_BIT(1, 11),
	MODE_AUTO_DET_PORT4		= ARCSAS_BIT(1, 12),
	MODE_AUTO_DET_PORT5		= ARCSAS_BIT(1, 13),
	MODE_AUTO_DET_PORT6		= ARCSAS_BIT(1, 14),
	MODE_AUTO_DET_PORT7		= ARCSAS_BIT(1, 15),
	MODE_AUTO_DET_EN		= MODE_AUTO_DET_PORT0 | MODE_AUTO_DET_PORT1 | MODE_AUTO_DET_PORT2 | MODE_AUTO_DET_PORT3 | MODE_AUTO_DET_PORT4 | MODE_AUTO_DET_PORT5 | MODE_AUTO_DET_PORT6 | MODE_AUTO_DET_PORT7,
	/*
	************************************************ 
	**  SAS/SATA port common (A/B)registers 
	************************************************
	*/
	COMMON_CONFIG			= 0x100, /* configuration register, SAS/SATA port configuration*/
	COMMON_CONTROL			= 0x104, /* control register, SAS/SATA port control/status*/
	COMMON_LST_ADDR			= 0x108, /* command list DMA addr */
	COMMON_LST_ADDR_HI		= 0x10c, /* command list DMA addr hi */
	COMMON_FIS_ADDR			= 0x110, /* FIS rx buf addr */
	COMMON_FIS_ADDR_HI		= 0x114, /* FIS rx buf addr hi */
	COMMON_1320_STP_REG_SET_0	= 0x118, /* STP/SATA Register Set Enable */
	COMMON_1320_STP_REG_SET_1	= 0x11c,

	COMMON_POST_Q_CONFIG		= 0x120, /* delivery queue configuration */
	COMMON_POST_Q_ADDR		= 0x124, /* delivery queue base address */
	COMMON_POST_Q_ADDR_HI		= 0x128, /* delivery queue base address hi */
	COMMON_POST_Q_WR_PTR		= 0x12c, /* delivery queue write pointer */
	COMMON_POST_Q_RD_PTR		= 0x130, /* delivery queue read pointer */
	COMMON_DONE_Q_CONFIG		= 0x134, /* completion queue configuration */
	COMMON_DONE_Q_ADDR		= 0x138, /* completion queue base address */
	COMMON_DONE_Q_ADDR_HI		= 0x13c, /* completion queue base address hi */
	COMMON_DONE_Q_WR_PTR		= 0x140, /* completion queue write pointer */
	COMMON_DONE_Q_RD_PTR		= 0x144, /* completion queue read pointer */
	COMMON_COAL_CONFIG		= 0x148, /* interrupt coalescing config */
	COMMON_COAL_TIMEOUT		= 0x14c, /* interrupt coalescing time wait */
	COMMON_IRQ_STATE		= 0x150, /* interrupt status */
	COMMON_IRQ_MASK			= 0x154, /* interrupt enable/disable mask */
	COMMON_SRS_IRQ_STAT_0		= 0x158, /* 1300 1320 SATA register set status interrupt status */
	COMMON_SRS_IRQ_MASK_0		= 0x15c, /* 1300 1320 SATA register set intr enable/disable mask Enable SRS interrupt*/
	COMMON_SRS_IRQ_STAT_1		= 0x160, /* 1320 SATA register set status interrupt status */
	COMMON_SRS_IRQ_MASK_1		= 0x164, /* 1320 SATA register set intr enable/disable mask */
	COMMON_NON_SPEC_NCQ_ERR0	= 0x0168, /* Non Specific NCQ Error 0 */
	COMMON_NON_SPEC_NCQ_ERR1	= 0x016c, /* Non Specific NCQ Error 1 */
	/*
	***************************************
	**
	***************************************
	*/
	COMMON_CMD_ADDR_1300		= 0x1b8, /* Command Address */
	COMMON_CMD_DATA_1300		= 0x1bc, /* Command Data */
	PORT_IRQ_STATE0_1300 		= 0x160, /* Port 0 interrupt status */
	PORT_IRQ_MASK0_1300		= 0x164, /* Port 0 interrupt enable/disable mask */
	/* ports 1-3 follow after this 0x168,0x16c,0x170,0x174,0x178,0x17c*/
	PORT_IRQ_STATE4_1300		= 0x200, /* Port 4 interrupt status */
	PORT_IRQ_MASK4_1300		= 0x204, /* Port 4 interrupt enable/disable mask */
	/* ports 5-7 follow after this 0x208,0x20c,0x210,0x214,0x218,0x21c*/
	PORT_PHY_CONTROL0_1300		= 0x180, /* port 0 serial status/control */
	/* ports 1-3 follow after this 0x184,0x188,0x18c*/
	PORT_PHY_CONTROL4_1300		= 0x220, /* port 4 serial status/control */
	/* ports 5-7 follow after this 0x224,0x228,0x22c*/
	PORT_CONFIG_ADDR0_1300		= 0x1c0, /* Port 0 config address */
	PORT_CONFIG_DATA0_1300		= 0x1c4, /* Port 0 config data */
	/* ports 1-3 follow after this 0x1c8,0x1cc,0x1d0,0x1d4,0x1d8,0x1dc*/
	PORT_CONFIG_ADDR4_1300		= 0x230, /* Port 4 config address */
	PORT_CONFIG_DATA4_1300		= 0x234, /* Port 4 config data */
	/* ports 5-7 follow after this 0x238,0x23c,0x240,0x244,0x248,0x24c*/
	PORT_VSR_ADDR0_1300		= 0x1e0, /* port 0 Vendor Specific Register addr */
	PORT_VSR_DATA0_1300		= 0x1e4, /* port 0 Vendor Specific Register Data */
	/* ports 1-3 follow after this 0x1e8,0x1ec,0x1f0,0x1f4,0x1f8,0x1fc*/
	PORT_VSR_ADDR4_1300		= 0x250, /* port 4 Vendor Specific Register addr */
	PORT_VSR_DATA4_1300		= 0x254, /* port 4 Vendor Specific Register Data */
	/* ports 5-7 follow after this 0x258,0x25c,0x260,0x264,0x268,0x26c*/
	/*
	***************************************
	**
	***************************************
	*/

	COMMON_CMD_ADDR_1320		= 0x170, /* Command Address */
	COMMON_CMD_DATA_1320		= 0x174, /* Command Data */
	PORT_IRQ_STATE0_1320		= 0x180, /* Port 0 interrupt status */
	PORT_IRQ_MASK0_1320		= 0x184, /* Port 0 interrupt enable/disable mask */
	/* ports 1-3 follow after this 0x188,0x18C,0x190,0x194,0x198,0x19C*/
	PORT_IRQ_STATE4_1320		= 0x1a0, /* Port 4 interrupt status */
	PORT_IRQ_MASK4_1320		= 0x1a4, /* Port 4 interrupt enable/disable mask */
	/* ports 5-7 follow after this 0x1A8,0x1AC,0x1B0,0x1B4,0x1B8,0x1BC*/
	PORT_PHY_CONTROL0_1320		= 0x1d0, /* port 0 serial status/control */
	/* ports 1-3 follow after this 0x1D4,0x1D8,0x1DC*/
	PORT_PHY_CONTROL4_1320		= 0x1e0, /* port 4 serial status/control */
	/* ports 5-7 follow after this 0x1E4,0x1E8,0x1EC*/
	PORT_CONFIG_ADDR0_1320		= 0x200, /* Port 0 config address */
	PORT_CONFIG_DATA0_1320		= 0x204, /* Port 0 config data */
	/* ports 1-3 follow after this 0x208,0x20C,0x210,0x214,0x218,0x21C*/
	PORT_CONFIG_ADDR4_1320		= 0x220, /* Port 4 config address */
	PORT_CONFIG_DATA4_1320		= 0x224, /* Port 4 config data */
	/* ports 5-7 follow after this 0x228,0x22C,0x230,0x234,0x238,0x23C*/
	PORT_VSR_ADDR0_1320		= 0x250, /* port 0 Vendor Specific Register addr */
	PORT_VSR_DATA0_1320		= 0x254, /* port 0 Vendor Specific Register Data */
	/* ports 1-3 follow after this 0x258,0x25C,0x260,0x264,0x268,0x26C*/
	/* multiplexing !!!!!!!!!!!! */
	PORT_VSR_ADDR4_1320		= 0x250, /* port 4 Vendor Specific Register addr */
	PORT_VSR_DATA4_1320		= 0x254, /* port 4 Vendor Specific Register Data */
	/* ports 5-7 follow after this 0x258,0x25c,0x260,0x264,0x268,0x26C*/
	PORT_VSR_ADDR_ALL_1320		= 0x290, /* All port Vendor Specific Register addr */
	PORT_VSR_DATA_ALL_1320		= 0x294, /* All port Vendor Specific Register Data */

	ARC1320_NON_NCQ_ERR_0		= 0x168, /* SRS Non-specific NCQ Error */
	ARC1320_NON_NCQ_ERR_1		= 0x16c,
	ARC1320_MEM_PARITY_ERR		= 0x178, /* Memory parity error */
	/*
	***************************************
	**
	***************************************
	*/
	/* COMMON_CONFIG register bits */
	ARCSAS_CMD_TBL_BE		= ARCSAS_BIT(1, 0),
	ARCSAS_OPEN_ADDR_BE		= ARCSAS_BIT(1, 1),
	ARCSAS_RSPNS_FRAME_BE		= ARCSAS_BIT(1, 2),
	ARCSAS_DATA_BE			= ARCSAS_BIT(1, 3),
	ARCSAS_SAS_SATA_RST		= ARCSAS_BIT(1, 5),
	EN_SATA_REG_16			= ARCSAS_BIT(1, 16),
	EN_SATA_REG_17			= ARCSAS_BIT(1, 17),
	EN_SATA_REG_18			= ARCSAS_BIT(1, 18),
	EN_SATA_REG_19			= ARCSAS_BIT(1, 19),
	EN_SATA_REG_20			= ARCSAS_BIT(1, 20),
	EN_SATA_REG_21			= ARCSAS_BIT(1, 21),
	EN_SATA_REG_22			= ARCSAS_BIT(1, 22),
	EN_SATA_REG_23			= ARCSAS_BIT(1, 23),
	EN_SATA_REG_24			= ARCSAS_BIT(1, 24),
	EN_SATA_REG_25			= ARCSAS_BIT(1, 25),
	EN_SATA_REG_26			= ARCSAS_BIT(1, 26),
	EN_SATA_REG_27			= ARCSAS_BIT(1, 27),
	EN_SATA_REG_28			= ARCSAS_BIT(1, 28),
	EN_SATA_REG_29			= ARCSAS_BIT(1, 29),
	EN_SATA_REG_30			= ARCSAS_BIT(1, 30),
	EN_SATA_REG_31			= ARCSAS_BIT(1, 31),
	/* pci config register */
	ARCSAS_PCI_REG_CMD		= 0x04,
	ARCSAS_PCI_REG_DEV_CTRL_1300	= 0xE8,
	ARCSAS_PCI_REG_DEV_CTRL_1320	= 0x78,
	ARCSAS_PCI_REG_LINK_STAT_1300	= 0xF2,
	ARCSAS_PCI_REG_LINK_STAT_1320	= 0x82,
	ARCSAS_PCI_REG_PHY_CTRL1_1300	= 0x40,
	//ARCSAS_PCI_REG_MSI_CAP_ID	= 0x50,
	ARCSAS_PCI_REG_MSI_CTRL		= 0x50,
	ARCSAS_PCI_REG_VEN_UNIQ3	= 0x60,
	ARCSAS_PCI_REG_PHY_CTRL2_1300	= 0x90,
	/* pci config register bit */
	ARCSAS_PCI_IO_EN		= ARCSAS_BIT(1, 0),
	ARCSAS_PCI_MEM_EN		= ARCSAS_BIT(1, 1),
	ARCSAS_PCI_BM_EN		= ARCSAS_BIT(1, 2),    /* enable bus master */
	ARCSAS_PCI_INT_DIS		= ARCSAS_BIT(1, 10),   /* disable INTx for MSI_EN */
	ARCSAS_PCI_DEV_EN		= ARCSAS_PCI_IO_EN | ARCSAS_PCI_MEM_EN | ARCSAS_PCI_BM_EN,
	ARCSAS_PCI_MSI_EN		= ARCSAS_BIT(1, 16),
	ARCSAS_PCI_MULTI_MESSAGE_EN	= ARCSAS_BIT(1, 20) | ARCSAS_BIT(1, 21) | ARCSAS_BIT(1, 22),
	ARCSAS_PCI_MULTI_MESSAGE_CAP 	= ARCSAS_BIT(1, 19) | ARCSAS_BIT(1, 18) | ARCSAS_BIT(1, 17),
	ARCSAS_PCI_RD_REQ_SIZE		= 0x2000,
	ARCSAS_PCI_RD_REQ_MASK		= 0x00007000,
	ARCSAS_PCI_PAD_TEST_MASK	= 0xFF000000, /* Test Pad bits located in Vendor Unique 3*/
	ARCSAS_PCI_PHY_OFF		= (0xF << 12),
	ARCSAS_PCI_PHY_PWR_ON		= (0xF << 24),
	ARCSAS_PCI_NEG_LINK_WD		= (0x3F << 4),
	ARCSAS_PCI_NEG_LINK_WD_OFFS	= 4,
	ARCSAS_PCI_LINK_SPD		= (0x0F << 0),
	ARCSAS_PCI_LINK_SPD_OFFS	= 0,

	/* COMMON_CONTROL : port control/status bits (R104h) */
	EN_CMD_ISSUE			= ARCSAS_BIT(1, 0), /* enable cmd issue */
	RESET_CMD_ISSUE			= ARCSAS_BIT(1, 1), /* reset cmd issue */
	ERR_STOP_CMD_ISSUE		= ARCSAS_BIT(1, 3), /* cmd stop-on-err enable */
	EN_FIS_RCV			= ARCSAS_BIT(1, 4), /* FIS rx enable */
	CMD_CMPL_SELF_CLEAR		= ARCSAS_BIT(1, 5), /* self-clearing int mode */
	EN_SATA_RETRY_2			= ARCSAS_BIT(1, 6), /*for 1320*/
	EN_RSPNS_RCV			= ARCSAS_BIT(1, 7), /* raw response rx */
	EN_SATA_RETRY_1			= ARCSAS_BIT(1, 8), /*for 1300*/
	EN_PORT_0_XMT			= ARCSAS_BIT(1, 12),/*for 1300*/
	EN_PORT_1_XMT			= ARCSAS_BIT(1, 13),/*for 1300*/
	EN_PORT_2_XMT			= ARCSAS_BIT(1, 14),/*for 1300*/
	EN_PORT_3_XMT			= ARCSAS_BIT(1, 15),/*for 1300*/
	EN_SATA_REG_0			= ARCSAS_BIT(1, 16),
	EN_SATA_REG_1			= ARCSAS_BIT(1, 17),
	EN_SATA_REG_2			= ARCSAS_BIT(1, 18),
	EN_SATA_REG_3			= ARCSAS_BIT(1, 19),
	EN_SATA_REG_4			= ARCSAS_BIT(1, 20),
	EN_SATA_REG_5			= ARCSAS_BIT(1, 21),
	EN_SATA_REG_6			= ARCSAS_BIT(1, 22),
	EN_SATA_REG_7			= ARCSAS_BIT(1, 23),
	EN_SATA_REG_8			= ARCSAS_BIT(1, 24),
	EN_SATA_REG_9			= ARCSAS_BIT(1, 25),
	EN_SATA_REG_10			= ARCSAS_BIT(1, 26),
	EN_SATA_REG_11			= ARCSAS_BIT(1, 27),
	EN_SATA_REG_12			= ARCSAS_BIT(1, 28),
	EN_SATA_REG_13			= ARCSAS_BIT(1, 29),
	EN_SATA_REG_14			= ARCSAS_BIT(1, 30),
	EN_SATA_REG_15			= ARCSAS_BIT(1, 31),

	EN_PORT_XMT_START		= 12,/*EN_PORT_0_XMT = ARCSAS_BIT(1, 12),for 1300*/
	EN_PORT_XMT_START2		= 8, /*EN_PORT_0_XMT = ARCSAS_BIT(1, 08),for 1320*/

        /* COMMON_POST_Q_CONFIG (R120h) bits */
	POST_QUEUE_SIZE_MASK		= (0xFFF << 0),
	POST_QUEUE_ENABLE		= ARCSAS_BIT(1, 16),
};
/*
************************************************************************
************************************************************************
*/
enum 
{
	MAP_TGT_COUNT = 0,
	MAP_TGT_INFO
};
/*
************************************************************************
************************************************************************
*/
enum 
{
	STATUS_REQUEST_OK = 0,
	STATUS_REQUEST_QUEUE_FULL
};
/*
************************************************************************
************************************************************************
*/
enum sas_dev_type
{
	NO_DEVICE   = 0,	  /* protocol */
	SAS_END_DEV = 1,	  /* protocol */
	EDGE_DEV    = 2,	  /* protocol */
	FANOUT_DEV  = 3,	  /* protocol */
	SAS_HA      = 4,
	SATA_DEV    = 5,
	SATA_PM     = 7,
	SATA_PM_PORT= 8,
};
/*
************************************************************************
************************************************************************
*/
enum sas_phy_linkrate
{
	PHY_LINKRATE_NONE = 0,
	PHY_LINKRATE_UNKNOWN = 0,
	PHY_DISABLED,
	PHY_RESET_PROBLEM,
	PHY_SPINUP_HOLD,
	PHY_PORT_SELECTOR,
	PHY_LINKRATE_1_5 = 0x08,
	PHY_LINKRATE_G1  = PHY_LINKRATE_1_5,
	PHY_LINKRATE_3   = 0x09,
	PHY_LINKRATE_G2  = PHY_LINKRATE_3,
	PHY_LINKRATE_6   = 0x0A,
};
/*
************************************************************************
************************************************************************
*/
enum sas_oob_mode 
{
	OOB_NOT_CONNECTED,
	SATA_OOB_MODE,
	SAS_OOB_MODE
};
/*
************************************************************************
************************************************************************
*/
enum sas_proto 
{
	SATA_PROTO    = 1,
	SAS_PROTO_SMP = 2,		/* protocol */
	SAS_PROTO_STP = 4,		/* protocol */
	SAS_PROTO_SSP = 8,		/* protocol */
	SAS_PROTO_ALL = 0xE,
};
enum
{
	SAS_SIMPLE_LEVEL_DESCENT = 0,
	SAS_UNIQUE_LEVEL_DESCENT
};
enum
{
	SAS_10_COMPATIBLE = 0,
	SAS_11_COMPATIBLE
};
/*
************************************************************************
** definitions for SMP function results
************************************************************************
*/
enum SMPFunctionResult
{
	SMP_REQUEST_ACCEPTED = 0, 
	SMP_FUNCTION_ACCEPTED = 0,
	SMP_UNKNOWN_FUNCTION,
	SMP_FUNCTION_FAILED,
	SMP_INVALID_REQUEST_FRAME_LENGTH,
	SMP_PHY_DOES_NOT_EXIST = 0x10,
	SMP_INDEX_DOES_NOT_EXIST,
	SMP_PHY_DOES_NOT_SUPPORT_SATA,
	SMP_UNKNOWN_PHY_OPERATION,
	SMP_UNKNOWN_PHY_TEST_FUNCTION,
	SMP_UNKNOWN_PHY_TEST_FUNCTION_IN_PROGRESS,
	SMP_PHY_VACANT,
	SMP_PHY_EVENT_INFORMATION_SOURCE_NOT_SUPPORTED,
	SMP_ZONE_VIOLATION = 0x20,
	SMP_PHYSICAL_PRESENCE_NOT_ASSERTED,
	SMP_UNKNOWN_ENABLE_DISABLE_ZONING_VALUE
};
/*
************************************************************************
** RoutingAttribute
************************************************************************
*/
enum RoutingAttribute
{
	DIRECT = 0,
	SUBTRACTIVE,
	TABLE,
	/*
		this attribute is a psuedo attribute, used to reflect the function
		result of SMP_PHY_VACANT in a fabricated discover response
	*/
	PHY_NOT_USED = 15
};
/*
************************************************************************
** ConnectorType
************************************************************************
*/
enum ConnectorType
{
	UNKNOWN_CONNECTOR = 0,
	SFF_8470_EXTERNAL_WIDE,
	SFF_8484_INTERNAL_WIDE = 0x10,
	SFF_8484_COMPACT_INTERNAL_WIDE,
	SFF_8482_BACKPLANE = 0x20,
	SATA_HOST_PLUG,
	SAS_DEVICE_PLUG,
	SATA_DEVICE_PLUG
};
/*
************************************************************************
** RouteFlag
************************************************************************
*/
enum DisableRouteEntry
{
	ENABLED = 0,
	DISABLED
};
/*
************************************************************************
** PhyOperation
************************************************************************
*/
enum PhyOperation
{
	PHY_NOP = 0,
	LINK_RESET,
	HARD_RESET,
	DISABLE,
	CLEAR_ERROR_LOG = 5,
	CLEAR_AFFILIATION,
	TRANSMIT_SATA_PORT_SELECTION_SIGNAL,
	CLEAR_STP_NEXUS_LOSS
};
/*
************************************************************************
** EnableDisableZoning
************************************************************************
*/
enum EnableDisableZoning
{
	NO_CHANGE=0,
	ENABLE_ZONING,
	DISABLE_ZONING,
};
/*
************************************************************************
** BroadcastType
************************************************************************
*/
enum BroadcastType
{
	BROADCAST_CHANGE = 0,
	BROADCAST_RESERVED_CHANGE0,
	BROADCAST_RESERVED_CHANGE1,
	BROADCAST_SES,
	BROADCAST_EXPANDER,
	BROADCAST_ASYNCHRONOUS_EVENT,
	BROADCAST_RESERVED_CHANGE3,
	BROADCAST_RESERVED_CHANGE4,	
};
/*
************************************************************************
** DescriptorType
************************************************************************
*/
enum DescriptorType
{
	LONG_FORMAT_DESCRIPTOR=0,
	SHORT_FORMAT_DESCRIPTOR,
};
/*
************************************************************************
** PortType
************************************************************************
*/
enum PortType
{
	PORT_IS_TARGET=0,
	PORT_IS_INITIATOR
};
/*
************************************************************************
** The structures assume a char bitfield is valid, this is compiler
** dependent defines would be more portable, but less descriptive
** the Identify frame is exchanged following OOB, for this
** code it contains the identity information for the attached device
** and the initiator application client.
************************************************************************
*/
enum 
{
	FRAME_PORT_TYPE_SMP		= ARCSAS_BIT(1, 1),/*0x02*/
	FRAME_PORT_TYPE_STP		= ARCSAS_BIT(1, 2),/*0x04*/
	FRAME_PORT_TYPE_SSP		= ARCSAS_BIT(1, 3),/*0x08*/
	FRAME_ATT_DEV_TYPE_SMP		= ARCSAS_BIT(1, 1),/*0x02*/
	FRAME_ATT_DEV_TYPE_STP		= ARCSAS_BIT(1, 2),/*0x04*/
	FRAME_ATT_DEV_TYPE_SSP		= ARCSAS_BIT(1, 3),/*0x08*/
	FRAME_ATT_DEV_SATA_DEV		= ARCSAS_BIT(1, 0),/*0x01*/
	FRAME_ATT_DEV_SATA_HOST		= ARCSAS_BIT(1, 0),/*0x01*/
	FRAME_ATT_DEV_SATA_SELECTOR	= ARCSAS_BIT(1, 7),/*0x80*/
};
/* 
************************************************************************
************************************************************************
*/
enum vsr_regs {
	VSR_IRQ_STATUS		= 0x00,
	VSR_IRQ_MASK		= 0x04,
	VSR_PHY_CONFIG		= 0x08,
	VSR_PHY_STATUS		= 0x0c,
	VSR_PHY_MODE_REG_1	= 0x064,
	VSR_PHY_FFE_CONTROL	= 0x10C,
	VSR_PHY_DFE_UPDATE_CRTL	= 0x110,
	VSR_REF_CLOCK_CRTL	= 0x1A0,
};
enum vsr_reg_bits {
	VSR_IRQ_PHY_TIMEOUT	= (1U << 10),
	VSR_PHY_STATUS_MASK	= 0x3f0000,
	VSR_PHY_STATUS_IDLE	= 0x00,
	VSR_PHY_STATUS_SAS_RDY	= 0x10,
	VSR_PHY_STATUS_HR_RDY	= 0x1d,
};
/* 
************************************************************************
**    discovery element structure
************************************************************************
*/
struct _DISCOVERY_ELEMENT
{
	PDiscover_Extension	pDiscovery;
	PDomain_Port		pPort;   
	unsigned char		nTgt;
	unsigned char		PhyId;
	unsigned char		CurrentState;
	unsigned char		context;
	void			*pBuffer;
};
/* 
************************************************************************
**    event structure
************************************************************************
*/
typedef struct _EVENT_ELEMENT   /*32BYTES*/
{
	/*
	********************************************************
	** <second>	bit 05,04,03,02,01,00: 0 - 59 
	** <minute>	bit 11,10,09,08,07,06: 0 - 59 
	** <month>	bit 15,14,13,12: 1 - 12 
	** <hour>	bit 20,19,18,17,16: 0 - 23 
	** <day>		bit 25,24,23,22,21: 1 - 31 
	** <year>	bit 30,29,28,27,26: 0=2000,31=2031 
	********************************************************
	*/
	u32	TimeStamp;	/*  4bytes: y.d.h.m.m.s */
	u16	DeviceID;	/*  2bytes: scsi id, scsi lun */ 
	u8	EventID;		/*  1bytes: 0...31*/
	u8	EventType;	/*  1bytes: slot type */
	/*
	********************************************************
	********************************************************
	*/
	u32	ResponseCode;		/*  4bytes: */
	u16	ParentIndexChildSlot;	/*  2bytes: Parent,Child */
	u16	EventFlag;		/*  2bytes: */
		#define ARCSAS_EVENT_FLAG_ERROR_INFO_RECORD		ARCSAS_BIT(1, 0)   /*EventScratchPad:"ERR_INFO_RECORD"             */
		#define ARCSAS_EVENT_FLAG_DEVICE_PLUG			ARCSAS_BIT(1, 1)   /*EventScratchPad:Enc#1 Slot#1,Pm#1 Dev#1, Dev#1*/
		#define ARCSAS_EVENT_FLAG_DEVICE_UNPLUG		ARCSAS_BIT(1, 2)   /*EventScratchPad:Enc#1 Slot#1,Pm#1 Dev#1, Dev#1*/
		#define ARCSAS_EVENT_FLAG_DEVICE_FAILED		ARCSAS_BIT(1, 3)   /*EventScratchPad:Enc#1 Slot#1,Pm#1 Dev#1, Dev#1*/
		#define ARCSAS_EVENT_FLAG_DEVICE_READING_ERROR	ARCSAS_BIT(1, 4)   /*EventScratchPad:Enc#1 Slot#1,Pm#1 Dev#1, Dev#1*/
		#define ARCSAS_EVENT_FLAG_HW_FAILED			ARCSAS_BIT(1, 5)   /*EventScratchPad:RTC Battery,...               */
		#define ARCSAS_EVENT_FLAG_RAID				ARCSAS_BIT(1, 6)   /*EventScratchPad:CREATE VOLUME                 */
		#define ARCSAS_EVENT_FLAG_INVALID_SLOT_NUMBER	ARCSAS_BIT(1, 7)   /*EventScratchPad:Invalid slot number           */
	u8	EventScratchPad[16];	/* 16bytes: scratch pad for EventFlag description*/
} EVENT_ELEMENT, *PEVENT_ELEMENT;
/*
************************************************************************
**   Device ID
************************************************************************
*/
struct _PCI_DEVICE_ID
{
	unsigned char	pci_device_id[4];
};
/*
************************************************************************
**	Primary Data Type
************************************************************************
*/
union _ARCSAS_U64
{
	struct {
		u32 low;
		u32 high;
	} parts;
	u64 value;
};
/*
************************************************************************
************************************************************************
*/
struct _Unmap_Block_Descriptor
{
	ARCSAS_U64	block;		/* byte 0,1,2,3,4,5,6,7 */
	u_int32_t	n_block;	/* byte 08,09,10,11 */
	u_int8_t	Reserved[4];	/* byte 12,13,14,15 */
};
/*
************************************************************************
************************************************************************
*/
struct _PRDTableWalkCtx
{
//	void			*pPRDEntry;
	PPRD_Buffer		pPRD_Buffer;
	unsigned int		avail;
	unsigned int		prd_entry_count;
};
/*
************************************************************************
************************************************************************
*/
struct mod_notif_param 
{
	void *p_param;
	unsigned short hi;
	unsigned short lo;

	    /* for event processing */
	u32 event_id;
	unsigned short dev_id;
	unsigned char severity_lvl;
	unsigned char param_count;
};
/*
************************************************************************
************************************************************************
*/
struct _ARCSAS_SGENTRY
{
	ARCSAS_U64	baseAddr;	/*DWord0,DWord1*/
	u32		flags;		/*DWord2:this field value always zero in Marvell PRD table*/
	u32		size;		/*Dword3*/
};
/*
************************************************************************
** Marvell PRD Table: DWord0: Data Base Address Lower 32 bits
**                    DWord1: Data Base Address Upper 32 bits
**                    DWord2: reserved (31:0 reserved)
**                    DWord3: Transfer bytes count (31:22 reserved,21:0 Transfer bytes count)
************************************************************************
*/
struct _sgd_3g_prd
{
	ARCSAS_U64		baseAddr;   /*DWord0,DWord1*/
	u32			Reserved;   /*DWord2:this field value always zero in Marvell PRD table*/
	u32			size;       /*Dword3*/
};
/*
************************************************************************
** ARC1320 PRD Table: DWord0: Data Base Address Lower 32 bits
**                    DWord1: Data Base Address Upper 32 bits
**                    DWord2: struct size_ctl_is
************************************************************************
*/
struct size_ctl_is 
{
#if defined(__BIG_ENDIAN_BITFIELD)
	u32		interface_select:4; /*bit28..31-interface select*/
	u32		sgl_chained:4;	/*bit24..27-misc control,bit24 is sgl chained flag*/
	u32		Reserved:2; 	/*bit22..23-reserved*/
	u32		size:22;		/*bit0...21-transfer byte count,little endian*/
#else
	u32		size:22;		/*bit0...21-transfer byte count,little endian*/
	u32		Reserved:2; 	/*bit22..23-reserved*/
	u32		sgl_chained:4;	/*bit24..27-misc control,bit24 is sgl chained flag*/
	u32		interface_select:4; /*bit28..31-interface select*/
#endif
};
#pragma pack(push, 4)
struct _sgd_6g_prd
{
	ARCSAS_U64	baseAddr;	/*DWord0, DWord1*/
	struct size_ctl_is	sci;		/*DWord2*/   
    /*
    ********************************************
    ** DWord2: size:bit0..21-transfer byte count,
    ** reserved: bit22..23-reserved,
    ** mc:bit24..27-misc control,bit24 is chained,
    ** is:bit28..31-interface select
    ********************************************
    */       
};
#pragma pack(pop)
/*
************************************************************************
**inquiry Vital product data page code
************************************************************************
*/
struct _EVPD_INQUIRYDATA 
{
	unsigned char DeviceType : 5;
	unsigned char DeviceTypeQualifier : 3;

	unsigned char EVPD_PageCode;
	unsigned char Reserved;
	unsigned char EVPD_PageLength;
};
/*
************************************************************************
************************************************************************
*/
struct Identify
{
/* byte 0 */
	#if defined(__BIG_ENDIAN_BITFIELD)
		unsigned char RestrictedByte0Bit7 : 1;
		unsigned char DeviceType          : 3;
		unsigned char AddressFrameType    : 4;
	#else
		/*  0x0: ADDRESS_IDENTIFY_FRAME */
		unsigned char AddressFrameType    : 4;
		/*  001b: End device
		 *  010b: Edge expander device
		 *  011b: Fanout expander device
		 *  Others: Reserved.
		 */
		unsigned char DeviceType          : 3;
		unsigned char RestrictedByte0Bit7 : 1;
	#endif
/* byte 1 */
	unsigned char RestrictedByte1;
/* byte 2 */
	unsigned char InitiatorBits;
/* byte 3 */
	unsigned char TargetBits;
/* byte 4-11 */
	unsigned char DeviceName[8];
/*  byte 12-19  */
	ARCSAS_U64 SASAddress;
/* byte 20 */
	unsigned char PhyIdentifier;
/* byte 21 */
#if defined(__BIG_ENDIAN_BITFIELD)
	unsigned char ReservedByte21Bit1_7 : 7;
	unsigned char BreakReplyCapable    : 1;
#else
	unsigned char BreakReplyCapable    : 1;
	unsigned char ReservedByte21Bit1_7 : 7;
#endif
/* byte 22-27 */
	unsigned char ReservedByte22_27[6];
/* byte 28-31 */
	u32 CRC;
};/* struct Identify */
/*
************************************************************************
**request specific bytes for a general input function 
************************************************************************
*/
struct SMPRequestGeneralInput
{
	/* byte 4-7 */
	u32 CRC;
};
/*
************************************************************************
**request specific bytes for a phy input function
************************************************************************
*/
struct SMPRequestPhyInput
{
	/* byte 4-7 */
	unsigned char IgnoredByte4_7[4];
	/* byte 8 */
	unsigned char ReservedByte8;
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char IgnoredByte10;
	/* byte 11 */
	unsigned char ReservedByte11;
	/* byte 12-15 */
	u32 CRC;
};/* struct SMPRequestPhyInput */
/*
************************************************************************
**request specific bytes for a self configuration expander function 
************************************************************************
*/
struct SMPRequestSelfConfigurationInput
{
	/* byte 4-6 */
	unsigned char IgnoredByte4_6[3];
	/* byte 7 */
	unsigned char StartSelfConfigurationStatusDescriptorIndex;
	/* byte 8-11 */
	u32 CRC;
};/* struct SMPRequestSelfConfigurationInput */
/*
************************************************************************
**request specific bytes for a request route function 
************************************************************************
*/
struct SMPRequestRouteInformationInput
{
	/* byte 4-5 */
	unsigned char IgnoredByte4_5[2];
	/* byte 6-7 */
	unsigned char ExpanderRouteIndex[2];
	/* byte 8 */
	unsigned char ReservedByte8;
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char IgnoredByte10;
	/* byte 11 */
	unsigned char ReservedByte11;	
	/* byte 12-15 */
	u32 CRC;
};/* struct SMPRequestRouteInformationInput */
/*
************************************************************************
**request specific bytes for SMP DISCOVER LIST function
************************************************************************
*/
struct SMPRequestDiscoverList
{
	/* byte 4-7 */
	unsigned char ReservedByte4_7[4];
	/* byte 8 */
	unsigned char StartingPhyIdentifier;
	/* byte 9 */
	unsigned char MaximumNumberOfDescriptors;
	
	/* byte 10 */
	unsigned char PhyFilter:4;
	unsigned char IgnoredByte10Bit4_6:3;
	unsigned char IgnoreZoneGroup:1;
	
	/* byte 11 */
	unsigned char DescriptorType:4;
	unsigned char IgnoredByte11Bit4_7:4;

	/* byte 12-15 */
	unsigned char IgnoredByte12_15[4];

	/* byte 16-27 */
	unsigned char VendorSpecific[12];

	/* byte 28-31 */
	u32 CRC;
};/* struct SMPRequestDiscoverList */
/*
************************************************************************
**request specific bytes for SMP REPORT EXPANDER ROUTE TABLE function 
************************************************************************
*/
struct SMPRequestReportExpanderRouteTable
{
	/* byte 4-7 */
	unsigned char ReservedByte4_7[4];
	/* byte 8-9 */
	unsigned char MaximumNumberOfDescriptors[2];
	
	/* byte 10-15 */
	unsigned char StartingRoutedSASAddressIndex[6];

	/* byte 16-18 */
	unsigned char IgnoredByte16_18[3];

	/* byte 19 */
	unsigned char StartingPhyIdentifier;

	/* byte 20-27 */
	unsigned char IgnoredByte20_27[8];

	/* byte 28-31 */
	u32 CRC;
};/* struct SMPRequestReportExpanderRouteTable */
/*
************************************************************************
**request specific bytes for SMP CONFIGUARE GENERAL function
************************************************************************
*/
struct SMPRequestConfigureGeneral
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char IgnoredByte6_7[2];
	
	/* byte 8 */
	unsigned char UpdateSTPBusInactivityTimeLimit:1;
	unsigned char UpdateSTPMaxConnectTimeLimit:1;
	unsigned char UpdateSTPSMPNexusLossTime:1;
	unsigned char IgnoredByte8Bit3_7:5;
	
	/* byte 9 */
	unsigned char IgnoredByte9;
	
	/* byte 10-11 */
	unsigned char STPBusInactivityTimeLimit[2];
	/* byte 12-13 */
	unsigned char STPMaxConnectTimeLimit[2];
	
	/* byte 14-15 */
	unsigned char STPSMPNexusLossTime[2];

	/* byte 16-19 */
	u32 CRC;
};/* struct SMPRequestConfigureGeneral */
/*
************************************************************************
**request specific bytes for ENABLE DISABLE ZONING function
************************************************************************
*/
struct SMPRequestEnableDisableZoning
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	
	/* byte 6-7 */
	unsigned char IgnoredByte4_5[2];

	/* byte 8 */
	unsigned char EnableDisableZoning:1;
	unsigned char ReservedByte8Bit1_7:7;

	/* byte 9-11 */
	unsigned char IgnoredByte9_11;

	/* byte 12-15 */
	u32 CRC;
};/* struct SMPRequestEnableDisableZoning */
/*
************************************************************************
**request specific bytes for SMP ZONED BROADCAST function
************************************************************************
*/
struct SMPRequestZonedBroadcast
{
	/* byte 4-5 */
	unsigned char RestrictByte4_5[2];
	/* byte 6 */
	unsigned char BroadcastType:3;
	unsigned char ReservedByte6Bit3_7:5;
	/* byte 7 */
	unsigned char NumberOfBroadcastSourceZoneGroups;
	/* byte 8-135 */
	unsigned char BroadcastSourceZoneGroup[0x80];	//TBD 
	/* value between 0 and 127 */
	/* 
		byte 135 -n: PAD field
		Contains zero,one,two or three bytes set to 0x00 such that 
		total length of the SMP request is a multiple of four
	*/
	unsigned char Pad[3];	
	/* byte n-4-n */
	u32 CRC;
};/* struct SMPRequestZonedBroadcast */
/*
************************************************************************
**request specific bytes for SMP ConfigureRouteInformation function 
************************************************************************
*/
struct SMPRequestConfigureRouteInformation
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char ExpanderRouteIndex[2];
	/* byte 8 */
	unsigned char ReservedByte8;
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10-11 */
	unsigned char ReservedByte10_11[2];
	/* byte 12 */
	unsigned char IgnoredByte12Bit0_6:7;
	unsigned char DisableRouteEntry:1; 	
	/* if a routing error is detected
	then the route is disabled by
	setting this bit*/
	/* byte 13-15 */
	unsigned char IgnoredByte13_15[3];
	/* byte 16-23 */
	unsigned char RoutedSASAddress[8];	
	/*identical to the AttachedSASAddress
	found through discovery*/
	/* byte 24-39 */
	unsigned char ReservedByte24_39[16];
	/* byte 40-43 */
	u32 CRC;
};/* struct SMPRequestConfigureRouteInformation */
/*
************************************************************************
**request specific bytes for SMP Phy Control function 
************************************************************************
*/
struct SMPRequestPhyControl
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char IgnoredByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char PhyOperation;
	/* byte 11 */
	unsigned char UpdatePartialPathwayTimeoutValue:1;
	unsigned char ReservedByte11Bit1_7:7;
	/* byte 12-31 */
	unsigned char IgnoredByte12_31[20];
	/* byte 32 */
	unsigned char IgnoredByte32Bit0_3:4;
	unsigned char ProgrammedMinimumPhysicalLinkRate:4;
	/* byte 33 */
	unsigned char IgnoredByte33Bit0_3:4;
	unsigned char ProgrammedMaximumPhysicalLinkRate:4;
	/* byte 34-35 */
	unsigned char IgnoredByte34_35[2];
	/* byte 36 */
	unsigned char PartialPathwayTimeoutValue:4;
	unsigned char ReservedByte36Bit4_7:4;
	/* byte 37-39 */
	unsigned char ReservedByte37_39[3];
	/* byte 40-43 */
	u32 CRC;
};/* struct SMPRequestPhyControl */
/*
************************************************************************
** request specific bytes for SMP Phy Test function 
************************************************************************
*/
struct SMPRequestPhyTest
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char IgnoredByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char PhyTestFunction;
	/* byte 11 */
	unsigned char PhyTestPattern;
	/* byte 12-14 */
	unsigned char ReservedByte12_14[3];
	/* byte 15 */
	unsigned char PhyTestPatternPhysicalLinkRate:4;
	unsigned char ReservedByte15Bit4_7:4;
	/* byte 16-18 */
	unsigned char ReservedByte16_18[3];
	/* byte 19 */
	unsigned char PhyTestPatternDwordsControl;
	/* byte 20-27 */
	unsigned char PhyTestPatternDwords[8];
	/* byte 28-39 */
	unsigned char ReservedByte28_39[12];	
	/* byte 40-43 */
	u32 CRC;
};/* struct SMPRequestPhyTest */
/*
************************************************************************
**request specific bytes for SMP CONFIGUARE PHY EVENT INFORMATION function 
************************************************************************
*/
struct SMPRequestConfigurePhyEventInformation
{
	/* byte 4-5 */
	unsigned char ExpectedExpanderChangeCount[2];
	/* byte 6 */
	unsigned char ClearPeaks:1;
	unsigned char ReservedByte6Bit1_7:6;
	/* byte 7-8 */
	unsigned char ReservedByte7_8[2];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char ReservedByte10;
	/* byte 11 */
	unsigned char NumberOfPhyEventConfigurationDescriptors;
	/* byte 11-138 */
	unsigned char PhyEventConfigurationDescriptors[0x80];	//TBD
	u32 CRC;
};/* struct SMPRequestConfigurePhyEventInformation */

/* SGPIO Registers that are available */
#define SGPIO_Vendor_Unique2_1300	0x44 /* Register name used in Spec */
#define SGPIO_Init_1300			SGPIO_Vendor_Unique2_1300 /*Name as appeared in code */
#define SGPIO_Control_1300		0x64
#define SGPIO_Data_Out_L_1300		0x68
#define SGPIO_Data_Out_H_1300		0x6C
#define SGPIO_Data_In_L_1300		0x70
#define SGPIO_Data_In_H_1300		0x74
/* For setting SGPIO mode during initializaton */
#define SGPIO_Mode_1300			ARCSAS_BIT(1, 7)
/* For setting SGPIO_Control Register */
#define SGPIO_TRANSFER_LEN_1300(x)	((x - 1) << 4)
#define SGPIO_SELECT_SPEED_1300(x)	(x << 1)
#define SGPIO_START_BIT_1300		ARCSAS_BIT(1, 0)
/* Clk Frequency availabe for SGPIO */
#define SGPIO_1KHz_1300			0
#define SGPIO_10KHz_1300		1
#define SGPIO_50KHz_1300		2
#define SGPIO_100KHz_1300		3

#define SGPIO_REG_CONFIG0_1320			0xC200	/* R0C200h/R0C300h */
#define SGPIO_REG_CONFIG1_1320			0xC204 /* R0C204h/R0C304h */
#define SGPIO_REG_CONFIG2_1320			0xC208 /* R0C208h/R0C308h */
#define SGPIO_REG_CONTROL_1320			0xC20C /* R0C20Ch/R0C30Ch */
#define SGPIO_REG_INT_CAUSE_1320		0xC210 /* R0C210h/R0C310h */
#define SGPIO_REG_INT_ENABLE_1320		0xC214 /* R0C214h/R0C314h */
#define SGPIO_REG_DRIVE_SOURCE_BASE_1320	0xC220 /* R0C220h/R0C320h ,0xC220...0xC234 , 0xC320...0xC334 ,Drive Source  0..23 */
#define SGPIO_REG_DRIVE_CONTROL_BASE_1320	0xC238 /* R0C238h/R0C338h ,0xC238...0xC24C , 0xC338...0xC34C ,Drive Control 0..23 */
#define SGPIO_REG_RAW_DOUT0_1320		0xC250 /* R0C250h/R0C350h ,DOut00..31 */
#define SGPIO_REG_RAW_DOUT1_1320		0xC254 /* R0C254h/R0C354h ,DOut32..63 */
#define SGPIO_REG_RAW_DOUT2_1320		0xC258 /* R0C258h/R0C358h ,DOut64..95 */
#define SGPIO_REG_RAW_DIN0_1320			0xC25C /* R0C25Ch/R0C35Ch ,DIn00...31 */
#define SGPIO_REG_RAW_DIN1_1320			0xC260 /* R0C260h/R0C360h ,DIn32...63 */
#define SGPIO_REG_RAW_DIN2_1320			0xC264 /* R0C264h/R0C364h ,DIn64...95 */
#define SGPIO_REG_ADDR_1320(sgpioX, addrY)	(addrY + sgpioX * 0x100) /* sgpioX=0(SGPIO0) or 1(SGPIO1) , addrY = SGPIO_REG_CONFIG0_1320,... */
#define SGPIO0				0   
#define SGPIO1				1
#define SGPIO_MINIMUM_BIT_STREAM	12
/* SGPIO_REG_CONFIG0_1320 */
#define SGPIO_EN			ARCSAS_BIT(1, 0)
#define BLINK_GEN_EN_B			ARCSAS_BIT(1, 1)/*blink generator A enable*/
#define BLINK_GEN_EN_A			ARCSAS_BIT(1, 2)/*blink generator B enable*/
#define INVRT_SCLK			ARCSAS_BIT(1, 3)
#define INVRT_SLOAD			ARCSAS_BIT(1, 4)
#define INVRT_SDOUT			ARCSAS_BIT(1, 5)
#define NEG_SLOAD_EDGE			ARCSAS_BIT(1, 6)
#define NEG_SDOUT_EDGE			ARCSAS_BIT(1, 7)
#define POS_SDIN_EDGE			ARCSAS_BIT(1, 8)
#define MANUAL_BIT_LEN			0x00ff0000
#define AUTO_BIT_LEN			0xff000000
#define MANUAL_BIT_LEN_OFFSET		16
#define AUTO_BIT_LEN_OFFSET		24

/* SGPIO_REG_CONFIG1_1320 */
#define BLINK_LOW_TM_A			0x0000000f
#define BLINK_HI_TM_A			0x000000f0
#define BLINK_LOW_TM_B			0x00000f00
#define BLINK_HI_TM_B			0x0000f000
#define MAX_ACTV_ON			0x000f0000
#define FORCE_ACTV_OFF			0x00f00000
#define STRCH_ACTV_ON			0x0f000000
#define STRCH_ACTV_OFF			0xf0000000

/* SGPIO_REG_CONFIG2_1320 */
#define SCLK_SELECT			0x000000ff /* 32Hz..100kHz */

/* SGPIO_REG_CONTROL_1320 */
#define SDIN_MODE_MASK			(0x3 << 0)

#define SDOUT_MODE_MASK			(0x3 << 2)

#define MANUAL_MODE_REP_CNT_MASK	(0xfff << 4)

#define MANUAL_MODE_SLOAD_PTRN_MASK	(0xf<<16)

#define AUTO_MODE_SLOAD_PTRN_MASK	(0xf<<20)

#define MAUNAL_MODE_REP_CNT_OFFSET	4
enum {
	SDOUT_MODE_SCLK_HALT = 0,
	SDOUT_MODE_MANUAL = 1,/*MANUAL_MODE_REP_CNT count down to stop and halt*/
	SDOUT_MODE_AUTO = 2,/*does not self-clear*/
	SDOUT_MODE_MANUAL_AUTO = 3,/*MANUAL_MODE_REP_CNT count down to stop and transition to auto*/
	SDIN_MODE_DISABLE = 0,
	SDIN_MODE_ONCE = 1,/*one shot mode,self-clear when capture is complete*/
	SDIN_MODE_CONT = 2,/*continue mode, does not self-clear*/
};
#define SDOUT_MODE_OFFSET		2

/* SGPIO_REG_INT_CAUSE_1320 */
#define SDIN_DONE			ARCSAS_BIT(1, 0)
#define MANUAL_MODE_REP_DONE		ARCSAS_BIT(1, 1)
#define MANUAL_MODE_REP_REMAIN		(0xfff << 8)

/* SGPIO_REG_DRIVE_SOURCE_BASE_1320 */
#define DRIVE_SOURCE_PHY0_A		0x00
#define DRIVE_SOURCE_SAS_PORT0_A	0x08
#define DRIVE_SOURCE_SAS_PORT4_A	0x10
#define DRIVE_SOURCE_PM_PHY0_A		DRIVE_SOURCE_SAS_PORT0_A

/* core_index=0(A) or 1(B), phy_index=phy number 0..3 */
#define DRIVE_SOURCE_PHY(core_index,phy_index)			(DRIVE_SOURCE_PHY0_A + phy_index + core_index * 4) 
/* core_index=0(A) or 1(B), phy_index=phy number 0..3 */
#define DRIVE_SOURCE_SAS_PORT0_3(core_index,phy_index)		(DRIVE_SOURCE_SAS_PORT0_A + phy_index + core_index * 4) 
/* core_index=0(A) or 1(B), phy_index=phy number 4..7 */
#define DRIVE_SOURCE_SAS_PORT4_7(core_index,phy_index)		(DRIVE_SOURCE_SAS_PORT4_A + phy_index + core_index * 4) 
/* core_index=0(A) or 1(B), phy_index=phy number 0..3, pm_port_index=PM port id */
#define DRIVE_SOURCE_PM_PHY(core_index,phy_index,pm_port_index)   (DRIVE_SOURCE_PM_PHY0_A + phy_index + (core_index * 4) + (pm_port_index * 8)) 

/* SGPIO_REG_DRIVE_CONTROL_BASE_1320 */
#define MAX_AUTO_CTRL_DWORD		6

#define DRIVE_ACTIVITY_LED_MASK		(0x7 << 5)
#define DRIVE_LOCATE_LED_MASK		(0x3 << 3)
#define DRIVE_ERROR_LED_MASK		0x7
#define DRIVE_ACTIVITY_LED_OFFSET	5
#define DRIVE_LOCATE_LED_OFFSET		3
#define DRIVE_ERROR_LED_OFFSET		0
/* 
************************************************************************
** LED light types 
************************************************************************
*/
#define SGPIO_LED_TYPE_ACTIVITY		0
#define SGPIO_LED_TYPE_LOCATE		1
#define SGPIO_LED_TYPE_ERROR		2
#define SGPIO_LED_TYPE_REBUILD		3		/* for Supermicro backplanes */
#define SGPIO_LED_TYPE_HOT_SPARE	4
#define SGPIO_LED_TYPE_ALL_OFF		0xff
#define SGPIO_LED_REBUILD_MASK		0x00000080
#define SGPIO_LED_ERROR_MASK		0x00000040
/*
************************************************************************
** LED light status - the following values are set according to hardware
**                    values, do not change unless necessary
************************************************************************
*/
enum {
	SGPIO_LED_STATUS_LOW			= 0,
	SGPIO_LED_STATUS_HIGH			= 1,
	SGPIO_LED_STATUS_BLINK_A		= 2,
	SGPIO_LED_STATUS_BLINK_INVRT_A		= 3, /*above for loc 0..3*/
	SGPIO_LED_STATUS_BLINK_SOF		= 4, /*only on active*/
	SGPIO_LED_STATUS_BLINK_EOF		= 5, /*only on active*/
	SGPIO_LED_STATUS_BLINK_B		= 6,
	SGPIO_LED_STATUS_BLINK_INVRT_B		= 7,
};

/* pre-set flags for RAID module to use */
#define LED_FLAG_REBUILD		0x01
#define LED_FLAG_HOT_SPARE		0x02
#define LED_FLAG_FAIL_DRIVE		0x03
#define LED_FLAG_ACTIVE			0x04
#define LED_FLAG_WORKING_CHECK		0x10
#define LED_FLAG_REBUILD_OFF		0x11
#define LED_FLAG_HOT_SPARE_OFF		0x12
#define LED_FLAG_FAIL_DRIVE_OFF		0x13
#define LED_FLAG_ALL_OFF		0xff

/* SGPIO_REG_RAW_DIN0_1320 SGPIO_REG_RAW_DOUT0_1320 */
#define MAX_SDIN_DWORD			3
#define MAX_SDOUT_DWORD			3
#define SDIN_BACK_PLAN_PRESENCE_PATTERN	0x492
#define SDIN_DATA_MASK				0xFFF
#define SDIN_DEVICE0_PRESENCE_PATTERN		0x3
#define SDIN_DEVICE1_PRESENCE_PATTERN		0x18
#define SDIN_DEVICE2_PRESENCE_PATTERN		0xC0
#define SDIN_DEVICE3_PRESENCE_PATTERN		0x600
/* application */
enum {
	REG_TYPE_CONFIG		= 0,
	REG_TYPE_RX		= 1,
	REG_TYPE_RX_GP		= 2,
	REG_TYPE_TC		= 3,
	REG_TYPE_TC_GP		= 4,
};
/*
************************************************************************
**	request sent specific to SMP Read SGPIO
**	Corresponds to READ_GPIO_REGISTER define
************************************************************************
*/
struct SMPRequestReadGPIORegister
{
	/* byte 4 */
	unsigned char RegisterCount;
	/* byte 5-7 */
	unsigned char ReservedByte[3];
	u32 CRC;
};
/*
************************************************************************
**	request sent specific to SMP Write SGPIO
**	Corresponds to WRITE_GPIO_REGISTER define
************************************************************************
*/
struct SMPRequestWriteGPIORegister
{
	/* byte 4 */
	unsigned char RegisterCount;
	/* byte 5-7 */
	unsigned char ReservedByte[3];
	/* byte 8-15 */
	unsigned char Data_Out_Low[8];
	/* byte 16-23 */
	unsigned char Data_Out_High[8];
	u32 CRC;
};
/*
************************************************************************
**	generic structure referencing an SMP Request, must be initialized
**	before being used
************************************************************************
*/
struct _SMPRequest
{
	/* byte 0 */
	unsigned char SMPFrameType;		/* always SMP_REQUEST_FRAME = 0x40 */
	/* byte 1 */
	unsigned char Function; 
	/* byte 2-3 */
	unsigned char ReservedByte2_3[2];
	/* bytes 4-n */
	union
	{
		/* #define REPORT_GENERAL 			0x00 */
		struct SMPRequestGeneralInput 				ReportGeneral;
		/* #define REPORT_MANUFACTURER_INFORMATION 	0x01 */
		struct SMPRequestGeneralInput 				ReportManufacturerInformation;
		/* #define READ_GPIO_REGISTER			0x02 */	
		struct SMPRequestReadGPIORegister			ReadGPIORegister;
		/* #define REPORT_SELF_CONFIGURATION_STATUS	0x03 */
		struct SMPRequestSelfConfigurationInput			ReportSelfConfigurationStatus;
		/* #define DISCOVER 				0x10 */
		struct SMPRequestPhyInput 				Discover;
		/* #define REPORT_PHY_ERROR_LOG 		0x11 */
		struct SMPRequestPhyInput 				ReportPhyErrorLog;
		/* #define REPORT_PHY_SATA 			0x12 */
		struct SMPRequestPhyInput 				ReportPhySATA;
		/* #define REPORT_ROUTE_INFORMATION 		0x13 */
		struct SMPRequestRouteInformationInput 			ReportRouteInformation;
		/* #define REPORT_PHY_EVENT_INFORMATION		0x14 */
		struct SMPRequestPhyInput				ReportPhyEventInformation;
		/* #define REPORT_PHY_BROADCAST_COUNTS		0x15 */
		struct SMPRequestPhyInput				ReportPhyBroadcaseCounts;
		/* #define DISCOVER_LIST			0x16 */
		struct SMPRequestDiscoverList				DiscoverList;	
		/* #define REPORT_EXPANDER_ROUTE_TABLE		0x17 */
		struct SMPRequestReportExpanderRouteTable		ReportExpanderRouteTable;
		/* #define CONFIGURE_GENERAL			0x80 */
		struct SMPRequestConfigureGeneral			ConfigureGeneral;
		/* #define ENABLE_DISABLE_ZONING		0x81 */
		struct SMPRequestEnableDisableZoning			EnableDisableZoning;
		/* #define WRITE_GPIO_REGISTER			0x82 */	
		struct SMPRequestWriteGPIORegister			WriteGPIORegister;
		/* #define ZONED_BROADCAST			0x85 */
		struct SMPRequestZonedBroadcast				ZonedBroadcast;
		/* #define CONFIGURE_ROUTE_INFORMATION 		0x90 */
		struct SMPRequestConfigureRouteInformation 		ConfigureRouteInformation;
		/* #define PHY_CONTROL 				0x91 */
		struct SMPRequestPhyControl 				PhyControl;
		/* #define PHY_TEST 				0x92 */
		struct SMPRequestPhyTest 				PhyTest;
		/* #define CONFIGURE_PHY_EVENT_INFORMATION	0x93 */
		struct SMPRequestConfigurePhyEventInformation		ConfigurePhyEventInformation;
	} Request;
};/* struct SMPRequest */
/*
************************************************************************
**	request specific bytes for SMP Report General response, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportGeneral
{
/* 
	byte 4-5: EXPANDER CHANGE COUNT
	Counts the number of Broadcast (Change)s originated by an expander device. 
	Management device servers in expander devices shall support this field. 
	Management device servers in other device types (e.g., end devices) shall 
	set this field to 0000h. This field shall be set to at least 0001h at power 
	on. If the expander device has originated Broadcast (Change) for any reason 
	described in 7.11 since transmitting a REPORT GENERAL response, it shall increment
	this field at least once from the value in the previous REPORT GENERAL response. 
	It shall not increment this field when forwarding a Broadcast (Change). This field 
	shall wrap to at least 0001h after the maximum value (i.e., FFFFh) has been reached.
*/
	unsigned char ExpanderChangeCount[2];
/*
	byte 6-7: EXPANDER ROUTE INDEXES
	Contains the maximum number of expander route indexes per phy for the expander 
	device. Management device servers in externally configurable expander devices
	containing phy-based expander route tables shall support this field. Management
	device servers in other device types (e.g., end devices, externally configurable 
	expander devices with expander-based expander route tables, and self-configuring 
	expander devices) shall set the EXPANDER ROUTE INDEXES field to zero. Not all 
	phys in an externally configurable expander device are required to support the 
	maximum number indicated by this field.
*/
	unsigned char ExpanderRouteIndexes[2];
/* 	byte 8: reserved */
	unsigned char ReservedByte8;
/* 
	byte 9:NUMBER OF PHYS
	Contains the number of phys in the device, including any virtual phys and any
	vacant phys
*/
	unsigned char NumberOfPhys;
/* 

	byte 10-bit0:
	EXTERNALLY CONFIGURABLE ROUTE TABLE
	Set to one indicates that the management device server is in an externally 
	configurable expander device that has a phy-based expander route table 
	that is required to be configured with the SMP CONFIGURE ROUTE INFORMATION 
	function. An EXTERNALLY CONFIGURABLE ROUTE TABLE bit set to zero indicates 
	that the management device server is not in an externally configurable expander 
	device (e.g., it is in an end device, in a self-configuring expander device, 
	or in an expander device with no phys with table routing attributes).

	byte 10-bit1:
	CONFIGURING
	Set to one indicates that the management device server is in a self-configuring 
	expander device, the self-configuring expander device's management application 
	client is currently performing the discover process, and it has identified at 
	least one change to its expander routing table. A CONFIGURING bit set to zero 
	indicates that the management device server is not in a self-configuring expander
	device currently performing the discover process and changing its expander 
	routing table. Changes in this bit from one to zero result in a Broadcast (Change)
	being originated. Management device servers in self-configuring expander 
	devices shall support this bit. Management device servers in externally 
	configurable expander devices and in other device types shall set the CONFIGURING 
	bit to zero.

	byte 10-bit2:
	CONFIGURES OTHERS
	Set to one indicates that the expander device is a self-configuring expander 
	device that performs the configuration subprocess. A CONFIGURES OTHERS bit set 
	to zero indicates the expander device may or may not perform the configuration 
	subprocess. Self-configuring expander devices compliant with this standard 
	shall set the CONFIGURES OTHERS bit to one.If the CONFIGURES OTHERS bit is set 
	to zero, the expander device may configure all externally configurable expander 
	devices in the SAS domain.
	
	byte 10-bit7:
	TABLE TO TABLE SUPPORTED
	Set to one indicates the expander device is a self-configuring expander
	device that supports its table routing phys being attached to table routing
	phys in other expander devices. The TABLE TO TABLE SUPPORTED bit shall only 
	be set to one if the EXTERNALLY CONFIGURABLE ROUTE TABLE bit is set to
	zero. A TABLE TO TABLE SUPPORTED bit set to zero indicates the expander device
	is not a self-configuring expander device that supports its table routing
	phys being attached to table routing phys in other expander devices.

*/
	unsigned char ConfigurableRouteTable:1;		/* BIT 0*/
	unsigned char Configuring:1;				/* BIT 1*/
	unsigned char ConfiguresOthers:1;			/* BIT 2*/
	unsigned char ReservedByte10Bit3_6:4;		/* BIT 3-6*/
	unsigned char TableToTableSupported:1;		/* BIT 7*/
/* 	byte 11: reserved */
	unsigned char ReservedByte11;
/* 
	byte 12-19:
	ENCLOSURE LOGICAL IDENTIFIER
	Identifies the enclosure, if any, in which the device is located, and is
	defined in SES-2. The ENCLOSURE LOGICAL IDENTIFIER field shall be set to 
	the same value reported by the enclosure services process, if any, for 
	the enclosure. An ENCLOSURE LOGICAL IDENTIFIER field set to zero indicates 
	no enclosure information is available.
*/
	unsigned char EnclosureLogicalIdentifier[8];
/* 	byte 20-29: reserved */
	unsigned char ReservedByte20_29[10];

/* 
	byte 30-31:
	STP BUS INACTIVITY TIME LIMIT
	Contains the bus inactivity time limit for STP connections which is set by
	the CONFIGURE GENERAL function
*/
	unsigned char STPBusInactivityTimeLimit[2];
/* 
	byte 32-33:
	STP MAXIMUM CONNECT TIME LIMIT
	Contains the maximum connect time limit for STP connections
	which is set by the CONFIGURE GENERAL function.
*/
	unsigned char STPMaxConnectTimeLimit[2];
/* 
	byte 34-35:
	STP SMP I_T NEXUS LOSS TIME
	Contains the time that an STP target port and an SMP initiator port retry
	certain connection requests which is set by the CONFIGURE GENERAL function.
*/
	unsigned char STPSMPNexusLossTime[2];
/*
	byte 36-bit0:
	ZONING ENABLED
	Set to one indicates that zoning is enabled in the expander device. A ZONING 
	ENABLED bit set to zero indicates that zoning is disabled in the expander 
	device. The ZONING ENABLED bit shall be set to zero if the ZONING SUPPORTED bit 
	is set to zero.

	byte 36-bit1:
	ZONING SUPPORTED
	Set to one indicates that zoning is supported by the expander device(i.e., it 
	is a zoning expander device). A ZONING SUPPORTED bit set to zero indicates that 
	zoning is not supported by the expander device.

	byte 36-bit2:
	PHYSICAL PRESENCE ASSERTED
	Set to one indicates that the expander device is currently detecting
	physical presence. A PHYSICAL PRESENCE ASSERTED bit set to zero indicates that 
	the expander device is not currently detecting physical presence. The PHYSICAL 
	PRESENCE ASSERTED bit shall be set to zero if the PHYSICAL PRESENCE SUPPORTED 
	bit is set to zero.

	byte 36-bit3:
	PHYSICAL PRESENCE SUPPORTED
	Set to one indicates that the expander device supports physical presense
	as a mechanism for allowing zoning to be enabled or disabled from phys in 
	zone groups without access to zone group 2. A PHYSICAL PRESENCE SUPPORTED bit 
	set to zero indicates that the expander device does not support physical 
	presence as a mechanism for allowing zoning to be enabled or disabled.
	
*/
	unsigned char ZoningEnabled:1;			/* Bit 0*/
	unsigned char ZoningSupported:1;		/* Bit 1*/
	unsigned char PhysicalPresenceAsserted:1;	/* Bit 2*/	
	unsigned char PhysicalPresenceSupported:1;	/* Bit 3*/
	unsigned char ReservedByte36Bit4_7:4;		/* Bit 4-7*/
/* 	byte 37: reserved */
	unsigned char ReservedByte37;
/* 
	byte 38-39:
	MAXIMUM NUMBER OF ROUTED SAS ADDRESSES
	Contains the number of routed SAS addresses in an expander-based expander route
	table. Management device servers in expander devices containing expander-based 
	expander route tables shall support this field. Management device servers in 
	other device types (e.g., end devices and expander devices with phy-based expander 
	route tables) shall set this field to 0000h.
*/
	unsigned char MaxNumRoutedSASAddr[2];
/* 	byte 40-43 CRC */	
	u32 CRC;
};/* struct SMPResponseReportGeneral */
/*
************************************************************************
**	request specific bytes for SMP Report Manufacturer Information response,
**	intended to be referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportManufacturerInformation
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char ReservedByte6_7[2];
	/* byte 8	*/
	unsigned char SAS11Format:1;
	unsigned char ReservedByte8_Bit1_7:7;
	/* byte 9-11 */
	unsigned char IgnoredByte9_11[3];
	/* byte 12-19 */
	unsigned char VendorIdentification[8];
	/* byte 20-35 */
	unsigned char ProductIdentification[16];
	/* byte 36-39 */
	unsigned char ProductRevisionLevel[4];
	/* byte 40-47 */
	unsigned char ComponentVendorIdentification[8];
	/* byte 48-49 */
	unsigned char ComponentID[2];
	/* byte 50 */
	unsigned char ComponentRevisionID;
	/* byte 51 */
	unsigned char ReservedByte51;
	/* byte 52-59 */
	unsigned char VendorSpecific[8];
	/* byte 60-63 */
	u32 CRC;
};
/*
************************************************************************
**	request specific bytes for SMP Report Self-Configuration Status response,
**	intended to be referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportSelfConfigurationStatus
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6 */
	unsigned char ReservedByte6;
	/* byte 7 */
	unsigned char StartingSelfConfigurationStatusDescriptorIndex;
	/* byte 8-12	*/
	unsigned char ReservedByte8_12[5];
	/* byte 13 */
	unsigned char MaxSupportedSelfConfigurationStatusDescriptors;
	/* byte 14 */
	unsigned char TotalNumberOfSelfConfigurationStatusDescriptors;
	/* byte 15 */
	unsigned char NumberOfSelfConfigurationStatusDescriptors;
	/* byte 16-31 */
	unsigned char SelfConfigurationStatusDescriptor[16];	//TBD
	/*
	byte SelfConfigurationStatusDescriptor_1[16];
		...
	byte SelfConfigurationStatusDescriptor_n[16];
		
	*/
	u32 CRC;
};
/*
************************************************************************
**	request specific bytes for SMP Report Phy Error Log response,
**	intended to be referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportPhyErrorLog
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char ReservedByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10-11 */
	unsigned char ReservedByte10_11[2];
	/* byte 12-15 */
	unsigned char InvalidDwordCount[4];
	/* byte 16-19 */
	unsigned char RunningDisparityErrorCount[4];
	/* byte 20-23 */
	unsigned char LossOfDwordSynchronizationCount[4];
	/* byte 24-27 */
	unsigned char PhyResetProblemCount[8];
	/* byte 28-31 */
	u32 CRC;
};
/*
************************************************************************
**	Response specific bytes for SMP Discover Descriptor, intended to be 
**	referenced by SMPResponseDiscover, SMPResponseDiscoverList
************************************************************************
*/
struct DiscoverLongFormatDescriptor
{
	unsigned char ReservedByte48_49[2];
	unsigned char VendorSpecific[2];
	unsigned char AttachedDeviceName[8];
	unsigned char ZoningEnabled:1;				/* bit 0 */
	unsigned char InsideZPSDS:1;				/* bit 1 */
	unsigned char ZoneGroupPersistent:1;			/* bit 2 */
	unsigned char ZoneAddressResolved:1;			/* bit 3 */	
	unsigned char RequestedInsideZPSDS:1;			/* bit 4 */
	unsigned char InsideZPSDSPersistent:1;			/* bit 5 */
	unsigned char RequestedInsideZPSDSChangedbyExpander:1;	/* bit 6 */
	unsigned char ReservedByte60Bit7:1;
	unsigned char ReservedByte61_62[2];
	unsigned char ZoneGroup;
	unsigned char SelfConfigurationStatus;
	unsigned char SelfConfgurationLevelsCompleted;
	unsigned char ReservedByte66_67[2];
	unsigned char SelfConfigurationSASAddress[8];
};/* DiscoverLongFormatDescriptor */
/*
************************************************************************
** Response specific bytes for SMP Discover, intended to be referenced
** by SMPResponse
************************************************************************
*/
struct SMPResponseDiscover
{
	unsigned char ExpanderChangeCount[2];
	unsigned char ReservedByte6_8[3];
	unsigned char PhyIdentifier;
	unsigned char ReservedByte10_11[2];
	unsigned char ReservedByte12Bit0_3:4;		/* Bit0-3 */
	unsigned char AttachedDeviceType:3;		/* Bit4-6 */
	unsigned char IgnoredByte12Bit7:1;		/* Bit7 */
	unsigned char NegotiatedPhysicalLinkRate:4;
	unsigned char ReservedByte13Bit4_7:4;
	unsigned char InitiatorBits;
	unsigned char TargetBits;
	unsigned char SASAddress[8];
	unsigned char AttachedSASAddress[8];
	unsigned char AttachedPhyIdentifier;
	#if defined(__BIG_ENDIAN_BITFIELD)
		unsigned char ReservedByte32Bit3_7 : 5;
		unsigned char AttachedInsideZPSDSPersistent : 1;	/* Bit 2 */
		unsigned char AttachedRequestedInsideZPSDS : 1;		/* Bit 1 */
		unsigned char AttachedBreakReplyCapable : 1;		/* Bit 0 */
	#else
		unsigned char AttachedBreakReplyCapable : 1;		/* Bit 0 */
		unsigned char AttachedRequestedInsideZPSDS : 1;		/* Bit 1 */
		unsigned char AttachedInsideZPSDSPersistent : 1;	/* Bit 2 */
		unsigned char ReservedByte32Bit3_7 : 5;
	#endif
	unsigned char ReservedByte34_39[6];
	unsigned char HardwareMinimumPhysicalLinkRate:4;
	unsigned char ProgrammedMinimumPhysicalLinkRate:4;
	unsigned char HardwareMaximumPhysicalLinkRate:4;
	unsigned char ProgrammedMaximumPhysicalLinkRate:4;
	unsigned char PhyChangeCount;
	unsigned char PartialPathwayTimeoutValue:4;
	unsigned char IgnoredByte36Bit4_6:3;
	unsigned char VirtualPhy:1;
	unsigned char RoutingAttribute:4;
	unsigned char ReservedByte44Bit4_7:4;
	unsigned char ConnectorType:7;
	unsigned char ReservedByte45Bit7:1;
	unsigned char ConnectorElementIndex;
	unsigned char ConnectorPhysicalLink;
	struct DiscoverLongFormatDescriptor	LongFormatDescriptor;	
	u32 CRC;
};
/*
************************************************************************
**	this structure describes the Register Device to Host FIS defined in the
**	SATA specification
************************************************************************
*/
struct _SATA_FIS_REG_D2H
{
	unsigned char FISType;
	unsigned char ReservedByte25Bit0_5:6;
	unsigned char Interrupt:1;
	unsigned char ReservedByte25Bit7:1;
	unsigned char Status;
	unsigned char Error;
	unsigned char SectorNumber;
	unsigned char CylLow;
	unsigned char CylHigh;
	unsigned char DevHead;
	unsigned char SectorNumberExp;
	unsigned char CylLowExp;
	unsigned char CylHighExp;
	unsigned char ReservedByte35;
	unsigned char SectorCount;
	unsigned char SectorCountExp;
	unsigned char ReservedByte38_43[6];
};/* struct _SATA_FIS_REG_D2H */
/*
************************************************************************
**	request specific bytes for SMP Report PHY SATA response,
**	intended to be referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportPhySATA
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char ReservedByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10 */
	unsigned char ReservedByte10;
	/* byte 11 */
	unsigned char AffiliationValid:1;
	unsigned char AffiliationSupported:1;
	unsigned char STPNexusLossOccurred:1;
	unsigned char ReservedByte11Bit3_7:5;
	/* byte 12-15 */
	unsigned char ReservedByte10_15[4];
	/* byte 16-23 */
	unsigned char STPSASAddress[8];	
	/* 24-43 */
	struct _SATA_FIS_REG_D2H FIS;
	/* byte 44-47 */
	unsigned char ReservedByte44_47[4];
	/* byte 48-55 */
	unsigned char AffiliatedSTPInitiatorSASAddress[8];
	/* byte 56-63 */
	unsigned char STPNexusLossSASAddress[8];
	/* byte 64-67 */
	u32 CRC;
};/* struct SMPResponseReportPhySATA */
/*
************************************************************************
**	response specific bytes for SMP Report Route Information, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportRouteInformation
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char ExpanderRouteIndex[2];
	/* byte 8 */
	unsigned char ReservedByte8;
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10-11 */
	unsigned char ReservedByte10_11[2];
	/* byte 12 */
	unsigned char IgnoredByte12Bit0_6:7;
	unsigned char ExpanderRouteEntryDisabled:1;
	/* byte 13-15 */
	unsigned char IgnoredByte13_15[3];
	/* byte 16-23 */
	unsigned char RoutedSASAddress[8];
	/* byte 24-39 */
	unsigned char ReservedByte24_39[16];
	/* byte 40-43 */
	u32 CRC;
};/* struct SMPResponseReportRouteInformation */
/*
************************************************************************
**	response specific bytes for SMP Report Phy Event Information, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportPhyEventInformation
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char ReservedByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10-14 */
	unsigned char ReservedByte10_14[5];
	/* byte 15 */
	unsigned char NumberOfPhyEventDescriptors;
	/* byte 16-27 */
	unsigned char PhyEventDescriptor[12];	//TBD
/*
	byte PhyEventDescriptor[12];
		...
	byte PhyEventDescriptor[12];
		
*/
	u32 CRC;
};/* struct SMPResponseReportPhyEventInformation */
/*
************************************************************************
**	response specific bytes for SMP Report Phy Broadcast Counts, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportBroadcastCounts
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-8 */
	unsigned char ReservedByte6_8[3];
	/* byte 9 */
	unsigned char PhyIdentifier;
	/* byte 10-11 */
	unsigned char ReservedByte10_11[2];
	/* byte 12 */
	unsigned char ChangeCountValid:1;
	unsigned char ReservedChange0CountValid:1;
	unsigned char ReservedChange1CountValid:1;
	unsigned char SESCountValid:1;
	unsigned char ExpanderCountValid:1;
	unsigned char AsysnchronousEventCountValid:1;
	unsigned char Reserved3CountValid:1;
	unsigned char Reserved4CountValid:1;
	/* byte 13-15 */
	unsigned char ReservedByte13_15[3];
	/* byte 16 */
	unsigned char BroadcastChangeCount;	
	/* byte 17 */
	unsigned char BroadcastReserved0ChangeCount;	
	/* byte 18 */
	unsigned char BroadcastReserved1ChangeCount;	
	/* byte 19 */
	unsigned char BroadcastSESCount;	
	/* byte 20 */
	unsigned char BroadcastExpanderCount;	
	/* byte 21 */
	unsigned char BroadcastAsynchronousEventCount;	
	/* byte 22 */
	unsigned char BroadcastReserved3ChangeCount;	
	/* byte 23 */
	unsigned char BroadcastReserved4ChangeCount;	
	u32 CRC;
};/* struct SMPResponseReportBroadcastCounts */
/*
************************************************************************
**	response specific bytes for SMP Discover List Descriptor, intended to be
**	referenced by SMPResponseDiscoverList
************************************************************************
*/
struct DiscoverShortFormatDescriptor
{
	/* byte 48 */
	unsigned char PhyIdentifier;
	/* byte 49 */
	unsigned char FunctionResult;
	/* byte 50 */
	unsigned char ReservedByte50Bit0_3:4;
	unsigned char AttachedDeviceType:3;
	unsigned char ReservedByte50Bit7:1;
	/* byte 51 */
	unsigned char NegotiatedPhysicalLinkRate:4;
	unsigned char ReservedByte51Bit4_7:4;
	/* byte 52 */
	unsigned char AttachedSataHost:1;
	unsigned char AttachedSMPInitiator:1;
	unsigned char AttachedSTPInitiator:1;
	unsigned char AttachedSSPInitiator:1;
	unsigned char ReservedByte52Bit4_7:4;
	/* byte 53 */
	unsigned char AttachedSataDevice:1;
	unsigned char AttachedSMPTarget:1;
	unsigned char AttachedSTPTarget:1;
	unsigned char AttachedSSPTarget:1;
	unsigned char ReservedByte53Bit4_6:3;
	unsigned char AttachedSataPortSelector:1;
	/* byte 54 */
	unsigned char RoutingAttribute:4;
	unsigned char ReservedByte54Bit4_6:3;
	unsigned char VirtualPhy:1;
	/* byte 55 */
	unsigned char ReservedByte55;
	/* byte 56 */
	unsigned char ZoneGroup;
	/* byte 57 */
	unsigned char ReservedByte57Bit0:1;
	unsigned char InsideZPSDS:1;
	unsigned char ZoneGroupPersistent:1;
	unsigned char ZoneAddressResolved:1;
	unsigned char RequestedInsideZPSDS:1;
	unsigned char InsideZPSDSPersistent:1;
	unsigned char ReservedByte57Bit6_7:2;
	/* byte 58 */
	unsigned char AttachedPhyIdentifier;
	/* byte 59 */
	unsigned char PhyChangeCount;
	/* byte 60-68 */
	unsigned char AttachedSASAddress[8];
	/* byte 69-72 */
	unsigned char ReservedByte68_72[4];
};/* DiscoverShortFormatDescriptor */
/*
************************************************************************
************************************************************************
*/
union DiscoverListDescriptor
{
	struct DiscoverShortFormatDescriptor	ShortFormatDescriptor;
	struct DiscoverLongFormatDescriptor	LongFormatDescriptor;	
};
/*
************************************************************************
**	response specific bytes for SMP Discover List, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseDiscoverList
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char ReservedByte6_8[2];
	/* byte 8 */
	unsigned char StartingPhyIdentifier;
	/* byte 9 */
	unsigned char NumberOfDescriptors;
	/* byte 10 */
	unsigned char PhyFilter:4;
	unsigned char ReservedByte10Bit4_7:4;
	/* byte 11 */
	unsigned char DescriptorType:4;
	unsigned char ReservedByte11Bit4_7:4;
	/* byte 12 */
	unsigned char DescriptorLength;
	/* byte 13-15 */
	unsigned char ReservedByte13_15[3];
	/* byte 16 */
	unsigned char ConfigurableRouteTable:1;
	unsigned char Congiguring:1;
	unsigned char ReservedByte16Bit2_5:4;
	unsigned char ZoningEnabled:1;
	unsigned char ZoningSupported:1;
	/* byte 17 ~ 31 */
	unsigned char ReservedByte17_31[15];	
	/* byte 32 ~ 47 */
	unsigned char VendorSpecific[16];	
	/* byte 48 ~ n */
	union DiscoverListDescriptor 	FirstDescriptor;
	u32 CRC;
};/* struct SMPResponseDiscoverList */
/*
************************************************************************
**	response specific bytes for Expander Route Table Descriptor, intended to be
**	referenced by SMPResponseReportExpanderTable
************************************************************************
*/
struct ExpanderRouteTableDescriptor
{
	/* byte 32-37 */
	unsigned char PhyBitMap[6];
	/* byte 38 */
	unsigned char ReservedByte38Bit0_6:7;
	unsigned char ZoneGroupValid:1;
	/* byte 39 */
	unsigned char ZoneGroup;
	/* byte 40-48 */
	unsigned char RoutedSASAddress[8];
};/* ExpanderRouteTableDescriptor */
/*
************************************************************************
**	response specific bytes for SMP Report Expander Route Table, intended to be
**	referenced by SMPResponse
************************************************************************
*/
struct SMPResponseReportExpanderTable
{
	/* byte 4-5 */
	unsigned char ExpanderChangeCount[2];
	/* byte 6-7 */
	unsigned char ExpanderRouteTableChangeCount[2];
	/* byte 8 */
	unsigned char ReservedByte8Bit0:1;
	unsigned char Configuring:1;
	unsigned char ReservedByte8Bit2_7:6;
	/* byte 9 */
	unsigned char ReservedByte9;
	/* byte 10-11 */
	unsigned char NumberOfDescriptors[2];
	/* byte 12-13 */
	unsigned char FirstRoutedSASAddressIndex[2];
	/* byte 14-15 */
	unsigned char LastRoutedSASAddressIndex[2];
	/* byte 13-15 */
	unsigned char ReservedByte13_15[3];
	/* byte 16-18 */
	unsigned char ReservedByte16_18[3];	
	/* byte 19 */
	unsigned char StartingPhyIdentifier;
	/* byte 20-31 */
	unsigned char ReservedByte20_31[12];	
	/* byte 32 - n */
	struct ExpanderRouteTableDescriptor	FirstRouteTableDescriptor;
	u32 CRC;
};/* struct SMPResponseReportExpanderTable */
/*
************************************************************************
************************************************************************
*/
struct SMPResponseConfigureFunction
{
	/* byte 4-7 */
	u32 CRC;
};
/*
************************************************************************
**	response specific to SMP Read SGPIO
**	Corresponds to READ_GPIO_REGISTER define
************************************************************************
*/
struct SMPResponseReadGPIORegister
{
	/* byte 4-11 */
	unsigned char Data_In_Low[8];
	/* byte 12-19 */
	unsigned char Data_In_High[8];
	u32 CRC;
};
/*
************************************************************************
**	response specific to SMP Read SGPIO
**	Corresponds to WRITE_GPIO_REGISTER define
************************************************************************
*/
struct SMPResponseWriteGPIORegister
{
	u32 CRC;
};
/*
************************************************************************
**	generic structure referencing an SMP Response, must be initialized
**	before being used
************************************************************************
*/
struct SMPResponse
{
	/* byte 0 */
	unsigned char SMPFrameType; /* always 41h for SMP responses */
	/* byte 1 */
	unsigned char Function;
	/* byte 2 */
	unsigned char FunctionResult;
	/* byte 3 */
	unsigned char ReservedByte3;
	/* bytes 4-n */
	union
	{
		/* #define REPORT_GENERAL			0x00 */
		struct SMPResponseReportGeneral				ReportGeneral;
		/* #define REPORT_MANUFACTURER_INFORMATION 	0x01 */
		struct SMPResponseReportManufacturerInformation		ReportManufacturerInformation;
		/* #define READ_GPIO_REGISTER			0x02 */	
		struct SMPResponseReadGPIORegister			ReadGPIORegister;
		/* #define REPORT_SELF_CONFIGURATION_STATUS	0x03 */
		struct SMPResponseReportSelfConfigurationStatus		ReportSelfConfigurationStatus;
		/* #define DISCOVER				0x10 */
		struct SMPResponseDiscover				Discover;
		/* #define REPORT_PHY_ERROR_LOG			0x11 */
		struct SMPResponseReportPhyErrorLog			ReportPhyErrorLog;
		/* #define REPORT_PHY_SATA 			0x12 */
		struct SMPResponseReportPhySATA 			ReportPhySATA;
		/* #define REPORT_ROUTE_INFORMATION 		0x13 */
		struct SMPResponseReportRouteInformation 		ReportRouteInformation;
		/* #define REPORT_PHY_EVENT_INFORMATION		0x14 */
		struct SMPResponseReportPhyEventInformation		ReportPhyEventInformation;
		/* #define REPORT_PHY_BROADCAST_COUNTS		0x15 */
		struct SMPResponseReportBroadcastCounts			ReportBroadcastCounts;
		/* #define DISCOVER_LIST			0x16 */
		struct SMPResponseDiscoverList				DiscoverList;
		/* #define REPORT_EXPANDER_ROUTE_TABLE		0x17 */
		struct SMPResponseReportExpanderTable			ReportExpanderTable;
		/* #define CONFIGURE_GENERAL			0x80 */
		struct SMPResponseConfigureFunction			ConfigureGeneral;
		/* #define ENABLE_DISABLE_ZONING		0x81 */
		struct SMPResponseConfigureFunction			EnableDisableZoning;
		/* #define WRITE_GPIO_REGISTER			0x82 */	
		struct SMPResponseWriteGPIORegister			WriteGPIORegister;
		/* #define ZONED_BROADCAST			0x85 */
		struct SMPResponseConfigureFunction			ZonedBroadcast;
		/* #define CONFIGURE_ROUTE_INFORMATION 		0x90 */
		struct SMPResponseConfigureFunction			ConfigureRouteInformation;
		/* #define PHY_CONTROL 				0x91 */
		struct SMPResponseConfigureFunction 			PhyControl;
		/* #define PHY_TEST 				0x92 */
		struct SMPResponseConfigureFunction 			PhyTest;
		/* #define CONFIGURE_PHY_EVENT_INFORMATION	0x93 */
		struct SMPResponseConfigureFunction 			ConfigurePhyEventInformation;
	} Response;
};/* struct SMPResponse */
/*
************************************************************************
************************************************************************
*/
struct _AdapterInfo
{
	u32		flags;
	unsigned short	devId;
	u32		classCode;
	unsigned char	revId;
	unsigned char	bus;
	unsigned char	devFunc;
	void		*bar[6];
	u32		ExpRomBaseAddr;
	unsigned char	version;
	unsigned char	subVersion;

	unsigned short	FlashID;
	u32		FlashSize; 
	u32		FlashSectSize;
};
/*
************************************************************************
************************************************************************
*/
struct _Route_Table_Entry 
{
	u32   		Disabled;
	unsigned char	SASAddr[8];
};
/*
************************************************************************
** if TagStackType!=FIFO_TAG, use FILO, 
** if TagStackType==FIFO_TAG, use FIFO, PtrOut is the next tag to get 
**  and Top is the number of available tags in the stack 
** when use FIFO, get tag from PtrOut and free tag to (PtrOut+Top)%Size 
************************************************************************
*/
struct _Tag_Stack
{
	spinlock_t lock;
	unsigned short *Stack;
	unsigned short   Top;
	unsigned short   Size;
	unsigned short   PtrOut;
	unsigned char    TagStackType;
	unsigned char    Reserved[1];	
};
/*
************************************************************************
**		DOMAIN Port (multiple link, logical port) structure
************************************************************************
*/
struct _Domain_Port 
{
	PCore_Extension			pCore;
	unsigned char			Port_Index;	/* sas port device id */
	unsigned char			Port_State;
	/* Port initialization state */
	unsigned char			Type;		/* PORT_TYPE_SAS,PORT_TYPE_SATA,PORT_TYPE_PM */
		#define PORT_STATE_IDLE 		0x00
		#define PORT_STATE_INIT_DONE		0xFF
		#define PORT_TYPE_SATA			ARCSAS_BIT(1, 0)
		#define PORT_TYPE_SAS			ARCSAS_BIT(1, 1)
		#define PORT_TYPE_PM			ARCSAS_BIT(1, 2)
		#define PORT_SETTING_NCQ_RUNNING	ARCSAS_BIT(1, 0)
		#define PORT_SETTING_PM_EXISTING	ARCSAS_BIT(1, 1)	
		#define PORT_SETTING_PM_FUNCTIONAL	ARCSAS_BIT(1, 2)
		#define PORT_SETTING_DURING_RETRY	ARCSAS_BIT(1, 3)
		#define PORT_SETTING_DURING_RECOVERY	ARCSAS_BIT(1, 4)
	unsigned char			Setting;		/* PORT_SETTING_XXX */
	unsigned char			MemberPhyMap;	/* record group of phy "WidePort" */
	unsigned char			reserve[3];
	atomic_t			Expander_Number;
	atomic_t			Device_Number;	/* How many devices does this port have now? */   
	atomic_t			DiscoveryInstance;
	//Bool				devices_get_ready;
	PDomain_Phy			pPhy;		/* pointer back to the phy of this port */
	PDomain_PM			pPM;

	//struct osm_list_head		pExpTreeRoot;	/* point to ->pSibling */
	struct osm_list_head		Device_List;
	spinlock_t			device_list_lock;
	//unsigned long			device_list_flags;
	struct osm_list_head		Expander_List;
	spinlock_t			expander_list_lock;
	//unsigned long			expander_list_flags;
	atomic_t			Post_Ses_Page_Instance;
	//struct timer_list		DiscoveryPolling_timer;
};
/* 
************************************************************************
**	Physical Port (PHY based) structure  
**	Port has Phy, Phy has Port
************************************************************************
*/
struct _Domain_Phy
{
	PDomain_Port		pPort;      
	/* from PORT_CONFIG_ADDR0-3, PhyId/device protocol/sas_addr/SIG/wide_port */
	u32			DevInfo;
	ARCSAS_U64		DevSASAddr;
	u32			AttDevInfo;
	ARCSAS_U64		AttDevSASAddr;
	unsigned char		Type;
	unsigned char		WidePortPhyMap;/*record group of phy*/
	//unsigned char		Reserved;
	/* from PORT_PHY_CONTROL0-3, REG_PORT_PHY_CONTROL, linkrate, current status */
	u32			PhyStatus;
	/* from PORT_IRQ_MASK0-3 */
	u32			IRQStatus;
	unsigned char		OOB_Mode;
};
#define BUG_OF_SUPER_MICRO_REPORT_ERROR_MASK	0x0F00/*0B00,0700,0B01,0F01,0701,0F00*/
/*
************************************************************************
**    Domain of SATA Port Multiplier
************************************************************************
*/
struct _Domain_PM 
{
	//struct osm_list_head		Queue_Pointer;
	//struct osm_list_head		Exp_Queue_Pointer;
	struct osm_list_head		Sent_CCB_List;	/* List of requests that were sent to the device, in order */
	unsigned long			sent_list_flags;
	spinlock_t			sent_ccb_list_lock;
	unsigned short			Pci_Ven_ID;
	unsigned short			Pci_Dev_ID;

	//unsigned short		PluginNotify;
	unsigned short			Status;		/* DEVICE_STATUS_XXX */

	//unsigned char			AdapterID;
	unsigned char			ProductRevision;
	unsigned char			CorePhyNum;
	unsigned char			SpecRevision;	/* 10 means 1.0, 11 means 1.1 */
	unsigned char			NumberOfPorts;

	unsigned char			EntryIndex;	/* port mutiplier device id */
	atomic_t			Outstanding_CCB;/* for error carrying out */
	unsigned char			RetryCnt;
	unsigned char			State;					
	struct timer_list		timer;
	//unsigned long			flags;
	//unsigned char			Timer_Status;		/* for error carrying out */
	unsigned char			Timeout_Count;		/* number of times this device has timed out */
	unsigned char			CheckedPort;
	unsigned char			ActPortNo;

	unsigned char			Feature_Enabled;
	unsigned char			PlugStatus;
	#define PM_DEVICE_PLUG_IN	ARCSAS_BIT(1, 0)
	#define PM_DEVICE_PLUG_OUT	ARCSAS_BIT(1, 1)
	unsigned char			register_set;		/* which register set this device is using */
	unsigned char			Error_Handling_State;

	u32				SError;

	PDomain_Device			Devices[ARCSAS_MAX_DEVICE_PER_PM];
	PDomain_Expander		pExpander;
	PDomain_Port			pPort;
};
/*
************************************************************************
**	Domain of SAS Expander  
************************************************************************
*/
struct _Domain_Expander
{
	struct osm_list_head		Exp_Queue_Pointer;
	struct osm_list_head		Device_List;
	spinlock_t			device_list_lock;
	ARCSAS_U64			SASAddr;
	PDomain_Port			pPort;
	PDomain_Expander		pParent;
	unsigned short			ComponentID;
	unsigned char			ComponentRevisionID;
	unsigned char			NegotiatedLinkRate;

	/* How many devices this port has now? */   
	atomic_t			Device_Number;      
	unsigned char			Phy_Count;	/* # of phys on this expander */
	unsigned char			ParentPhyCount;	/* # of phys used to connect this expander to its parent */
	unsigned char			EntryIndex;	/* sas expander device id */

	//unsigned char			Reserved[3];
	unsigned char			STP_INIT:1;
	unsigned char			SMP_INIT:1;
	unsigned char			SSP_INIT:1;
	unsigned char			STP_TRGT:1;
	unsigned char			SMP_TRGT:1;
	unsigned char			SSP_TRGT:1;
	unsigned char			route_table_configurable:1;
	unsigned char			Reserved_bit:1;

	u32				Route_Index_Count;   
	/*   struct _Route_Table_Entry	Route_Table[MAX_PHY_NUM][MAX_ROUTE_INDEX];*/
	//struct _Route_Table_Entry	*Route_Table;

	unsigned char			ParentPhyId[ARCSAS_WIDEPORT_PHYS];
	unsigned char			SelfPhyId[ARCSAS_WIDEPORT_PHYS]; /* 4 */
	unsigned char			VendorID[16];

	unsigned char			ProductID[24];

	unsigned char			ProductRev[8];

	unsigned char			ComponentVendorID[16];
};
/*
************************************************************************
**    Domain of Device...disk, tape, scanner, mo, CDROM, CDRW......
************************************************************************
*/
struct _Domain_Device 
{
	SesEnclosureElement		ses_enclosure_element;	/*8 bytes*/
	OverallEnclosureStatus		SesOverallEnclosureStatus;	/* BYTE size: could be used as control or status */
	struct osm_list_head		Port_Queue_Pointer;
	struct osm_list_head		Exp_Queue_Pointer;
	struct osm_list_head		Sent_CCB_List;		/* List of requests that were sent to the device, in order */
	struct osm_list_head		CCB_Waiting_List; 	/* Waiting Request Queue */
	spinlock_t			sent_ccb_list_lock;
	spinlock_t			ncq_lock;
	struct _Core_Extension		*pCore;
	PDomain_Expander		pExpander;		/*SAS Expander		*/
	PDomain_Device			pSesDevice;		/*pointer to the SES device if it is a SES element, if not with NULL*/
	PDomain_Port			pPort;			/*SAS Port, SATA Port	*/
	PDomain_PM			pPM;			/*SATA Port Multiplier	*/
	struct work_struct		arcsas_add_scsi_device;
	struct work_struct		arcsas_remove_scsi_device;
	ARCSAS_U64			WWN;
	ARCSAS_U64			SASAddr;
	ARCSAS_U64			Max_LBA;
	ARCSAS_U64			TotalFormattedLbaCount;
	struct scsi_device		*psdev;
	struct timer_list		timer;
	unsigned long			TaskFile_Error_jiffies;
	atomic_t			Outstanding_CCB;	/* for error carrying out */
	atomic_t			TaskFile_Error_Count;
	atomic_t			Timer_Status;		/* for error carrying out */
	u32				SectorSize;
	u32				NCQTagMask;		/*32bit for SATA NCQ TAG	*/
	u32				ses_pages_supported;
	/* 
	 * Different device should have a different struct here. 
	 * Now it's SATA device only.
	*/
	unsigned short			Ability;		/* Be able to support NCQ, 48 bit LBA. */
	unsigned short			Setting;		/* The supported features are enabled or not. */
	unsigned short			EntryIndex;		/* sas device device id ,scsilun-scsiid */
	unsigned short			Retry_Count;
	unsigned char			pm_port_no;		/*this value is physical port number of pm chipset for its attached devices*/
	unsigned char			PIO_Mode;
	unsigned char			MDMA_Mode;
	unsigned char			UDMA_Mode;
	unsigned char			Current_PIO;
	unsigned char			Current_MDMA;
	unsigned char			Current_UDMA;
	unsigned char			register_set;		/* which register set this device is using */
	unsigned char			Phy_Id;               
	unsigned char			Queue_Depth;
	unsigned char			Error_Handling_State;
	unsigned char			Is_Waiting;		/* for hibernation serial state machine - which device is waiting to be initialized */
	unsigned char			NegotiatedLinkRate;
	unsigned char			Timeout_Count;		/* number of times this device has timed out */
	unsigned char			MaxOfEnclosureSlotNumber; /* max slot number in this enclosure */
	unsigned char			Sibling_phy_cnt;	/* count for testing if it is wide port end device */
	unsigned char			CmdErrorCount;
	unsigned char			State;			/* DEVICE_STATE_XXX */
	unsigned char			Status;			/* DEVICE_STATUS_XXX */
	unsigned char			Connection;
	unsigned char			Dev_Type;
	unsigned char			log2_per_phys_sector;
	unsigned char			reserved[2];
	/* 
	************************************************************
	**                  SES briefing
	**   A device could be a SSP/STP/SATA/ATAPI device 
	**   which can be determined by the Device_Type.
	**   A SSP device could be a disk drive or a SES device.
	**   A SSP/STP device could be an elememt of a SES device 
	**   (for example, disk drives in an enclosure).
	**   Whether a SSP/STP device is an elememt of a SES device 
	**   or not could be determined by the pSesDevice pointer.
	**   If the pSesDevice is NULL, it is not a SES element.
	************************************************************
	*/
	unsigned char			SesOverallElementIndex;	/* include type overall */
	unsigned char			SesElementIndex;		/* used only if a SES element, not include overall */
	unsigned char			SesSlotNumber;
	unsigned char			SesElementType;
	/* 
	*************************************************************
	** could be used as control or status, 
	** type can be determined by SesElementType (device or array) 
	*************************************************************
	*/
	unsigned char			Serial_Number[20];
	unsigned char			Model_Number[40];
	unsigned char			Firmware_Revision[8];
	unsigned char			saslun[8];
	u16				Rotation_Rate; /* 0 rpm =0001h, Non-rotating medium (SSD);7200 rpm = 1C20h;10000 rpm = 2710h;15000 rpm = 3A98h */
	u16				Max_Block_Per_DSM_CMD; /* Maximum number of 512-byte blocks per DATA SET MANAGEMENT command */
	u16				lowest_aligned;
	u16				Additional_support;
	u16				Command_Set_Default;
	u16				queued_cmd_request;
};
/* 
************************************************************************
************************************************************************
*/
struct _Core_Extension
{
	PCCB					*Running_CCB;			/*The ccbs in this list are all posted to HDs*/
	struct AdapterControlBlock		*pACB;
	struct tasklet_struct			isr_tasklet;
	//struct tasklet_struct			port_device_changed_tasklet;
	Domain_Port				Ports[ARCSAS_MAX_CORE_PHYSICAL_PORT_NUMBER];		/*(8) Domain Ports */
	unsigned short				Slot_Count_Supported;
	//struct timer_list			timer_request;
	//unsigned char				Timer_Status;		/* for error carrying out */
	//unsigned char				reserved;
	unsigned char				MaxRegisterSet;
	unsigned char				MaxCmdSlotWidth;	/* 9 for 512, 10 for 1024 */
	/* 
	*******************************************************
	**   new shared DevicesExpandersPMs_EntryIndex system:
	**   0 to MAX_DEVICE_ID-1 for drives........................(0   to 128-1   ),=> (  0,127)
	**   MAX_DEVICE_ID to MAX_EXPANDER_ID-1 for expanders.......(128 to 128 + 8-1),=> (128,143)
	**   MAX_EXPANDER_ID to MAX_PM_ID-1 for PM..................(144 to 144+8-1	),=> (144,151)
	*******************************************************
	*/
	unsigned char					Ports_EntryIndex[ARCSAS_MAX_CORE_PHYSICAL_PORT_NUMBER];/*(8) must be 4 byte alignment */
	u32						Resetting_Slot[16];
	void						*io_map_base;
	void __iomem					*mem_map_base;
	void __iomem					*mem_map_base_ext;
	unsigned char					last_port_index;
	//unsigned char					Scratch_Count1;				/* used for core API purposes */
	//unsigned char					Scratch_Count2;				/* used for core API purposes */
	unsigned char					Phy_Num;				/* How many phy ports we have? */
	unsigned char					State;
	unsigned char					core_index;
	struct osm_list_head				CCB_Waiting_List; 		/* Waiting Request Queue */
	struct osm_list_head				CCB_Complete_List;		/* Completed Request Queue */
	struct osm_list_head				Core_CCB_List;			/* list of free internal requests */
	struct osm_list_head				Context_List;			/* list of core context data structure */
	struct osm_list_head				SAS_Scratch_List;			/* list of free scratch buffers for SATA */
	struct osm_list_head				SMP_Scratch_List;		/* list of free scratch buffers for SMP Requests */
	struct osm_list_head				PRD_Buffer_List;			/* list of SG buffers for command table */
	//struct osm_list_head				Wait_RegisterSet_CCB_List;	/* List of CCBs that wait reg set */
	//spinlock_t					eh_lock;
	spinlock_t					ccb_waiting_lst_lock;
	spinlock_t  					ccb_cmpl_list_lock;
	spinlock_t  					ccb_list_lock;			/* linked list lock for whole driver. */
	spinlock_t 					context_list_lock;
	spinlock_t 					prd_list_lock;
	spinlock_t 					sassb_list_lock;
	spinlock_t 					smpsb_list_lock;
	spinlock_t 					doneq_lock;
	//unsigned long					doneq_lock_flags;
	//unsigned long					ccb_cmpl_list_flag;
	//unsigned long					ccb_waiting_lst_flags;
	//unsigned int					count;
	/* Command List */
	void *						CmdSlot;			/* command list Vir: Can be PMV_PATA_Command_Header or PARCSAS_Command_Header */
	ARCSAS_U64					CmdSlot_DMA;			/* command list Phy: */
	/* Received FIS */
	void *						RX_FIS;				/* received FIS Vir: SATA FIS */
	ARCSAS_U64					RX_FIS_DMA;			/* received FIS Phy: */
	/* The 32 command tables. */
	void *						Cmd_Table;			/* 32 command tables Vir: */
	ARCSAS_U64					Cmd_Table_DMA;			/* 32 command tables Phy: */
	/* Delivery Queue */
	void *						PostQ;				/* delivery queue Vir: */
	ARCSAS_U64					PostQ_DMA;			/* delivery queue Phy: */
	/* Completion Queue */
	void						*DoneQ;				/* completion queue Vir: */
	ARCSAS_U64					DoneQ_DMA;			/* completion queue Phy: */

	void						*pDiscoverBuffer;
	unsigned short					last_PostQ_index;
	unsigned short					last_DoneQ_index;
	unsigned short					current_DoneQ_index;
	unsigned short					Max_PostQ_Index;				/* Size of the deliver queue */
	unsigned short					Max_DoneQ_Index;				/* Size of completion queue, this differs for normal mode and hibernation */
	/* Command List Slot Number Pool*/
	Tag_Stack					CmdSlot_Tag_Pool;     /*512*/
	//unsigned long					CmdSlot_Tag_Pool_lock_flags;
	Tag_Stack					Port_Tag_Pool;        /*8*/
	/* Phy info */
	Domain_Phy					Phy[ARCSAS_MAX_CORE_PHYSICAL_PORT_NUMBER];/*(8)*/
	struct dma_pool					*cmnd_list_pool;		/*command list pool */
	osm_dma_addr_t					cmnd_list_pool_dma;	/*command list pool dma address */
	void 						*cmnd_list_pool_vaddr;

	struct dma_pool					*rcv_fis_pool;		/*received FIS pool */
	osm_dma_addr_t					rcv_fis_pool_dma;	/*received FIS pool dma address */
	void						*rcv_fis_pool_vaddr;

	struct dma_pool					*cmnd_tbl_pool;		/*command table pool */
	osm_dma_addr_t					cmnd_tbl_pool_dma;	/*command table pool dma address */
	void						*cmnd_tbl_pool_vaddr;

	struct dma_pool					*post_queue_pool;	/*post queue pool */
	osm_dma_addr_t					post_queue_pool_dma;	/*post queue pool dma address */
	void						*post_queue_pool_vaddr;

	struct dma_pool					*cmpl_queue_pool;	/*completion queue pool */
	osm_dma_addr_t					cmpl_queue_pool_dma;	/*completion queue pool dma address */
	void						*cmpl_queue_pool_vaddr;

	struct dma_pool					*sas_scrtch_pool;		/*SAS scratch buffer pool */
	osm_dma_addr_t					sas_scrtch_pool_dma;	/*SAS scratch buffer pool dma address */
	void						*sas_scrtch_pool_vaddr;	/*SAS scratch buffer pool virtual address */

	struct dma_pool					*sg_tbl_pool;		/*SG table buffer pool */
	osm_dma_addr_t					sg_tbl_pool_dma;		/*SG table buffer pool dma address */
	void						*sg_tbl_pool_vaddr;	/*SG table buffer pool virtual address */
	//struct completion				cmpl;

	unsigned long					core_flag;						      /* Is during dump */
	#define ARCSAS_CORE_FLAG_HBA_CORE_ID0		0x01
	#define ARCSAS_CORE_FLAG_HBA_CORE_ID1		0x02
	u32			sata_register_set[2];
	u32			free_ccb_count;
};
/*
************************************************************************
************************************************************************
*/
struct _SENSE_CODE_SET
{
	unsigned short 				bitMask; /* Status(8..15):Error(0..7) */
	unsigned short 				reserved;
	unsigned char  				RW; /* 1 - write */
	unsigned char  				Key;
	unsigned char  				ASC;
	unsigned char  				ASCQ;
};
/*
************************************************************************
** TgtDevMap structure for core driver.
************************************************************************
*/
struct _Target_Dev
{
	unsigned char				Connection;
	unsigned char				DevType;
	unsigned char 				PhyId;			/* If SATA device, set PhyId of expander which attached sata device */
	unsigned char				TargetBits;		/* from Response.Discover.TargetBits */
	unsigned char				NegotiatedLinkRate:4; 
	unsigned char				MaxLinkRateCapability:4;	/* give four bits for Max linkrate */ 
	unsigned char				Reserved0[3];
	unsigned char				PhyChangeCount;
	unsigned char				DevSASAddr[SAS_ADDR_SIZE];
};
/*
************************************************************************
************************************************************************
*/
struct _Target_Disc_Info
{
	unsigned char		TotalDevCounts;	/* Total device(only include SAS/SATA) counts attached behind a dedicated expander */
	unsigned char		TgtId;			/* dedicated expander ID */
	unsigned char		PhyCount;
	unsigned char		TargetBits;		/* from Response.Discover.TargetBits */
	unsigned char		NegotiatedLinkRate; /* for SAS speed setting */
	unsigned char		ParentPhyCount;
	unsigned char		ComponentRevisionID;
	unsigned char		phyIdToContinueDisc;
	unsigned char		ParentPhyId[ARCSAS_WIDEPORT_PHYS]; /* 4 */
	unsigned char		SelfPhyId[ARCSAS_WIDEPORT_PHYS]; /* 4 */
	unsigned char		VendorID[16];
	unsigned char		ProductID[24];
	unsigned char		ProductRev[8];
	unsigned char		ComponentVendorID[16];
	unsigned short		ComponentID;
	unsigned short		MaxRouteIndexes;
	unsigned char		ParentSASAddr[SAS_ADDR_SIZE];
	unsigned char		SASAddr[SAS_ADDR_SIZE];
};
/*
************************************************************************
************************************************************************
*/
struct _Target_Dev_Map
{
	Target_Info		Info;
	Target_Dev		TgtDev[0];
};
/*
************************************************************************
************************************************************************
*/
struct _SMPBuffer_Context
{
	struct _SMPRequest	SMPBuffer;
	unsigned char		SASAddr[SAS_ADDR_SIZE];
	PDiscover_Extension	pDiscovery;
	SMP_CBFunc		PCallBack;  

	unsigned char		nTgt;
	unsigned char		phyId;
	unsigned char		BufferIndex;
	atomic_t		isBufferFree;
};
/*
************************************************************************
************************************************************************
*/
struct _Config_Route_Info
{
	PDomain_Port	pPort; 
	void		*pParent;
	unsigned short	RouteIndex;
	unsigned char	phyId;
	unsigned char	issuedFlag;
	unsigned char	SASAddr[SAS_ADDR_SIZE];
};
/*
************************************************************************
************************************************************************
*/
struct _Discover_Port_Info
{
	PDomain_Port		pPort;
	unsigned char		DiscoveryReturn;
	unsigned char		BroadcastReceived;	
	Config_Route_Info	ConfigRouteInfo[ARCSAS_CONFIG_ROUTES];/*128 + 8 = 136*/
};
/*
************************************************************************
************************************************************************
*/
struct _Discover_Target /* Expander */
{
	Target_Info	Info;
	unsigned char	DiscoveryState;
	unsigned char	NeedConfigured;
	unsigned char	PhyId;
	void		*pParent;
	void		(* PCallBack)(PDiscover_Extension, PDomain_Port);
	unsigned short	CurrentIndex[ARCSAS_ROUTE_INDEX];
	PDomain_Port	pPort;
	Target_Dev	TgtDev[ARCSAS_MAX_DEVICE_PER_EXPANDER];
};
/*
************************************************************************
************************************************************************
*/
struct _Discover_Extension
{
	unsigned char		NumberOfMaxTarget;
	unsigned char		TargetNumber;
	unsigned char		numOfDiscoveryReq;
	unsigned char		Reserved;
	u32			DiscoverMask;
	PDomain_Port		PortsWaitToBeDiscover[ARCSAS_EXPANDER_SUPPORTED];
	Discover_PortInfo	PortInfo[ARCSAS_MAX_CORE_PHYSICAL_PORT_NUMBER];
	SMPBuffer_Context	SMPContext[ARCSAS_DISCOVER_REQUESTS];
	Discover_Target		Target[0];
	struct work_struct	arcsas_do_smp_phy_control_reset;
	struct work_struct	arcsas_do_smp_state_machine;
};
/*
************************************************************************
**	SATA FIS: Register-Host to Device
************************************************************************
*/
struct _SATA_FIS_REG_H2D
{
/* DW 0 */
	unsigned char	FIS_Type;
#if defined(__BIG_ENDIAN_BITFIELD)
	unsigned char	C : 1;
	unsigned char	Reserved0 : 3;
	unsigned char	PM_Port : 4;
#else
	unsigned char	PM_Port : 4;
	unsigned char	Reserved0 : 3;
	unsigned char	C : 1;
#endif
	unsigned char	Command;
	unsigned char	Features;
/* DW 1 */
	unsigned char	LBA_Low;
	unsigned char	LBA_Mid;
	unsigned char	LBA_High;
	unsigned char	Device;
/* DW 3 */
	unsigned char	LBA_Low_Exp;
	unsigned char	LBA_Mid_Exp;
	unsigned char	LBA_High_Exp;
	unsigned char	Features_Exp;
/* DW 4 */
	unsigned char	Sector_Count;
	unsigned char	Sector_Count_Exp;
	unsigned char	Reserved1;
	unsigned char	Control;

	unsigned char	Reserved2[4];
};
typedef struct _saved_fis {
	u32 dw1;
	u32 dw2;
	u32 dw3;
	u32 dw4;
} saved_fis, PSAVED_FIS;
/*
************************************************************************
************************************************************************
*/
struct _ATA_TaskFile 
{
	unsigned char	Features;
	unsigned char	Sector_Count;
	unsigned char	LBA_Low;
	unsigned char	LBA_Mid;
	unsigned char	LBA_High;
	unsigned char	Device;
	unsigned char	Command;

	unsigned char	Control;

	/* extension */
	unsigned char	Feature_Exp;
	unsigned char	Sector_Count_Exp;
	unsigned char	LBA_Low_Exp;
	unsigned char	LBA_Mid_Exp;
	unsigned char	LBA_High_Exp;
};
/*
************************************************************************
************************************************************************
*/
/* ATA device identify frame unit in WORD*/
struct _ATA_Identify_Data 
{
	unsigned short General_Config;				/*	0	*/
	unsigned short Obsolete0;				/*	1	*/
	unsigned short Specific_Config;				/*	2	*/
	unsigned short Obsolete1;				/*	3	*/
	unsigned short Retired0[2];				/*	4-5	*/
	unsigned short Obsolete2;				/*	6	*/
	unsigned short Reserved0[2];				/*	7-8	*/
	unsigned short Retired1;				/*	9	*/
	unsigned char  Serial_Number[20];			/*	10-19	*/
	unsigned short Retired2[2];				/*	20-21	*/
	unsigned short Obsolete3;				/*	22	*/
	unsigned char  Firmware_Revision[8];			/*	23-26	*/
	unsigned char  Model_Number[40];			/*	27-46	*/
	unsigned short Maximum_Block_Transfer;			/*	47	*/
	unsigned short Reserved1;				/*	48	*/
	unsigned short Capabilities[2];				/*	49-50	*/
	unsigned short Obsolete4[2];				/*	51-52	*/
	unsigned short Fields_Valid;				/*	53	*/
	unsigned short Obsolete5[5];				/*	54-58	*/
	unsigned short Current_Multiple_Sector_Setting;		/*	59	*/
	unsigned short User_Addressable_Sectors[2];		/*	60-61	*/
	unsigned short ATAPI_DMADIR;				/*	62	*/
	unsigned short Multiword_DMA_Modes;			/*	63	*/
	unsigned short PIO_Modes;				/*	64	*/
	unsigned short Minimum_Multiword_DMA_Cycle_Time;	/*	65	*/
	unsigned short Recommended_Multiword_DMA_Cycle_Time;	/*	66	*/
	unsigned short Minimum_PIO_Cycle_Time;			/*	67	*/
	unsigned short Minimum_PIO_Cycle_Time_IORDY;		/*	68	*/
	unsigned short Additional_support;			/*	69	*/
	unsigned short Reserved2;				/*	70	*/
	unsigned short ATAPI_Reserved[4];			/*	71-74	*/
	unsigned short Queue_Depth;				/*	75	*/
	unsigned short SATA_Capabilities;			/*	76	*/
	unsigned short SATA_Reserved;				/*	77	*/
	unsigned short SATA_Feature_Supported;			/*	78	*/
	unsigned short SATA_Feature_Enabled;			/*	79	*/
 	unsigned short Major_Version;				/*	80	*/
	unsigned short Minor_Version;				/*	81	*/
	unsigned short Command_Set_Supported[2];		/*	82-83	*/
	unsigned short Command_Set_Supported_Extension;		/*	84	*/
	unsigned short Command_Set_Enabled[2];			/*	85-86	*/
	unsigned short Command_Set_Default;			/*	87	*/
	unsigned short UDMA_Modes;				/*	88	*/
	unsigned short Time_For_Security_Erase;			/*	89	*/
	unsigned short Time_For_Enhanced_Security_Erase;	/*	90	*/
	unsigned short Current_Advanced_Power_Manage_Value;	/*	91	*/
	unsigned short Master_Password_Revision;		/*	92	*/
	unsigned short Hardware_Reset_Result;			/*	93	*/
	unsigned short Acoustic_Manage_Value;			/*	94	*/
	unsigned short Stream_Minimum_Request_Size;		/*	95	*/
	unsigned short Stream_Transfer_Time_DMA;		/*	96	*/
	unsigned short Stream_Access_Latency;			/*	97	*/
	unsigned short Stream_Performance_Granularity[2];	/*	98-99	*/
	unsigned short Max_LBA[4];				/*	100-103	*/
	unsigned short Stream_Transfer_Time_PIO;		/*	104	*/	
	unsigned short Max_Block_Per_DSM_CMD;			/*	105	*/
	unsigned short Physical_Logical_Sector_Size;		/*	106	*/
	unsigned short Delay_Acoustic_Testing;			/*	107	*/
	unsigned short NAA;					/*	108	*/
	unsigned short Unique_ID1;				/*	109	*/
	unsigned short Unique_ID2;				/*	110	*/
	unsigned short Unique_ID3;				/*	111	*/
	unsigned short Reserved4[4];				/*	112-115	*/
	unsigned short Reserved5;				/*	116	*/
	unsigned short Words_Per_Logical_Sector[2];		/*	117-118	*/
	unsigned short Reserved6[8];				/*	119-126	*/
	unsigned short Removable_Media_Status_Notification;	/*	127	*/
	unsigned short Security_Status;				/*	128	*/
	unsigned short Vendor_Specific[31];			/*	129-159	*/
	unsigned short CFA_Power_Mode;				/*	160	*/
	unsigned short Reserved7[7];				/*	161-175	*/
	unsigned short Device_Nominal_Form_Factor;		/*  168 bit3:0-form factor => Block Device Characteristics VPD Page:NOMINAL FORM FACTOR */
	unsigned short Data_Set_Management_Supported;		/*  169 bit0-data set management supported */
	unsigned short Additional_Product_ID[4];		/*	170-173 */
	unsigned short Reserved8[2];				/*	174-175 */
	unsigned short Current_Media_Serial_Number[30];		/*	176-205	*/
	unsigned short SCT_Command_Transport;			/*	206	*/
	unsigned short Reserved9[2];				/*	207-208	*/
	unsigned short logic_sector_offset;			/*	209	*/
	unsigned short Reserved11[7];				/*	210-216	*/
	unsigned short Rotation_Rate;				/*  217 => Block Device Characteristics VPD Page:MEDIUM ROTATION RATE */
	unsigned short Reserved10[37];				/*	218-254	*/	
	unsigned short Integrity_Word;				/*	255	*/
};
/*
************************************************************************
************************************************************************
*/
struct _SCT_Status_Response
{
	unsigned short Format_Version;
	unsigned short SCT_Version;
	unsigned short SCT_Spec;
	unsigned short Status_Flags[2];
	unsigned char Drive_State;
	unsigned char Reserved1[3];
	unsigned short Extended_Status_Code;
	unsigned short Action_Code;
	unsigned short Function_Code;
	unsigned char Reserved2[20];
	unsigned short LBA[4];
	unsigned char Reserved3[152];
	unsigned char HDA_Temp;
	unsigned char Reserved4;
	unsigned char Max_Temp;
	unsigned char Reserved5;
	unsigned char Life_Max_Temp;
	unsigned char Reserved6[275];
	unsigned char Vendor_Specific[32];
};
/*
************************************************************************
**  SAS device identify frame  
**  R100h ID FRAME: CONFIG_ID_FRAME0-CONFIG_ID_FRAME7 ,DWORD0-DWORD7
**  R11Ch ATTACHED ID FRAME: CONFIG_ATT_ID_FRAME0-CONFIG_ATT_ID_FRAME7 ,DWORD0-DWORD7
************************************************************************
*/
struct _SAS_Identify_Data 
{
	/* Byte 0 */
	unsigned char  frame_type:4;
	unsigned char  dev_type:3;
	unsigned char  Reserved0:1;
	/* Byte 1 */
	unsigned char  Reserved1;
	/* Byte 2 */
	union {
		struct {
			unsigned char  Reserved20:1;
			unsigned char  smp_iport:1;
			unsigned char  stp_iport:1;
			unsigned char  ssp_iport:1;
			unsigned char  Reserved24567:4;
		};
		unsigned char initiator_bits;
	};
	/* Byte 3 */
	union {
		struct {
			unsigned char  Reserved30:1;
			unsigned char smp_tport:1;
			unsigned char stp_tport:1;
			unsigned char ssp_tport:1;
			unsigned char Reserved34567:4;
		};
		unsigned char target_bits;
	};
	/* Byte 4 ~ 11 */
	unsigned char Reserved411[8];
	/* Byte 12 ~ 19 */
	unsigned char sas_addr[SAS_ADDR_SIZE];
	/* Byte 20 */
	unsigned char phy_id;
	/* Byte 21 ~ 27 */
	unsigned char Reserved2127[7];
	/* Byte 28 ~ 31*/
	u32 crc;/*big endian*/
};
/*
************************************************************************
**CONFIG_PHY_STATUS bits 
************************************************************************
*/
struct _REG_CONFIG_PHY_STATUS
{
#if defined(__BIG_ENDIAN_BITFIELD)
	u32   Reserved2:13;
	u32   TXIMP:4;
	u32   RXIMP:4;
	u32   PLL_CAL_DONE:1;
	u32   PHY_PLL_LOCK:1;
	u32   KVCO:3;
	u32   IMP_CAL_DONE:1;
	u32   NGTD_SPEED:1;
	u32   Reserved1:1;
	u32   PHY_RDY:1;
	u32   DW_SYNC:1;   
	u32   OOB_DTCTD:1;
#else
	u32   OOB_DTCTD:1;
	u32   DW_SYNC:1;   
	u32   PHY_RDY:1;
	u32   Reserved1:1;
	u32   NGTD_SPEED:1;
	u32   IMP_CAL_DONE:1;
	u32   KVCO:3;
	u32   PHY_PLL_LOCK:1;
	u32   PLL_CAL_DONE:1;
	u32   RXIMP:4;
	u32   TXIMP:4;
	u32   Reserved2:13;
#endif
};
/*
************************************************************************
**  Hardware related format. Never change their size. Must follow hardware 
**  specification.
**  for command list SSP port, SMP port,STP port
************************************************************************
*/
struct _ARCSAS_Command_Header
{
#if defined(__BIG_ENDIAN_BITFIELD)
/* DWORD 0 */
	u32   PRD_Entry_Count : 16;

	u32   SSP_FrameType : 3;	/* SSP only, 0-frame type set by HW, 1-frame type given by SSP_FrameType */
	u32   SSP_PassThru : 1;
	u32   SSP_FirstBurst : 1;	/* SSP only, generate Burst without waiting for XFER-RDY */
	u32   SSP_VerifyDataLen : 1;	/* SSP only, verify Data length */
	u32   SSP_TLR : 1;		/* SSP only, set if enabling SSP transport layer retry */
	u32   ProtectInfoRecord : 1;	/* SSP only, set if protection information record present */
	u32   Reset: 1;			/* SATA only, set if it is for device reset */
	u32   First_DMA : 1;		/* SATA only, set if it is a first party DMA command */
	u32   ATAPI : 1;		/* SATA only, set if it is a ATAPI PIO */
	u32   BIST : 1;			/* SATA only, set if it is a BIST FIS */
	u32   PM_Port : 4;		/* Port Multiplier field in command FIS */

/* DWORD 1 */
	u32   Reserved2 : 7;
	u32   MaxRspFrameLength : 9;	/* max response frame length in DW, HW will put in status buffer structure */
	u32   Reserved1 : 6;
	u32   FrameLength : 10;		/* command frame length in DW, including frame length, excluding CRC */
/* DWORD 2 */
	u32   TargetTag : 16;		/* Target Port Transfer Tag, for target to tag multiple XFER_RDY */
	u32   Tag : 16;			/* command tag */
#else
/* DWORD 0 */
	u32   PM_Port : 4;		/* Port Multiplier field in command FIS */
	u32   BIST : 1;			/* SATA only, set if it is a BIST FIS */
	u32   ATAPI : 1;		/* SATA only, set if it is a ATAPI PIO */
	u32   First_DMA : 1;		/* SATA only, set if it is a first party DMA command */
	u32   Reset: 1;			/* SATA only, set if it is for device reset */
	u32   ProtectInfoRecord : 1;	/* SSP only, set if protection information record present */
	u32   SSP_TLR : 1;		/* SSP only, set if enabling SSP transport layer retry */
	u32   SSP_VerifyDataLen : 1;	/* SSP only, verify Data length */
	u32   SSP_FirstBurst : 1;	/* SSP only, generate Burst without waiting for XFER-RDY */
	u32   SSP_PassThru : 1;
	u32   SSP_FrameType : 3;	/* SSP only, 0-frame type set by HW, 1-frame type given by SSP_FrameType */

	u32   PRD_Entry_Count : 16;
/* DWORD 1 */
	u32   FrameLength : 10;		/* command frame length in DW, including frame length, excluding CRC */
	u32   Reserved1 : 6;

	u32   MaxRspFrameLength : 9;	/* max response frame length in DW, HW will put in status buffer structure */
	u32   Reserved2 : 7;
/* DWORD 2 */
	u32   Tag : 16;			/* command tag */
	u32   TargetTag : 16;		/* Target Port Transfer Tag, for target to tag multiple XFER_RDY */
#endif

/* DWORD 3 */
	u32   DataXferLen;		/* in bytes */      
/* DWORD 4 */
	u32   Cmd_Table_Address;
/* DWORD 5 */
	u32   Cmd_Table_Address_High;
/* DWORD 6 */
	u32   OpenAddrFrame_Address;
/* DWORD 7 */
	u32   OpenAddrFrame_Address_High;
/* DWORD 8 */
	u32   StatusBuff_Address;
/* DWORD 9 */
	u32   StatusBuff_Address_High;
/* DWORD 10 */
	u32   PRD_Table_Address;
/* DWORD 11 */
	u32   PRD_Table_Address_High;
/* DWORD 12-15 */
	u32   Reserved[4];
};
/*
************************************************************************
**for command table SSP frame header
************************************************************************
*/
struct _SSP_Frame_Header
{
	unsigned char   Frame_Type;
	unsigned char   Hashed_Dest_SAS_Addr[3];
	unsigned char   Reseved1;
	unsigned char   Hashed_Src_SAS_Addr[3];
	unsigned char   Reseved2[2];
#if defined(__BIG_ENDIAN_BITFIELD)
	unsigned char   Reserved3:5;
	unsigned char   RetryData:1;	/* set in XFER_RDY to allow ReTx Data frames */
	unsigned char   ReTxed:1;	/* set in TASK/XFER_RDY/RESPONSE frames indicating this is a re-txed */
	unsigned char   CDP:1;		/* Change Data Pointer */

	unsigned char   Reserved4:6;
	unsigned char   NumberOfFillBytes:2;
#else
	unsigned char   CDP:1;		/* Change Data Pointer */
	unsigned char   ReTxed:1;	/* set in TASK/XFER_RDY/RESPONSE frames indicating this is a re-txed */
	unsigned char   RetryData:1;	/* set in XFER_RDY to allow ReTx Data frames */
	unsigned char   Reserved3:5;

	unsigned char   NumberOfFillBytes:2;
	unsigned char   Reserved4:6;
#endif
	unsigned char   Reserved5[4];
	unsigned short  Tag;		/* command tag */
	unsigned short  TargetTag;	/* Target Port Transfer Tag, for target to tag multiple XFER_RDY */
	u32  DataOffset;
};
/*
************************************************************************
**SSP Command UI
************************************************************************
*/
struct _SSP_COMMAND_IU
{
	unsigned char   LUN[8];
	unsigned char   Reserved1;
#if defined(__BIG_ENDIAN_BITFIELD)
	unsigned char   AddCDBLen:6;      /* in DW */
	unsigned char   Reserved4:2;
	unsigned char   Reserved3;
	unsigned char   FirstBurst:1;
	unsigned char   Reserved2:4;
	unsigned char   TaskAttrib:3;
#else
	unsigned char   TaskAttrib:3;
	unsigned char   Reserved2:4;
	unsigned char   FirstBurst:1;
	unsigned char   Reserved3;
	unsigned char   Reserved4:2;
	unsigned char   AddCDBLen:6;      /* in DW */
#endif
	unsigned char   CDB[16];
/*   char   AddCDB[n] */
};
/*
************************************************************************
**SSP TASK UI
************************************************************************
*/
struct _SSP_TASK_IU
{
	unsigned char   LUN[8];
	unsigned char   Reserved1[2];
	unsigned char   TaskFunction;
	unsigned char   Reserved2;
	unsigned short   Tag;
	unsigned char   Reserved3[14];
};
/*
************************************************************************
**SSP XFER_RDY UI
************************************************************************
*/
struct _SSP_XFERRDY_IU
{
	u32   DataOffset;
	u32   DataLen;
	unsigned char   Reserved3[4];
};
#define	SCSI_SENSE_CURRENT_ERRORS		0x70
#define 	SCSI_SENSE_DEFERRED_ERRORS	0x71
/*
************************************************************************
************************************************************************
*/
struct _ARCSAS_Sense_Data
{
	uint8_t ResponseCode:7;//Byte 0; In linux this is response code, in Windows this is error code
	uint8_t Valid:1;
	uint8_t SegmentNumber;//Byte 1
	uint8_t SenseKey:4;//Byte 2
	uint8_t Reserved:1;
	uint8_t IncorrectLength:1;
	uint8_t EndOfMedia:1;
	uint8_t FileMark:1;
	uint8_t Information[4];//Byte 3
	uint8_t AdditionalSenseLength;//Byte 7
	uint8_t CommandSpecificInformation[4];//Byte 8
	uint8_t AdditionalSenseCode;//Byte 12
	uint8_t AdditionalSenseCodeQualifier;//Byte 13
	uint8_t FieldReplaceableUnitCode;//Byte 14
	uint8_t SenseKeySpecific[3];//Byte 15
};
/*
************************************************************************
** SSP RESPONSE UI
************************************************************************
*/
struct _SSP_RESPONSE_IU
{
	unsigned char Reserved1[10];
	#ifdef __BIG_ENDIAN_BITFIELD
		unsigned char Reserved2:6;
		unsigned char DataPres:2;
	#else
		unsigned char DataPres:2;
		unsigned char Reserved2:6;
	#endif
	unsigned char Status;
	u32 Reserved3;
	u32 SenseDataLen;
	u32 RespDataLen;
	unsigned char Data[MAX_SSP_RESP_SENSE_SIZE];
};
/*
************************************************************************
** SSP Protection Information Record
************************************************************************
*/
struct _PROTECT_INFO_RECORD
{
	#if defined(__BIG_ENDIAN_BITFIELD)
		u32 Reserved2:4;
		u32 USR_DT_SZ:12;
		u32 Reserved1:10;
		u32 CHK_DSBL_MD:1;
		u32 GRD_CHK:1;
		u32 APP_CHK:1;
		u32 REF_CHK:1;
		u32 RMV_PROT:1;
		u32 INSRT_PROT:1;

		u32 LBRT;      /* Logical Block Reference Tag */

		u32 LBAT_MASK:16;
		u32 LBAT:16;      /* Logical Block Application Tag */
	#else
		u32 INSRT_PROT:1;
		u32 RMV_PROT:1;
		u32 REF_CHK:1;
		u32 APP_CHK:1;
		u32 GRD_CHK:1;
		u32 CHK_DSBL_MD:1;
		u32 Reserved1:10;
		u32 USR_DT_SZ:12;
		u32 Reserved2:4;

		u32 LBRT;      /* Logical Block Reference Tag */

		u32 LBAT:16;      /* Logical Block Application Tag */
		u32 LBAT_MASK:16;
	#endif
};
/*
************************************************************************
** SSP Command Table
************************************************************************
*/
struct _ARCSAS_SSP_Command_Table
{
	SSP_Frame_Header Frame_Header;
	union
	{
		struct {
			struct _SSP_COMMAND_IU commandIU;
			PROTECT_INFO_RECORD PIR;
		} command;
		struct _SSP_TASK_IU task;
		struct _SSP_XFERRDY_IU xfer_rdy;
		struct _SSP_RESPONSE_IU response;
	} data;
};
/*
************************************************************************
** SATA STP Command Table
************************************************************************
*/
struct _ARCSAS_SATA_STP_Command_Table
{
	unsigned char   FIS[64];                       /* Command FIS */
	unsigned char   ATAPI_CDB[32];                    /* ATAPI CDB */
};
/*
************************************************************************
** Open Address Frame
************************************************************************
*/
struct _OPEN_ADDRESS_FRAME
{
	#if defined(__BIG_ENDIAN_BITFIELD)
		unsigned char   Initiator:1;   
		/*
		************************************************************************
		** An INITIATOR PORT bit set to one specifies that the source port is acting as a SAS initiator port.
		** An INITIATOR PORT bit set to zero specifies that the source port is acting as a SAS target port
		************************************************************************
		*/
		unsigned char   Protocol:3;
		#define PROTOCOL_SMP      0x0
		#define PROTOCOL_SSP      0x1
		#define PROTOCOL_STP      0x2
		unsigned char   Frame_Type:4;
		/* defines for address frame types */
		#define ADDRESS_IDENTIFY_FRAME 		0x00
		#define ADDRESS_OPEN_FRAME 		0x01

		unsigned char   Feature:4;
		unsigned char   Connect_Rate:4;
	#else
		unsigned char   Frame_Type:4;
		#define ADDRESS_IDENTIFY_FRAME 		0x00
		#define ADDRESS_OPEN_FRAME 		0x01

		unsigned char   Protocol:3;
		#define PROTOCOL_SMP      0x0
		#define PROTOCOL_SSP      0x1
		#define PROTOCOL_STP      0x2
		unsigned char   Initiator:1;

		unsigned char   Connect_Rate:4;
		unsigned char   Feature:4;
	#endif
		unsigned char   Connect_Tag[2];		/*byte2..byte3:INITIATOR CONNECTION TAG*/
		unsigned char   Dest_SAS_Addr[8];	/*byte4..byte11:DESTINATION SAS ADDRESS*/
			/* HW will generate Byte 12 after... */
		unsigned char   Src_SAS_Addr[8];	/*byte12..byte19:SOURCE SAS ADDRESS*/
		unsigned char   Cmp_Features1;		/*byte20:COMPATIBLE FEATURES*/
		unsigned char   Blocked_Count;		/*byte21:PATHWAY BLOCKED COUNT*/	
		unsigned char   AWT[2];			/*byte22..byte23:ARBITRATION WAIT TIME*/
		unsigned char   Cmp_Features2[4];	/*byte24..byte27:MORE COMPATIBLE FEATURES*/
		u32		FirstBurstSize;		/*byte28..byte31:CRC for hardware use*/
};
/*
************************************************************************
** Error Information Record
************************************************************************
*/
struct _ERR_INFO_RECORD
{
#if defined(__BIG_ENDIAN_BITFIELD)
	/* Error Information Record 0 */
	u32   CMD_ISS_STPD:1;
	u32    PI_ERR:1;
	u32    RESP_BFFR_OFLW:1;
	u32    RTRY_LMT_ERR:1;
	u32    UNKNWN_FIS_ERR:1;
	u32    DMAT_RCVD:1;
	u32    SYNC_ERR:1;
	u32    TFILE_ERR:1;
	u32    R_ERR:1;
	u32    Reserved2:2;
	u32    RD_DATA_OFFST_ERR:1;
	u32    XFR_RDY_OFFST_ERR:1;
	u32    UNEXP_XFER_RDY_ERR:1;
	u32    Reserved1:1;
	u32    DATA_OVR_UNDR_FLW_ERR:1;
	u32    INTRLCK_ERR:1;
	u32    NAK_ERR:1;
	u32    ACK_NAK_TO:1;
	u32    CNCTN_CLSD_ERR:1;
	u32    OPEN_TMOUT_ERR:1;
	u32    PTH_BLKD_ERR:1;
	u32    NO_DEST_ERR:1;
	u32    STP_RSRCS_BSY_ERR:1;
	u32    BRK_RCVD_ERR:1;
	u32    BAD_DEST_ERR:1;
	u32    PRTCL_NOT_SPRTD_ERR:1;
	u32    CNCTN_RT_NT_SPRTD_ERR:1;
	u32    WRONG_DEST_ERR:1;
	u32    CREDIT_TO_ERR:1;
	u32    WD_TMR_TO_ERR:1;
	u32    BFFR_PERR:1;
	/* Error Information Record 1 */
	u32    SLOT_BSY_ERR:1;
	u32    Reserved3:16;
	u32    GRD_CHK_ERR:1;
	u32    APP_CHK_ERR:1;
	u32    REF_CHK_ERR:1;
	u32    USR_BLK_NM:12;
#else
	/* Error Information Record 0 */
	u32    BFFR_PERR:1;
	u32    WD_TMR_TO_ERR:1;
	u32    CREDIT_TO_ERR:1;
	u32    WRONG_DEST_ERR:1;
	u32    CNCTN_RT_NT_SPRTD_ERR:1;
	u32    PRTCL_NOT_SPRTD_ERR:1;
	u32    BAD_DEST_ERR:1;
	u32    BRK_RCVD_ERR:1;
	u32    STP_RSRCS_BSY_ERR:1;
	u32    NO_DEST_ERR:1;
	u32    PTH_BLKD_ERR:1;
	u32    OPEN_TMOUT_ERR:1;
	u32    CNCTN_CLSD_ERR:1;
	u32    ACK_NAK_TO:1;
	u32    NAK_ERR:1;
	u32    INTRLCK_ERR:1;
	u32    DATA_OVR_UNDR_FLW_ERR:1;
	u32    Reserved1:1;      
	u32    UNEXP_XFER_RDY_ERR:1;
	u32    XFR_RDY_OFFST_ERR:1;
	u32    RD_DATA_OFFST_ERR:1;
	u32    Reserved2:2;
	u32    R_ERR:1;
	u32    TFILE_ERR:1;
	u32    SYNC_ERR:1;
	u32    DMAT_RCVD:1;
	u32    UNKNWN_FIS_ERR:1;
	u32    RTRY_LMT_ERR:1;
	u32    RESP_BFFR_OFLW:1;
	u32    PI_ERR:1;
	u32    CMD_ISS_STPD:1;
	/* Error Information Record 1 */
	u32    USR_BLK_NM:12;
	u32    REF_CHK_ERR:1;
	u32    APP_CHK_ERR:1;
	u32    GRD_CHK_ERR:1;
	u32    Reserved3:16;
	u32    SLOT_BSY_ERR:1;
#endif
};
/*
************************************************************************
** Status Buffer
************************************************************************
*/
struct _STATUS_BUFFER
{
	struct _ERR_INFO_RECORD Err_Info;
	union
	{
		struct _SSP_RESPONSE_IU SSP_Resp;
		unsigned char   SMP_Resp[MAX_SMP_RESP_SIZE];
	} data;
};
/*
************************************************************************
** Command Table
************************************************************************
*/
struct _ARCSAS_Command_Table
{
	OPEN_ADDRESS_FRAME Open_Addr_Frame;
	STATUS_BUFFER Status_Buff;

	union{
		ARCSAS_SSP_Command_Table      SSP_CMD_Table;
		ARCSAS_SMP_Command_Table      SMP_CMD_Table;
		ARCSAS_SATA_STP_Command_Table STP_CMD_Table;
	} table;
};
/*
************************************************************************
** Delivery Queue Entry
************************************************************************
*/
struct _POST_Q_ENTRY
{
#ifdef __BIG_ENDIAN_BITFIELD
	u32 CMD:3;
	u32 MODE:1;
	u32 PRIORITY:1;
	u32 SATA_REG_SET:7;
	u32 PHY:8;
	u32 SLOT_NM:12;
#else
	u32 SLOT_NM:12;
	u32 PHY:8;
	u32 SATA_REG_SET:7;
	u32 PRIORITY:1;
	u32 MODE:1;
	u32 CMD:3;
#endif
};
/*
************************************************************************
**	Completion Queue Entry DWORD, 
**	32 bit,include SLOT_NM:12 for complete Q tag number
************************************************************************
*/
struct _DONE_Q_ENTRY
{
	#ifdef __BIG_ENDIAN_BITFIELD
		u32 Reserved2:9;
		u32 RSPNS_GOOD:1;
		u32 SLOT_RST_CMPL:1;
		u32 CMD_RCVD:1;        /* target mode */
		u32 ATTENTION:1;
		u32 RSPNS_XFRD:1;
		u32 ERR_RCRD_XFRD:1;
		u32 CMD_CMPL:1;

		u32 Reserved1:4;
		u32 SLOT_NM:12;
	#else
		u32 SLOT_NM:12;
		u32 Reserved1:4;

		u32 CMD_CMPL:1;
		u32 ERR_RCRD_XFRD:1;
		u32 RSPNS_XFRD:1;
		u32 ATTENTION:1;
		u32 CMD_RCVD:1;        /* target mode */
		u32 SLOT_RST_CMPL:1;
		u32 RSPNS_GOOD:1;
		u32 Reserved2:9;
	#endif
};
/*
************************************************************************
**  For obtaining WWN, inquiry EVPD data structures
************************************************************************
*/
typedef struct _DesignationDescriptor
{
	unsigned char CodeSet:4;
	unsigned char ProtocolIdentifier:4;
    	/*byte0*/
	unsigned char DesignatorType:4;
	unsigned char Association:2;
	unsigned char Reserved1:1;
	unsigned char PIV:1;
	/*byte1*/
	unsigned char Reserved2;
	/*byte2*/
	unsigned char DesignatorLength;
	/*byte3*/
}DesignationDescriptor, *PDesignationDescriptor;
/*
************************************************************************
************************************************************************
*/
struct _SAS_Scratch_Buffer 
{
	struct osm_list_head	Queue_Pointer;
	void		 *Scratch_Vir;
	ARCSAS_U64	Scratch_DMA;
};
/*
************************************************************************
************************************************************************
*/
struct _SMP_Scratch_Buffer 
{
	struct osm_list_head	Queue_Pointer;
	SMPRequest	SMP_Req;
};
/*
************************************************************************
************************************************************************
*/
struct _PRD_Buffer 
{
	struct osm_list_head	Queue_Pointer;
	void 		*PRD_VIRTUAL;
	ARCSAS_U64	PRD_PHYSICAL;
};
/*
************************************************************************
************************************************************************
*/
struct _CORE_CONTEXT 
{
	struct osm_list_head Queue_Pointer;
	void *pointer;
	/* Should move the Scratch_Buffer variable here. 
	   But so far CSMI also use it. */
	unsigned char Context_Type;
	union {
		struct {
			unsigned char Original_CDB[MAX_CDB_SIZE];
			void *Data_Buffer;
		}cdb;
		struct {
			unsigned short start;
			unsigned short end;
			u32 remaining;
			void *pointer;
		}api_req;
	}u;
};
/*
************************************************************************
************************************************************************
*/
struct _ARCSAS_SG_Table {
	u32			Byte_Count;
	PARCSAS_SGENTRY		pSGEntry;
	unsigned short		Max_Entry_Count;
	unsigned short		Valid_Entry_Count;
};
/*
************************************************************************
** struct CommandControlBlock is the general request type passed through different modules. 
** Must be 64 byte aligned.
************************************************************************
*/
struct CommandControlBlock {
	struct osm_list_head			Queue_Pointer;
	struct osm_list_head			Complete_Queue_Pointer;
	unsigned char				CCB_Status;
	unsigned char				Tag;				/* Request tag */
	unsigned char				CLI_done;
	/*
	***************************************
	**      CCB_TYPE_OS = 2,
	**      CCB_TYPE_CACHE,
	**      CCB_TYPE_RAID,
	**      CCB_TYPE_MP,
	**      CCB_TYPE_INTERNAL,
	***************************************
	*/
	unsigned char				Sense_Info_Buffer_Length;
	unsigned char				Cdb[MAX_CDB_SIZE];
	unsigned short				EntryIndex;			/* sas device device id, scsilun-scsiid */
	unsigned short				CCB_Flag;			/* Check the CCB_FLAG definition */
	unsigned short				SlotNo; 
	unsigned short				NCQTAG;                     		/* for SATA NCQ tag */
	u32					Data_Transfer_Length; 
	u32					Sector_Count;
	u32					Cmd_Flag;
	int					retry;
	void					*Cmd_Initiator;			/* Which module(extension pointer) creates this request. */
	void					*Data_Buffer; 
	void					*Sense_Buffer;
	void					*Org_Req;			/* The original request. */ 
	void					*Scratch_Buffer;			/* pointer to the scratch buffer that this request used */
	void					(*Completion)(void *, PCCB);	/* call back function */
	u64					Time_Out;			/* how many seconds we should wait before treating request as timed-out */
	PCore_Context				CoreContext;			/* Each module should only use Context to store module information. */
	PPRD_Buffer				PRD_Buffer;
	PDomain_Device				pDevice;
	PDomain_PM				pPM;
	PDomain_Expander			pExpander;
	ARCSAS_U64				LBA;
	unsigned short				Cdb_len;
	unsigned short				TargetId;
	u32					Lun;
	struct _ARCSAS_SG_Table			SG_Table;
};
/*
************************************************************************
************************************************************************
*/
union _SAS_ADDR
{
	struct {
		u32		low;
		u32		high;
	} parts;
	unsigned char		b[8];
	unsigned short		w[4];
	u32			d[2];       
};
/*
************************************************************************
************************************************************************
*/
struct _PHY_TUNING 
{
	unsigned char	AMP:4;			/* 4 bits,  amplitude  */
	unsigned char	Pre_Emphasis:3;		/* 3 bits,  pre-emphasis */
	unsigned char	Reserved_1bit_1:1;	/* 1 bit,   reserved space */
	unsigned char	Drive_En:6;		/* 6 bits,  drive enable */
	unsigned char	Pre_Half_En:1;		/* 1 bit,	Half Pre-emphasis Enable*/
	unsigned char	Reserved_1bit_2:1;	/* 1 bit, 	reserved space */
	unsigned char	Reserved[2];		/* 2 bytes, reserved space */
};
/*
************************************************************************
************************************************************************
*/
typedef struct _Domain_SGPIO
{
	unsigned char	Data_In_Low[8];
	unsigned char	Data_In_High[8];
	u32		CRC;
}Domain_SGPIO, *PDomain_SGPIO;
/*
************************************************************************
************************************************************************
*/
typedef struct _sgpio_config_reg 
{
	unsigned char       sup_dev_cnt;
	unsigned char       gp_reg_cnt:4;
	unsigned char       cfg_reg_cnt:3;
	unsigned char       sgpio_enable:1;
	unsigned char       version:4;
	unsigned char       rsrv_byte2_bit4_7:4;
	unsigned char       rsrv_byte3;
}sgpio_config_reg, *psgpio_config_reg;
/*
************************************************************************
**	BIOS_Info_Page is saved in Flash/NVRAM, total 256 bytes.
**	The data area is valid only Signature="MRVL".
**	If any member fills with 0xFF, the member is invalid.
************************************************************************
*/
struct _BIOS_Info_Page
{
	// Dword 0
	unsigned char     	Signature[4];                	/* 4 bytes, structure signature,should be "MRVL" at first initial */

	// Dword 1
	unsigned char     	MinorRev;                 	/* 2 bytes, NVRAM data structure version */
	unsigned char		MajorRev;
	unsigned short    	Next_Page;		/* 2 bytes, For future data structure expansion, 0 is for NULL pointer and current page is the last page. */

	// Dword 2
	unsigned char     	Major;                   	/* 1 byte, BIOS major version */
	unsigned char     	Minor;                  		/* 1 byte, BIOS minor version */
	unsigned char     	OEM_Num;                     	/* 1 byte, OEM number */
	unsigned char     	Build_Num;                    	/* 1 byte, Build number */

	// Dword 3
	unsigned char     	Page_Code;		/* 1 byte, eg. 0 for the 1st page  */
	unsigned char     	Max_PHY_Num;		/* 1 byte, maximum PHY number */
	unsigned char		Reserved2[2];

	// Dword 4
	u32     		BIOS_Flag;                     	/* 4 bytes, should be 0x0000,0000 at first initial
						HBA flag:  refers to HBA_FLAG_XX
						bit 0   --- HBA_FLAG_BBS_ENABLE
						bit 1   --- BIOS_FLAG_SILENT_MODE_ENABLE
						bit 2   --- HBA_FLAG_ERROR_STOP
						bit 3   --- BIOS_FLAG_INT13_ENABLE
						bit 4   --- HBA_FLAG_ERROR_STOP
						bit 5   --- HBA_FLAG_ERROR_PASS
						bit 6   --- BIOS_FLAG_SILENT_MODE_ENABLE
						*/
	// Dword 5	                                              
	u32     		Boot_Device;		/* 4 bytes, select boot device */
						/* for ata device, it is CRC of the serial number + model number. */
						/* for sas device, it is CRC of sas address */
						/* for VD, it is VD GUI */

	// Dword 6-8											  
	u32     		Reserved3[3];		/* 12 bytes, reserved	*/

	// Dword 9-13
	unsigned char     	Serial_Num[20];		/* 20 bytes, controller serial number */

	// Dword 14-29
	SAS_ADDR		SAS_Address[8];		/* 64 bytes, SAS address for each port */

	// Dword 30-43
	unsigned char     	Reserved4[56];		/* 56 bytes, reserve space for future,initial as 0xFF */   

	// Dword 44-45
	unsigned char     	PHY_Rate[8];		/* 8 bytes,  0:  1.5G, 1: 3.0G, should be 0x01 at first initial */

	// Dword 46-53
	PHY_TUNING		PHY_Tuning[8];		/* 32 bytes, PHY tuning parameters for each PHY*/

	// Dword 54-62
	u32			Reserved5[9];		/* 9 dword, reserve space for future,initial as 0xFF */

	// Dword 63
	unsigned char     	Reserved6[3];		/* 3 bytes, reserve space for future,initial as 0xFF */
	unsigned char     	Check_Sum;		/* 1 byte,   checksum for this structure,Satisfy sum of every 8-bit value of this structure */
};				/* total 256 bytes */
/*
************************************************************************
** adapter    Flash_Devices_Map[128];
************************************************************************
*/
#pragma pack(push, 4)
struct _FLASH_DEVICE_MAP
{
	u32		Flag;	/*DWord0*/   
	ARCSAS_U64	Pattern;	/*DWord1,DWord2*/
};
#pragma pack(pop)
/*
************************************************************************
** adapter descriptor
************************************************************************
*/
struct AdapterControlBlock 
{
	struct workqueue_struct	*scsi_lookup;
	struct _Core_Extension	*pCore[ARCSAS_CORE_NUMBER];
	PDISCOVERY_ELEMENT	pDiscovery_element;
	struct tasklet_struct	sata_error_handling;
	struct pci_dev		*pcidev;
	struct Scsi_Host	*host;
	void __iomem		*mem_map_base_ext;
	uint8_t           	irq;
	/* Timer Callback */
	//struct osm_list_head	timerlist;
	unsigned char		DevicesExpandersPMs_EntryIndex[ARCSAS_MAX_PM_ID + 1];		/*(0..127,128..143,144..151) must be 4 byte alignment */
	unsigned char		SPICmd[16];/*record: MXIC_SPI_CMD or ATMEL_SPI_CMD*/
	/* adapter information */
	unsigned char		Revision_Id;
				#define ODIN2_00_REVISION	0x00
				#define VANIR_A0_REVISION	0xA0
				#define VANIR_B0_REVISION	0x01
				#define VANIR_C0_REVISION	0x02
				#define VANIR_C1_REVISION	0x03
	unsigned char		enclosure_mask;
	void *			dma_coherent;
	/* dma_coherent used for memory free */
	osm_dma_addr_t		dma_coherent_handle;
	/* dma_coherent_handle used for memory free */
	unsigned short		dma_size;
	unsigned short		vendor;
	unsigned short		device;
	unsigned short		system_device_id;
	unsigned short		subsystem_vendor;
	unsigned short		subsystem_device;

	unsigned short		FlashID;
	unsigned short		acb_flag;

	/* System resource */
	/* 
	**************************************************************
	** Because we are supporting performance mode, 
	** we cannot use array for Domain_Device 
	** user pointer, remember to allocate continuous memory for it
	**************************************************************
	*/
 	EVENT_ELEMENT		*EventElement; 
	PDomain_Device		DevicesLun;						/*is scsi lun device,128*/
	PDomain_Device		Devices;						/*is scsi device,128,MAX_DEVICE_ID*/
	Domain_Expander		*Expanders;						/*is SAS Expander,8,ARCSAS_EXPANDER_SUPPORTED*/
	Domain_PM		*PMs;							/*is SATA Port Mutiplier,8,MAX_PM_SUPPORTED_PERFORMANCE*/
	Tag_Stack		Device_Tag_Pool;      /*128*/
	Tag_Stack		Expander_Tag_Pool;    /*16*/
	Tag_Stack		PM_Tag_Pool;          /*8*/
	DISCOVERY_ELEMENT	discoveryQ[ARCSAS_DISCOVERY_DPC];
	atomic_t		outstandingccbcount;
	atomic_t		all_device_count;
	u32			event_element_index;
	u32			BIOS_Flag;
//	u64			sata_register_set;
	struct _FLASH_DEVICE_MAP	Flash_Devices_Map[128];
	unsigned char		last_expander_mapped_id;
	unsigned char		last_portmutiplier_mapped_id;
	unsigned char		last_device_mapped_id;
	unsigned char		Device_Count_Supported;		 /* 136, port device supported count    */
	unsigned char		Expander_Count_Supported;             /*   8, port expander supported count  */ 
	unsigned char		PortMutiplier_Count_Supported; 
	int			discoveryQ_index;
	int			discoveryQ_count;
	//struct workqueue_struct	*dpc_queue;	/* Event queue */
	spinlock_t 		host_lock;
	spinlock_t		discoveryQ_lock;
	spinlock_t		doneq_lock;
	spinlock_t		event_element_lock;
	spinlock_t		SMART_timer_lock;
	spinlock_t		ehandler_lock;
	spinlock_t		dv_event_lock;
	struct timer_list	SMART_timer_request;            /* for smart polling */
	atomic_t		SMART_timer_status;
	Domain_Device		SesSgpioVirtualDevice;/*scsi id 136,ARCSAS_VIRTUAL_DEVICE_ID*/
	unsigned char		EnclosureElementBuffer[8][SES_STATUS_CONTROL_BUFFER_SIZE];/* index 01234567 is real ses device,index 8 is virtual ses device */
	struct task_struct	*eh_handler;
	struct task_struct	*lookup_devices_handler;
	struct work_struct	arcsas_scsi_scan_host;
	struct work_struct 	Polling_DiscoveryInstance;
	struct osm_list_head	timeout_list;
	struct list_head	dv_event_list;
};

struct dv_event_work {
        struct list_head        list;
        struct work_struct      work;
        struct delayed_work     delayed_work;
        struct AdapterControlBlock *pACB;
        PDomain_Device		pDevice;
        u16                     device_handle;
        u8                      cancel_pending_work;
        u8                      VF_ID;
        u8                      VP_ID;
        u8                      ignore;
        u16                     event;
        void                    *event_data;
};

/*
************************************************************************
************************************************************************
*/
struct _Link_Endpoint 
{
	unsigned short      DevID;
	unsigned char       DevType;         /* Refer to DEVICE_TYPE_xxx, (additional type like EDGE_EXPANDER and FANOUT_EXPANDER might be added). */
	unsigned char       PhyCount;          /* Number of PHYs for this endpoint.  Greater than 1 if it is wide port. */
	unsigned char       PhyId[ARCSAS_WIDEPORT_PHYS];   /* Assuming wide port has max of 8 PHYs. */
	unsigned char       SAS_Address[8]; /* Filled with 0 if not SAS device. */
	unsigned char       Reserved1[12];
};
/*
************************************************************************
************************************************************************
*/
struct _Link_Entity 
{
	Link_Endpoint	Parent;
	unsigned char	Reserved[8];
	Link_Endpoint	Self;
};
/*
************************************************************************
************************************************************************
*/
struct _Version_Info
{
	u32        VerMajor;
	u32        VerMinor;
	u32        VerOEM;
	u32        VerBuild;
};
/*
************************************************************************
** total 104 bytes 
************************************************************************
*/
struct _Adapter_Info 
{
	Version_Info	DriverVersion;
	Version_Info	BIOSVersion;
	Version_Info    FirmwareVersion;

	u32		SystemIOBusNumber;
	u32		SlotNumber;
	u32		InterruptLevel;
	u32		InterruptVector;
    
	unsigned short	VenID;
	unsigned short	SubVenID;
	unsigned short	DevID;
	unsigned short	SubDevID;
    
	unsigned char	PortCount;        /* How many ports, like 4 ports, or 4S1P. */
	unsigned char 	PortSupportType;  /* Like SATA port, SAS port, PATA port, use ARCSAS_BIT */
	unsigned char	Features;         /* Feature bits.  See FEATURE_XXX. If corresponding bit is set, that feature is supported. */
	unsigned char	AlarmSupport;
	unsigned char	RevisionID;		  /* Chip revision */
	unsigned char	Reserved2[11];
    
	unsigned char	MaxTotalBlocks;    
	unsigned char	MaxBlockPerPD;
	unsigned char	MaxHD;
	unsigned char	MaxExpander;
	unsigned char	MaxPM;
	unsigned char	MaxLogicalDrive;
	unsigned short	LogicalDriverMode;

	unsigned char	WWN[8];           /* For future VDS use. */
};
/*
************************************************************************
** SCSI DEVICE INFO
** get host peripheral device infomation,  total 248 bytes
** DEVICE_TYPE_HD,DEVICE_TYPE_TAPE,DEVICE_TYPE_ENCLOSURE
************************************************************************
*/
struct _HD_Info
{
	Link_Entity	Link;             /* Including self DevID & DevType */
	unsigned char	AdapterID;
	unsigned char	Status;           /* Refer to HD_STATUS_XXX */
	unsigned char	HDType;           /* HD_Type_xxx, replaced by new driver with ConnectionType & DeviceType */
	unsigned char	PIOMode;		  /* Max PIO mode */
	unsigned char	MDMAMode;		  /* Max MDMA mode */
	unsigned char	UDMAMode;		  /* Max UDMA mode */
	unsigned char	ConnectionType;	  /* DC_XXX, ConnectionType & DeviceType in new driver to replace HDType above */
	unsigned char	DeviceType;	      /* DT_XXX */
	u32		FeatureSupport;    /* Support 1.5G, 3G, TCQ, NCQ, and etc, ARCSAS_BIT related */
	unsigned char	Model_Number[40];
	unsigned char	Serial_Number[20];
	unsigned char	Firmware_Revision[8];
	ARCSAS_U64	Size;               /* unit: 1KB */
	unsigned char	WWN[8];            /* ATA/ATAPI-8 has such definitions for the identify buffer */
	unsigned char	CurrentPIOMode;		/* Current PIO mode */
	unsigned char	CurrentMDMAMode;	/* Current MDMA mode */
	unsigned char	CurrentUDMAMode;	/* Current UDMA mode */
	unsigned char	SesOverallElementIndex;
	u32		BlockSize;
	unsigned char	ActivityLEDStatus;
	unsigned char	LocateLEDStatus;
	unsigned char	ErrorLEDStatus;
	unsigned char	SesElementType;
	unsigned char	SesSlotNumber;
	unsigned char	HD_SSD_Type;
	u16		SesEntryIndex;
	u16		Rotation_Rate;
	unsigned char	Reserved1[70];
};
#define SD_STATUS_WRITE_PROTECTED               ARCSAS_BIT(1, 0)
/*
************************************************************************
**              total 132 bytes 
************************************************************************
*/
struct _Expander_Info
{
	Link_Entity	Link;            /* Including self DevID & DevType */

	unsigned char	AdapterID;
	Bool		Configuring;      
	Bool		RouteTableConfigurable;
	unsigned char	PhyCount;

	unsigned short	ExpChangeCount;
	unsigned short	MaxRouteIndexes;

	unsigned char	VendorID[8 + 1];
	unsigned char	ProductID[16 + 1];
	unsigned char	ProductRev[4 + 1];
	unsigned char	ComponentVendorID[8 + 1];

	unsigned short	ComponentID;
	unsigned char	ComponentRevisionID;
	unsigned char	Reserved1[17];
};
/*
************************************************************************
************************************************************************
*/
struct _PM_Info
{
	Link_Entity	Link;           /* Including self DevID & DevType */
	unsigned char	AdapterID;
	unsigned char	ProductRevision;
	unsigned char	PMSpecRevision; /* 10 means 1.0, 11 means 1.1 */
	unsigned char	NumberOfPorts;
	unsigned short	Pci_Ven_ID;
	unsigned short	Pci_Dev_ID;
	unsigned char	Reserved1[8];
};
/*
************************************************************************
************************************************************************
*/
struct _SD_Config
{  
	Bool		WriteCacheOn;   /* 1: enable write cache */
	Bool		SMARTOn;        /* 1: enable S.M.A.R.T */
	Bool		Online;         /* 1: to set HD online */
	unsigned char	DriveSpeed;	  // For SATA & SAS.  SD_SPEED_1_5G, SD_SPEED_3G etc
	unsigned char	Reserved[2];
	unsigned short	ScsiDeviceId;
};
/*
************************************************************************
************************************************************************
*/
struct  _SD_SmartStatus
{
	Bool		SmartThresholdExceeded;        
	unsigned char	CCB_Status;
	unsigned short	ScsiDeviceId;
};
/*
************************************************************************
************************************************************************
*/
struct _PassThrough_Config
{
	// We put Data_Buffer[] at the very beginning of this structure because SCSI commander did so.
	unsigned char	Data_Buffer[MAX_PASS_THRU_DATA_BUFFER_SIZE]; // set by driver if read, by application if write
	unsigned char	Reserved1[128];
	u32		Data_Length;	// set by driver if read, by application if write 
	unsigned short	DevId;		//	PD ID (used by application only)
	unsigned char	CDB_Type;	// define a CDB type for each CDB category (used by application only)
	unsigned char	Reserved2;
	u32		lba;
	unsigned char	Reserved3[64];
};
/*
************************************************************************
************************************************************************
*/
struct _Assigned_Uncached_Memory
{
	void *			Virtual_Address;
	ARCSAS_U64		Physical_Address;
	u32			Byte_Size;
	u32			Reserved0;
};
/*
************************************************************************
************************************************************************
*/
struct mv_hba_msg 
{
	void *			data;
	u32			msg;
	u32			param;
	struct osm_list_head	msg_list;
};
/*
************************************************************************
************************************************************************
*/
/*
 * SCSI Inquiry Data Format
 */
struct SCSI_INQ_DATA
{
	unsigned char   DeviceType;
	unsigned char   DeviceTypeQualifier;
	unsigned char   Version;
	unsigned char   ResponseDataFormat;
	unsigned char   AdditionalLength;
	unsigned char   Reserved;
	unsigned char   Flags[2];
	unsigned char   VendorId[8];
	unsigned char   ProductId[16];
	unsigned char   ProductRevisionLevel[4];
	unsigned char   Reserved2;                                  /* Provides NULL terminator to name */
};
typedef struct _UNICODE_STRING {
	unsigned short Length;
	unsigned short MaximumLength;
	unsigned short *Buffer;
} UNICODE_STRING, *PUNICODE_STRING;

typedef struct _ANSI_STRING {
	unsigned short Length;
	unsigned short MaximumLength;
	unsigned char *Buffer;
} ANSI_STRING, *PANSI_STRING;
/*
************************************************************************
************************************************************************
*/
/* H/W Access Routines */
#define READ_PORT_IRQ_STATE(pACB, mem_map_base, phy_index)		((phy_index > 3) ? osm_readl(mem_map_base + port_irq_state4 + (phy_index - 4) * 8) : osm_readl(mem_map_base + port_irq_state0 + phy_index * 8)) 
#define WRITE_PORT_IRQ_STATE(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_irq_state4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_irq_state0 + phy_index * 8);} 
#define READ_PORT_IRQ_MASK(pACB, mem_map_base, phy_index)		((phy_index > 3) ? osm_readl(mem_map_base + port_irq_mask4 + (phy_index - 4) * 8) : osm_readl(mem_map_base + port_irq_mask0 + phy_index * 8)) 
#define WRITE_PORT_IRQ_MASK(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_irq_mask4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_irq_mask0 + phy_index * 8);}
#define READ_PORT_PHY_CONTROL(pACB, mem_map_base, phy_index)		((phy_index > 3) ? osm_readl(mem_map_base + port_phy_control4 + (phy_index - 4) * 4) : osm_readl(mem_map_base + port_phy_control0 + phy_index * 4)) 
#define WRITE_PORT_PHY_CONTROL(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_phy_control4 + (phy_index - 4) * 4); else osm_writel(temp, mem_map_base + port_phy_control0 + phy_index * 4);} 
#define READ_PORT_CONFIG_DATA(pACB, mem_map_base, phy_index)		((phy_index > 3) ? osm_readl(mem_map_base + port_config_data4 + (phy_index - 4) * 8) : osm_readl(mem_map_base + port_config_data0 + phy_index * 8)) 
#define WRITE_PORT_CONFIG_DATA(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_config_data4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_config_data0 + phy_index * 8);} 
#define WRITE_PORT_CONFIG_ADDR(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_config_addr4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_config_addr0 + phy_index * 8);} 
#define READ_PORT_VSR_DATA(pACB, mem_map_base, phy_index)		((phy_index > 3) ? osm_readl(mem_map_base + port_vsr_data4 + (phy_index - 4) * 8) : osm_readl(mem_map_base + port_vsr_data0 + phy_index * 8)) 
#define WRITE_PORT_VSR_DATA(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_vsr_data4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_vsr_data0 + phy_index * 8);} 
#define WRITE_PORT_VSR_ADDR(pACB, mem_map_base, phy_index, temp)	{if (phy_index > 3) osm_writel(temp, mem_map_base + port_vsr_addr4 + (phy_index - 4) * 8); else osm_writel(temp, mem_map_base + port_vsr_addr0 + phy_index * 8);} 

#define ARCSAS_IS_PHY_READY(pACB, mem_map_base, id)			(READ_PORT_PHY_CONTROL(pACB, mem_map_base, id) & ARCSAS_PHY_READY_MASK)
#define ARCSAS_IS_PHY_CHANGE(pACB, mem_map_base, id)			(READ_PORT_IRQ_STATE(pACB, mem_map_base, id) & IRQ_PHY_RDY_CHNG_MASK)
#define ARCSAS_IS_ASYNC_NOTIFY(pACB, mem_map_base, id)			(READ_PORT_IRQ_STATE(pACB, mem_map_base, id) & ASYNC_NTFCN_RCVD_MASK)
#define ARCSAS_CLEAR_PORT_IRQ(pACB, mem_map_base, id)			{u32 reg; reg = READ_PORT_IRQ_STATE(pACB, mem_map_base, id); WRITE_PORT_IRQ_STATE(pACB, mem_map_base, id, reg);}
#define ARCSAS_IS_BROADCAST_CHANGE(pACB, mem_map_base, id)		(READ_PORT_IRQ_STATE(pACB, mem_map_base, id) & IRQ_BRDCST_CHNG_RCVD_MASK)
#define ARCSAS_IS_PHY_DEC_ERROR(pACB, mem_map_base, id)			(READ_PORT_IRQ_STATE(pACB, mem_map_base, id) & STP_SATA_PHY_DEC_ERR_MASK)
#define ARCSAS_IS_UNASSOC_FIS(pACB, mem_map_base, id)			(READ_PORT_IRQ_STATE(pACB, mem_map_base, id) & IRQ_UNASSOC_FIS_RCVD_MASK)
#define ARCSAS_GET_COMMON_IRQ_STATUS(mem_map_base, irqs)		{irqs = osm_readl(mem_map_base + COMMON_IRQ_STATE);}

#define READ_PORT_IRQ_STATE_1320(mem_map_base, phy_index)		(osm_readl(mem_map_base + PORT_IRQ_STATE0_1320 + phy_index * 8)) 
#define WRITE_PORT_IRQ_STATE_1320(mem_map_base, phy_index, temp)	osm_writel(temp, mem_map_base + PORT_IRQ_STATE0_1320 + phy_index * 8) 
#define READ_PORT_IRQ_MASK_1320(mem_map_base, phy_index)		(osm_readl(mem_map_base + PORT_IRQ_MASK0_1320 + phy_index * 8)) 
#define WRITE_PORT_IRQ_MASK_1320(mem_map_base, phy_index, temp)		osm_writel(temp, mem_map_base + PORT_IRQ_MASK0_1320 + phy_index * 8)
#define READ_PORT_PHY_CONTROL_1320(mem_map_base, phy_index)		(osm_readl(mem_map_base + PORT_PHY_CONTROL0_1320 + phy_index * 4)) 
#define WRITE_PORT_PHY_CONTROL_1320(mem_map_base, phy_index, temp)	osm_writel(temp, mem_map_base + PORT_PHY_CONTROL0_1320 + phy_index * 4) 
#define READ_PORT_CONFIG_DATA_1320(mem_map_base, phy_index)		(osm_readl(mem_map_base + PORT_CONFIG_DATA0_1320 + phy_index * 8)) 
#define WRITE_PORT_CONFIG_DATA_1320(mem_map_base, phy_index, temp)	osm_writel(temp, mem_map_base + PORT_CONFIG_DATA0_1320 + phy_index * 8) 
#define WRITE_PORT_CONFIG_ADDR_1320(mem_map_base, phy_index, temp)	osm_writel(temp, mem_map_base + PORT_CONFIG_ADDR0_1320 + phy_index * 8) 
#define READ_PORT_VSR_DATA_1320(mem_map_base, phy_index)		(osm_readl(mem_map_base + PORT_VSR_DATA0_1320 + phy_index * 8)) 
#define WRITE_PORT_VSR_DATA_1320(mem_map_base, phy_index, temp)		osm_writel(temp, mem_map_base + PORT_VSR_DATA0_1320 + phy_index * 8) 
#define WRITE_PORT_VSR_ADDR_1320(mem_map_base, phy_index, temp)		osm_writel(temp, mem_map_base + PORT_VSR_ADDR0_1320 + phy_index * 8) 

#define ARCSAS_IS_PHY_READY_1320(mem_map_base, id)		(READ_PORT_PHY_CONTROL_1320(mem_map_base, id) & ARCSAS_PHY_READY_MASK)
#define ARCSAS_IS_PHY_CHANGE_1320(mem_map_base, id)		(READ_PORT_IRQ_STATE_1320(mem_map_base, id) & IRQ_PHY_RDY_CHNG_MASK)
#define ARCSAS_IS_ASYNC_NOTIFY_1320(mem_map_base, id)		(READ_PORT_IRQ_STATE_1320(mem_map_base, id) & ASYNC_NTFCN_RCVD_MASK)
#define ARCSAS_CLEAR_PORT_IRQ_1320(mem_map_base, id)		{u32 reg; reg = READ_PORT_IRQ_STATE_1320(mem_map_base, id); WRITE_PORT_IRQ_STATE_1320(mem_map_base, id, reg);}
#define ARCSAS_IS_BROADCAST_CHANGE_1320(mem_map_base, id)	(READ_PORT_IRQ_STATE_1320(mem_map_base, id) & IRQ_BRDCST_CHNG_RCVD_MASK)
#define ARCSAS_IS_UNASSOC_FIS_1320(mem_map_base, id)		(READ_PORT_IRQ_STATE_1320(mem_map_base, id) & IRQ_UNASSOC_FIS_RCVD_MASK)

#define SATA_SIGNATURE_STATUS_MASK					0xc0
#define SATA_DEVICE_READY						0x40

#define ARCSAS_REG_READ_DWORD(mem_map_base, offset)		osm_readl(mem_map_base + offset)
#define ARCSAS_REG_WRITE_DWORD(mem_map_base, offset, temp)	osm_writel(temp, mem_map_base + offset)

/*
************************************************************************
************************************************************************
*/
#define DEVICE_ENTRY_INDEX_TO_TARGET_ID(_dev_id)	((unsigned char)((_dev_id) & 0x00FF))
#define DEVICE_ENTRY_INDEX_TO_TARGET_LUN(_dev_id)	((unsigned char)(((_dev_id) & 0xFF00) >> 8))
#define MAKE_DEVICE_ENTRY_INDEX(_target_id, _lun)	(((unsigned short)(_target_id)) | (((unsigned short)(_lun)) << 8))
#define U64_SET_VALUE(v64, v32)				do {v64.value = v32;} while (0)
#define SCSI_IS_READ(cmd)				((cmd == SCSI_CMD_READ_6) || (cmd == SCSI_CMD_READ_10)  || (cmd == SCSI_CMD_READ_12)  || (cmd == SCSI_CMD_READ_16))
#define SCSI_IS_WRITE(cmd)				((cmd == SCSI_CMD_WRITE_6) || (cmd == SCSI_CMD_WRITE_10) || (cmd == SCSI_CMD_WRITE_12) || (cmd == SCSI_CMD_WRITE_16))
#define SCSI_IS_MODE_SENSE(cmd)				((cmd == SCSI_CMD_MODE_SENSE_6) || (cmd == SCSI_CMD_MODE_SENSE_10))
#define SCSI_IS_REQUEST_SENSE(cmd)			((cmd == SCSI_CMD_REQUEST_SENSE))
#define SCSI_IS_VERIFY(cmd)				((cmd == SCSI_CMD_VERIFY_10) || (cmd == SCSI_CMD_VERIFY_16))
#define SCSI_IS_ATA_PASSTHRU_CMD(cmd)			((cmd == SCSI_CMD_ATA_PASSTHRU_16) ||(cmd == SCSI_CMD_ATA_PASSTHRU_12))
#define SCSI_IS_INTERNAL(cmd)				((cmd) == SCSI_CMD_ARECA_SPECIFIC)
#define SCSI_IS_SES(cmd)				((cmd == SCSI_CMD_RCV_DIAG_RSLT) ||(cmd == SCSI_CMD_SND_DIAG))
/* SCSI command CDB helper functions. */
#define SCSI_CDB10_GET_LBA(cdb)				((u32)(((u32) cdb[2] << 24) | ((u32) cdb[3] << 16) | ((u32) cdb[4] << 8)  | (u32) cdb[5]))
#define SCSI_CDB10_SET_LBA(cdb, lba)			{cdb[2] = (unsigned char)(lba >> 24); cdb[3] = (unsigned char)(lba >> 16); cdb[4] = (unsigned char)(lba >> 8); cdb[5] = (unsigned char)lba;}
#define SCSI_CDB10_GET_SECTOR(cdb)			((cdb[7] << 8) | cdb[8])
#define SCSI_CDB10_SET_SECTOR(cdb, sector)		{cdb[7] = (unsigned char)(sector >> 8); cdb[8] = (unsigned char)sector;}

#define IS_A_SMP_REQ(pCCB)				((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_SMP))
#define IS_A_TSK_REQ(pCCB)				((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_TASK_MGMT))
#define IS_A_SOFT_RESET_REQ(pCCB)			((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_SOFT_RESET_1 || pCCB->Cdb[2] == CDB_CORE_SOFT_RESET_0))
#define IS_A_PM_REQ(pCCB)				((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_PM_READ_REG || pCCB->Cdb[2] == CDB_CORE_PM_WRITE_REG))
#define IS_A_SATA_ENABLE_READ_AHEAD_REQ(pCCB)		((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_ENABLE_READ_AHEAD))
#define IS_A_SATA_READ_LOG_EXT_REQ(pCCB)		((pCCB->Cdb[0] == SCSI_CMD_ARECA_SPECIFIC) && (pCCB->Cdb[1] == CDB_CORE_MODULE) && (pCCB->Cdb[2] == CDB_CORE_READ_LOG_EXT))
#define IS_ATA_MULTIPLE_READ_WRITE(x)			((x == ATA_CMD_READ_PIO_MULTIPLE) || (x == ATA_CMD_READ_PIO_MULTIPLE_EXT) || (x == ATA_CMD_WRITE_PIO_MULTIPLE) || (x == ATA_CMD_WRITE_PIO_MULTIPLE_EXT) || (x == ATA_CMD_WRITE_PIO_MULTIPLE_FUA_EXT))
/* The SHA f()-functions.  */
#define f1(x,y,z)					(z ^ (x & (y ^ z)))	/* x ? y : z */
#define f2(x,y,z)					(x ^ y ^ z)		/* XOR */
#define f3(x,y,z)					((x & y) + (z & (x ^ y)))	/* majority */
/* The SHA Mysterious Constants */
#define K1						0x5A827999L		/* Rounds  0-19: sqrt(2) * 2^30 */
#define K2						0x6ED9EBA1L		/* Rounds 20-39: sqrt(3) * 2^30 */
#define K3						0x8F1BBCDCL		/* Rounds 40-59: sqrt(5) * 2^30 */
#define K4						0xCA62C1D6L		/* Rounds 60-79: sqrt(10) * 2^30 */

#define I2C_DATA_LINE					0
#define I2C_CLOCK_LINE					1
#define GPCNTL						0x47			/* General Purpose Control	R/W */
#define GPREG						0x07			/* General Purpose Bits	R/W */
/*
************************************************************************
************************************************************************
*/
#define STATUS_FACILITY_SYSTEM			0x0
#define STATUS_FACILITY_STUBS			0x3
#define STATUS_FACILITY_RUNTIME			0x2
#define STATUS_FACILITY_IO_ERROR_CODE		0x4
//
// Define the severity codes
//
#define STATUS_SEVERITY_WARNING			0x2
#define STATUS_SEVERITY_SUCCESS			0x0
#define STATUS_SEVERITY_INFORMATIONAL		0x1
#define STATUS_SEVERITY_ERROR			0x3
//
// MessageId: ARCSAS_EVENT_DEVICE_PLUG
//
// MessageText:
//
// Areca SAS HBA detected a arrival of sas device (%2).
//
#define ARCSAS_EVENT_DEVICE_PLUG				((u32)0x40000001L)

//
// MessageId: ARCSAS_EVENT_DEVICE_UNPLUG
//
// MessageText:
//
// Areca SAS HBA detected a removal of sas device (%2).
//
#define ARCSAS_EVENT_DEVICE_UNPLUG				((u32)0x40000002L)

//
// MessageId: ARCSAS_EVENT_SATA_ERROR_HANDLE_RETRY
//
// MessageText:
//
// Areca SAS HBA sata error handle retrying request (%2).
//
#define ARCSAS_EVENT_SATA_ERROR_HANDLE_RETRY			((u32)0x80000003L)

//
// MessageId: ARCSAS_EVENT_SATA_ERROR_HANDLE_ABORT_REQUST
//
// MessageText:
//
// Areca SAS HBA sata error handle retrying failed, aborting request (%2).
//
#define ARCSAS_EVENT_SATA_ERROR_HANDLE_ABORT_REQUST		((u32)0x80000004L)

//
// MessageId: ARCSAS_EVENT_SATA_ERROR_HANDLE_PORT_RESET
//
// MessageText:
//
// Areca SAS HBA sata error handle retrying failed, resetting port (%2).
//
#define ARCSAS_EVENT_SATA_ERROR_HANDLE_PORT_RESET		((u32)0xC0000005L)

//
// MessageId: ARCSAS_EVENT_EH_SET_DISK_DOWN
//
// MessageText:
//
// Areca SAS HBA sata error handle retrying failed, set disk down (%2).
//
#define ARCSAS_EVENT_EH_SET_DISK_DOWN				((u32)0xC0000006L)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_SET_DISK_DOWN
//
// MessageText:
//
// Areca SAS HBA sas error handle retrying failed, set disk down (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_SET_DISK_DOWN		((u32)0xC0000007L)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_RESET_COMMAND_SLOT
//
// MessageText:
//
// Areca SAS HBA sas error handle previous task management timeout,reset Port (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_RESET_COMMAND_SLOT	((u32)0xC0000008L)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_TMF_ABORT_TASK
//
// MessageText:
//
// Areca SAS HBA sas error handle abort task management (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_TMF_ABORT_TASK		((u32)0x80000009L)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_ABORT_REQUEST
//
// MessageText:
//
// Areca SAS HBA sas error handle abort request (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_ABORT_REQUEST		((u32)0x8000000AL)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_TMF_LOGICAL_UNIT_RESET
//
// MessageText:
//
// Areca SAS HBA sas error handle scsi logical unit device reset (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_TMF_LOGICAL_UNIT_RESET	((u32)0x8000000BL)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_DEVICE_LINK_RESET
//
// MessageText:
//
// Areca SAS HBA sas error handle scsi device reset (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_DEVICE_LINK_RESET		((u32)0x8000000CL)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_DEVICE_HARD_RESET
//
// MessageText:
//
// Areca SAS HBA sas error handle scsi device reset (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_DEVICE_HARD_RESET		((u32)0x8000000DL)

//
// MessageId: ARCSAS_EVENT_SAS_ERROR_HANDLE_PORT_RESET
//
// MessageText:
//
// Areca SAS HBA sas error handle sas port reset (%2).
//
#define ARCSAS_EVENT_SAS_ERROR_HANDLE_PORT_RESET		((u32)0xC000000EL)

//
// MessageId: ARCSAS_EVENT_PM_ERROR_HANDLE_RETRY_REQUEST
//
// MessageText:
//
// Areca SAS HBA port mutiplier error handle retry request (%2).
//
#define ARCSAS_EVENT_PM_ERROR_HANDLE_RETRY_REQUEST		((u32)0x8000000FL)

//
// MessageId: ARCSAS_EVENT_PM_ERROR_HANDLE_ABORT_REQUEST
//
// MessageText:
//
// Areca SAS HBA port mutiplier error handle retrying failed, aborting request (%2).
//
#define ARCSAS_EVENT_PM_ERROR_HANDLE_ABORT_REQUEST		((u32)0x80000010L)

//
// MessageId: ARCSAS_EVENT_CORE_PM_ERROR_HANDLE_PORT_RESET
//
// MessageText:
//
// Areca SAS HBA port mutiplier error handle retrying failed, resetting port (%2).
//
#define ARCSAS_EVENT_CORE_PM_ERROR_HANDLE_PORT_RESET	((u32)0xC0000011L)

//
// MessageId: ARCSAS_EVENT_CORE_DEVICE_CCB_TIMEOUT
//
// MessageText:
//
// Areca SAS HBA core device ccb timeout (%2).
//
#define ARCSAS_EVENT_CORE_DEVICE_CCB_TIMEOUT		((u32)0x80000012L)

//
// MessageId: ARCSAS_EVENT_CORE_PM_CCB_TIMEOUT
//
// MessageText:
//
// Areca SAS HBA core port mutiplier ccb timeout (%2).
//
#define ARCSAS_EVENT_CORE_PM_CCB_TIMEOUT		((u32)0x80000013L)

//
// MessageId: ARCSAS_EVENT_SAS_COMMAND_DONE_CORE_HANDLE_TASK_FILE_ERROR
//
// MessageText:
//
// Areca SAS HBA sas command done core handle task file error (%2).
//
#define ARCSAS_EVENT_SAS_COMMAND_DONE_CORE_HANDLE_TASK_FILE_ERROR	((u32)0x80000014L)

//
// MessageId: ARCSAS_EVENT_PM_HANDLE_DEVICE_UNPLUG
//
// MessageText:
//
// Areca SAS HBA pm handle device unplug but port pPM got null (%2).
//
#define ARCSAS_EVENT_PM_HANDLE_DEVICE_UNPLUG	((u32)0x80000015L)

//
// MessageId: ARCSAS_EVENT_CORE_INIT_ALLOCATE_CACHED_MEMORY_FAILED
//
// MessageText:
//
// Areca SAS HBA core initialize, allocate core resource error.
//
#define ARCSAS_EVENT_CORE_INIT_ALLOCATE_CACHED_MEMORY_FAILED	((u32)0x80000016L)

//
// MessageId: ARCSAS_EVENT_CORE_INIT_ALLOCATE_UNCACHED_MEMORY_FAILED
//
// MessageText:
//
// Areca SAS HBA core initialize, allocate dma resource error.
//
#define ARCSAS_EVENT_CORE_INIT_ALLOCATE_UNCACHED_MEMORY_FAILED	((u32)0x80000017L)

//
// MessageId: ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_READ_PERROR
//
// MessageText:
//
// Areca SAS HBA sata port mutiplier hotplug, hardware register is not correct (%2).
//
#define ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_READ_PERROR	((u32)0x80000018L)

//
// MessageId: ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_PLUG_IN_STATUS
//
// MessageText:
//
// Areca SAS HBA sata port mutiplier hotplug, plug in status is not correct (%2).
//
#define ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_PLUG_IN_STATUS	((u32)0x80000019L)

//
// MessageId: ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_GET_DEVICE_RESOURCE_FAIL
//
// MessageText:
//
// Areca SAS HBA sata port mutiplier hotplug, get null from device pool.
//
#define ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_GET_DEVICE_RESOURCE_FAIL	((u32)0x8000001AL)

//
// MessageId: ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_PLUG_OUT_STATUS
//
// MessageText:
//
// Areca SAS HBA sata port mutiplier hotplug, plug out status is not correct (%2).
//
#define ARCSAS_EVENT_SATA_PM_HOTPLUG_CCB_CALLBACK_PLUG_OUT_STATUS	((u32)0x8000001BL)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_SMART_DETECT_ERROR
//
// MessageText:
//
// Areca SAS HBA sata hard disk smart threshold over (%2).
//
#define ARCSAS_EVENT_DONE_CCB_SMART_DETECT_ERROR		((u32)0x8000001CL)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_DISABLE_WRITE_CACHE
//
// MessageText:
//
// Areca SAS HBA hard disk write cache mode disable (%2).
//
#define ARCSAS_EVENT_DONE_CCB_DISABLE_WRITE_CACHE		((u32)0x4000001DL)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_ENABLE_WRITE_CACHE
//
// MessageText:
//
// Areca SAS HBA hard disk write cache mode enable (%2).
//
#define ARCSAS_EVENT_DONE_CCB_ENABLE_WRITE_CACHE		((u32)0x4000001EL)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_DISABLE_SMART
//
// MessageText:
//
// Areca SAS HBA sata hard disk smart disable (%2).
//
#define ARCSAS_EVENT_DONE_CCB_DISABLE_SMART			((u32)0x4000001FL)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_ENABLE_SMART
//
// MessageText:
//
// Areca SAS HBA sata hard disk smart enable (%2).
//
#define ARCSAS_EVENT_DONE_CCB_ENABLE_SMART			((u32)0x40000020L)

//
// MessageId: ARCSAS_EVENT_DISK_STATUS_MONITOR_DISABLE
//
// MessageText:
//
// Areca SAS HBA sas hard disk status monitor disable (%2).
//
#define ARCSAS_EVENT_DISK_STATUS_MONITOR_DISABLE		((u32)0x40000021L)

//
// MessageId: ARCSAS_EVENT_DISK_STATUS_MONITOR_ENABLE
//
// MessageText:
//
// Areca SAS HBA sas hard disk status monitor enable (%2).
//
#define ARCSAS_EVENT_DISK_STATUS_MONITOR_ENABLE		((u32)0x40000022L)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_SMART_POLLING_ERROR
//
// MessageText:
//
// Areca SAS HBA sata hard disk smart polling fail (%2).
//
#define ARCSAS_EVENT_DONE_CCB_SMART_POLLING_ERROR		((u32)0x80000023L)

//
// MessageId: ARCSAS_EVENT_DONE_CCB_SMART_THRESHOLD_OVER
//
// MessageText:
//
// Areca SAS HBA sata hard disk smart threshold over (%2).
//
#define ARCSAS_EVENT_DONE_CCB_SMART_THRESHOLD_OVER		((u32)0x80000024L)

//
// MessageId: ARCSAS_EVENT_CONSTRUCT_AND_SEND_COMMAND_OUT_OF_PRD_ENTRY
//
// MessageText:
//
// Areca SAS HBA run out of PRD entry, resource not enough.
//
#define ARCSAS_EVENT_CONSTRUCT_AND_SEND_COMMAND_OUT_OF_PRD_ENTRY	((u32)0x80000025L)

//
// MessageId: ARCSAS_EVENT_POST_WAITING_LIST_CCB_UNKNOWN_SEND_COMMAND
//
// MessageText:
//
// Areca SAS HBA unknown send command (%2).
//
#define ARCSAS_EVENT_POST_WAITING_LIST_CCB_UNKNOWN_SEND_COMMAND	((u32)0x80000026L)

//
// MessageId: ARCSAS_EVENT_DRAIN_DONE_QUEUE_TAG_ERROR
//
// MessageText:
//
// Areca SAS HBA drain done queue, tag error need service but tag empty (%2).
//
#define ARCSAS_EVENT_DRAIN_DONE_QUEUE_TAG_ERROR			((u32)0x80000027L)

//
// MessageId: ARCSAS_EVENT_ISR_COMMAND_TABLE_STATUS_ERROR
//
// MessageText:
//
// Areca SAS HBA drain done queue, pCmdTbl Status Buff Error Info occur (%2).
//
#define ARCSAS_EVENT_ISR_COMMAND_TABLE_STATUS_ERROR		((u32)0x80000028L)

//
// MessageId: ARCSAS_EVENT_POST_QUEUE_COMMAND_TABLE_STATUS_ERROR
//
// MessageText:
//
// Areca SAS HBA post queue, pCmdTbl Status Buff Error Info occur (%2).
//
#define ARCSAS_EVENT_POST_QUEUE_COMMAND_TABLE_STATUS_ERROR	((u32)0x80000029L)

//
// MessageId: ARCSAS_EVENT_HBA_DISKS_STATUS_MONITOR_FROM_ON_TO_OFF
//
// MessageText:
//
// Areca SAS HBA Disks Status Monitoring stopped.
//
#define ARCSAS_EVENT_HBA_DISKS_STATUS_MONITOR_FROM_ON_TO_OFF	((u32)0x4000002AL)

//
// MessageId: ARCSAS_EVENT_HBA_DISKS_STATUS_MONITOR_FROM_OFF_TO_ON
//
// MessageText:
//
// Areca SAS HBA Disks Status Monitoring actived.
//
#define ARCSAS_EVENT_HBA_DISKS_STATUS_MONITOR_FROM_OFF_TO_ON	((u32)0x4000002BL)
/*
************************************************************************
************************************************************************
*/
void arcsas_sg_table_init(PARCSAS_SG_Table pSGTable, unsigned short max_entry_count);
void arcsas_sg_table_append(PARCSAS_SG_Table pSGTable, u32 address, u32 addressHigh, u32 size);
void arcsas_interrupt_service_bh(unsigned long pCore);
void arcsas_complete_command_in_doneQ(PCore_Extension pCore, struct CommandControlBlock *pCCB, struct _Domain_Port *pPort);
void arcsas_port_state_machine(PCore_Extension pCore, PDomain_Port pPort);
void arcsas_check_update_flash_device_map(PACB pACB, Bool flag);
void arcsas_port_abort_ccbs(PDomain_Port pPort, unsigned char ScsiStatus, PDomain_Device pDevice);
void arcsas_post_ccb(PCore_Extension pCore, struct CommandControlBlock *pCCB);
void arcsas_discovery_smp_state_machine(PDiscover_Extension pDiscovery, PDomain_Port pPort, unsigned char nTgt, unsigned char CurrentState, unsigned char context, void *pBuffer);
void arcsas_free_coreccb_to_pool(PCore_Extension pCore, struct CommandControlBlock *pCCB);
void arcsas_free_core_context_to_pool(PCore_Extension pCore, PCore_Context *pContext);
void arcsas_release_pool_tag(PTag_Stack pTagStack, unsigned short tag);
PCore_Context arcsas_core_get_context_from_pool(PCore_Extension pCore);
void arcsas_post_waiting_list_ccb(PCore_Extension pCore, PDomain_Device pDevice);
void arcsas_remove_device(PDomain_Device pDevice);
struct CommandControlBlock *arcsas_core_get_coreccb_from_list(PCore_Extension pCore);
Bool  arcsas_reset_controller(struct AdapterControlBlock *pACB);
#if ARCSAS_SUPPORT_SGPIO
	Bool arcsas_core_sgpio_set_led(PDomain_Device pDevice, unsigned char flag);
#endif


